@@ -1,17 +1,10 @@
-(require 'mew-lang-jp)
-(require 'mew-lang-latin)
-
 (defvar FLD-MSG-LIST nil)
-(defvar TOTAL nil)
-(defvar WIDTH nil)
 (defvar ange-ftp-generate-anonymous-password nil)
 (defvar ange-ftp-tmp-name-template nil)
 (defvar auto-image-file-mode nil)
 (defvar coding-category-list nil)
 (defvar current-menubar nil)
 (defvar default-file-name-coding-system nil)
-(defvar default-toolbar nil)
-(defvar default-toolbar-visible-p nil)
 (defvar dnd-protocol-alist nil)
 (defvar elp-function-list nil)
 (defvar elp-results-buffer nil)
@@ -24,7 +17,6 @@
 (defvar mark-active nil)
 (defvar max-mini-window-height nil)
 (defvar mew-draft-toolbar nil)
-(defvar mew-icon-separate-spec nil)
 (defvar mew-message-toolbar nil)
 (defvar mew-summary-toolbar nil)
 (defvar mode-line-position nil)
@@ -40,7 +32,6 @@
 (defvar w32-get-true-file-link-count nil)
 (defvar w32-start-process-show-window nil)
 (defvar w3m-content-type-alist nil)
-(defvar w3m-namazu-index-alist nil)
 (defvar write-region-inhibit-fsync nil)
 (defvar x-dnd-test-function nil)
 
@@ -53,6 +44,7 @@
 (defun bitmap-insert-xbm-file (&rest args))
 (defun create-image (&rest args))
 (defun define-process-argument-editing (&rest args))
+(defun delete-overlay (&rest args))
 (defun display-graphic-p (&rest args))
 (defun dnd-get-local-file-name (&rest args))
 (defun dnd-get-local-file-uri (&rest args))
@@ -68,12 +60,19 @@
 (defun jit-lock-register (&rest args))
 (defun mac-set-file-type (&rest args))
 (defun make-network-process (&rest args))
+(defun make-overlay (&rest args))
+(defun mew-hscroll (&rest args))
 (defun mouse-set-point (&rest args))
+(defun move-overlay (&rest args))
 (defun multibyte-string-p (&rest args))
 (defun mw32script-init (&rest args))
 (defun mw32script-make-pathext-regexp (&rest args))
 (defun mw32script-openp (&rest args))
 (defun mw32script-resolve-script (&rest args))
+(defun overlay-get (&rest args))
+(defun overlay-put (&rest args))
+(defun overlayp (&rest args))
+(defun overlays-in (&rest args))
 (defun popup-menu (&rest args))
 (defun posn-window (&rest args))
 (defun profile-functions (&rest args))
@@ -82,6 +81,7 @@
 (defun set-buffer-menubar (&rest args))
 (defun set-buffer-multibyte (&rest args))
 (defun set-coding-priority (&rest args))
+(defun set-process-query-on-exit-flag (&rest args))
 (defun set-toolbar-button-down-flag (&rest args))
 (defun string-as-multibyte (&rest args))
 (defun string-as-unibyte (&rest args))
@@ -94,8 +94,11 @@
 (defun update-coding-systems-internal (&rest args))
 (defun w32-focus-frame (&rest args))
 (defun w3m-expand-file-name-as-url (&rest args))
-(defun w3m-namazu (&rest args))
 (defun w3m-region (&rest args))
 (defun w3m-url-local-p (&rest args))
 (defun w3m-url-to-file-name (&rest args))
+(defun window-edges (&rest args))
 (defun x-dnd-choose-type (&rest args))
+
+(require 'mew-lang-jp)
+(require 'mew-lang-latin)
