@@ -7,7 +7,7 @@
 
 #include "mew.h"
 
-private char version_message[] = "version 4.1 20030217 Yasunari Momoi";
+private char version_message[] = "version 5.3 20060727 Yasunari Momoi";
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -48,17 +48,17 @@ private char version_message[] = "version 4.1 20030217 Yasunari Momoi";
 #endif
 
 enum MBOXTYPE {
-    T_UNKNOWN,
-    T_MAILDIR,
-    T_MBOX,
-    T_STDIN,
+	T_UNKNOWN,
+	T_MAILDIR,
+	T_MBOX,
+	T_STDIN,
 };
 
 enum MBOXSTATE {
-    ST_UNKNOWN,
-    ST_HEADER,
-    ST_BODY_AFTER_EMPTY_LINE,
-    ST_BODY,
+	ST_UNKNOWN,
+	ST_HEADER,
+	ST_BODY_AFTER_EMPTY_LINE,
+	ST_BODY,
 };
 
 #define FBUFSIZ		(BUFSIZ * 32)
@@ -67,14 +67,19 @@ enum MBOXSTATE {
 #endif
 
 private char	FileBuf[FBUFSIZ];
-private char	InboxDir[PATH_MAX];
 private char	Mbox[PATH_MAX];
 private char	MboxLock[PATH_MAX];
+private char	*InboxDir = NULL;
+private char	*Suffix = NULL;
+private int	Use_Suffix = FALSE;
+private int	Suffix_len;
 private int	MboxType;
 private int	Backup;
 private int	GetCur;
 private int	UseCL;
+private int	PreserveUnixFrom;
 private int	CreateMTime = TRUE;
+private int	FileMode = S_IRUSR | S_IWUSR;
 private int	Exit = 0;
 
 /****************************************************************
@@ -88,10 +93,10 @@ private void	usage(const char *);
 private void	help(const char *);
 private void	version(const char *);
 private void	init_env(int, char **);
-private int	is_number(unsigned char *);
+private int	get_number(char *);
 private int	get_last_seq(void);
 private int	compare_string(char **, char **);
-private void	copyfile(char *, char *);
+private int	copyfile(char *, char *);
 private void	movefile(char *, char *, char *, int);
 private int	maildir_names(const char *, char **, char **, char **);
 private int	new_inbox_file(int, char[]);
@@ -162,18 +167,6 @@ Gethomedir(void)
 	return NULL;
 }
 
-#if !HAVE_STRDUP
-private char *
-strdup(const char *str)
-{
-	char* t;
-	MALLOC(t, strlen(str) + 1);
-	if (t)
-		strcpy(t, str);
-	return t;
-}
-#endif
-
 private void
 error(const char *fmt, ...)
 {
@@ -205,7 +198,7 @@ error(const char *fmt, ...)
 
 private void
 usage(const char *progname) {
-	fprintf(stderr, "Usage: %s [-abchsv] [-d maildir] [-i inboxdir]\n", progname);
+	fprintf(stderr, "Usage: %s [-abchosv] [-d maildir] [-i inboxdir] [-x suffix]\n", progname);
 }
 
 private const char *
@@ -223,6 +216,10 @@ help_message[] = {
 	"                  (no backup) (for maildir)",
 	"    -c            Use Content-Length: field. (for mbox)",
 	"    -u            Don't create inboxdir/.mew-mtime file.",
+	"    -f            Preserve Unix From (Envelope Sender). (for mbox)",
+	"    -p <fmode>    Specify file permission. (for mbox)",
+	"    -o            Use the suffix when creating messages.",
+	"    -x <suffix>   Use this suffix.",
 	NULL
 };
 
@@ -289,14 +286,14 @@ search_mbox_path(void)
 	if ((home = Gethomedir()) != NULL) {
 		if (strlen(home) + 9 > PATH_MAX)
 			error("pathname too long (%s)", home);
-		sprintf(Mbox, "%s/%s", home, MAILDIR);
+		snprintf(Mbox, sizeof(Mbox), "%s/%s", home, MAILDIR);
 		if (check_mbox_type(Mbox) != T_UNKNOWN)
 			return;
 	}
 	if ((mail = getenv("MAIL")) != NULL) {
 		if (strlen(mail) + 1 > PATH_MAX)
 			error("pathname too long (%s)", mail);
-		sprintf(Mbox, mail);
+		STRCPY(Mbox, mail);
 		if (check_mbox_type(Mbox) != T_UNKNOWN)
 			return;
 	}
@@ -306,12 +303,12 @@ search_mbox_path(void)
 			if (strlen(mbox_path_list[i]) + strlen(user) + 1
 			    > PATH_MAX)
 				error("pathname too long (%s)", user);
-			sprintf(Mbox, "%s%s", mbox_path_list[i], user);
+			snprintf(Mbox, sizeof(Mbox), "%s%s", mbox_path_list[i], user);
 			if (check_mbox_type(Mbox) != T_UNKNOWN)
 				return;
 		}
 	}
-	sprintf(Mbox, ".");
+	snprintf(Mbox, sizeof(Mbox), ".");
 	return;
 }
 
@@ -344,21 +341,34 @@ private void
 init_env(int argc, char **argv)
 {
 	set_sighandler();
-	sprintf(InboxDir, ".");
+	STRDUP(InboxDir, ".");
 	MboxType = T_UNKNOWN;
 	Backup = FALSE;
 	UseCL = FALSE;
 	search_mbox_path();
+	STRDUP(Suffix, SUFFIX);
+	Suffix_len = strlen(Suffix);
 }
 
 private int
-is_number(unsigned char *str)
+get_number(char *s)
 {
-	do {
-		if (!isdigit(*str))
-			return FALSE;
-	} while (*++str != '\0');
-	return TRUE;
+	int num = 0;
+	unsigned char c;
+
+	while ((c = *s) != NUL && isdigit(c)) {
+		num = num * 10 + c - '0';
+		s++;
+	}
+
+	if (num == 0)
+		return 0;
+	else if (*s == NUL)
+		return num;
+	else if (strncmp(s, Suffix, Suffix_len) == 0 && s[Suffix_len] == NUL)
+		return num;
+	else
+		return 0;
 }
 
 private int
@@ -372,9 +382,8 @@ get_last_seq(void)
 	if ((dirp = opendir(InboxDir)) == NULL)
 		error("opendir(%s)", InboxDir);
 	while ((dp = readdir(dirp)) != NULL) {
-		if (!is_number(dp->d_name))
+		if ((seq = get_number(dp->d_name)) == 0)
 			continue;
-		seq = atoi(dp->d_name);
 		last = last > seq ? last : seq;
 	}
 	closedir(dirp);
@@ -387,7 +396,7 @@ compare_string(char **i, char **j)
 	return strcmp(*i, *j);
 }
 
-private void
+private int
 copyfile(char *src, char *dst)
 {
 	struct timeval tv[2];
@@ -403,14 +412,16 @@ copyfile(char *src, char *dst)
 		error("open(%s) for write", dst);
 	while ((rlen = read(srcfd, FileBuf, FBUFSIZ)) > 0) {
 		if ((wlen = write(dstfd, FileBuf, rlen)) != rlen) {
-			close(dstfd);
+			if (close(dstfd))
+				goto werr;
 			unlink(dst);
 			error("write(%s) (read %d bytes/write %d bytes)",
 			      dst, rlen, wlen);
 		}
 	}
 	if (rlen < 0) {
-		close(dstfd);
+		if (close(dstfd))
+			goto werr;
 		unlink(dst);
 		error("read(%s)", src);
 	}
@@ -437,24 +448,32 @@ copyfile(char *src, char *dst)
 	if (chmod(dst, sb.st_mode))
 		warning("chmod(%s) failed", dst);
 #endif
+	return 0;
+
+ werr:
+	close(srcfd);
+	return -1;
 }
 
 private void
 movefile(char *fromfile, char *tofile, char *backupfile, int backup)
 {
 	if (backup && backupfile != NULL) {
-		copyfile(fromfile, tofile);
+		if (copyfile(fromfile, tofile))
+			error("copyfile(%s, %s)", fromfile, tofile);
 		if (rename(fromfile, backupfile))
 			error("rename(%s, %s)", fromfile, backupfile);
 	}
 	else if (backup) {
-		copyfile(fromfile, tofile);
+		if (copyfile(fromfile, tofile))
+			error("copyfile(%s, %s)", fromfile, tofile);
 	}
 	else {
 		if (rename(fromfile, tofile)) {
 			if (errno != EXDEV)
 				error("rename(%s, %s)", fromfile, tofile);
-			copyfile(fromfile, tofile);
+			if (copyfile(fromfile, tofile))
+				error("copyfile(%s, %s)", fromfile, tofile);
 			if (unlink(fromfile))
 				error("unlink(%s)", fromfile);
 		}
@@ -471,15 +490,15 @@ maildir_names(const char *maildir, char **newdir, char **curdir, char **tmpdir)
 		return -1;
 	if (newdir != NULL) {
 		MALLOC(*newdir, len);
-		sprintf(*newdir, "%s/%s", maildir, MAILDIR_NEW);
+		snprintf(*newdir, len, "%s/%s", maildir, MAILDIR_NEW);
 	}
 	if (curdir != NULL) {
 		MALLOC(*curdir, len);
-		sprintf(*curdir, "%s/%s", maildir, MAILDIR_CUR);
+		snprintf(*curdir, len, "%s/%s", maildir, MAILDIR_CUR);
 	}
 	if (tmpdir != NULL) {
 		MALLOC(*tmpdir, len);
-		sprintf(*tmpdir, "%s/%s", maildir, MAILDIR_TMP);
+		snprintf(*tmpdir, len, "%s/%s", maildir, MAILDIR_TMP);
 	}
 	return 0;
 }
@@ -489,11 +508,13 @@ private int
 new_inbox_file(int seq, char inboxfile[])
 {
 	char num[PATH_MAX];
+	char *suffix = (Use_Suffix == YES) ? Suffix : "";
+
 	do {
-		sprintf(num, "%d", ++seq);
+		snprintf(num, sizeof(num), "%d%s", ++seq, suffix);
 		if (strlen(InboxDir) + strlen(num) + 2 > PATH_MAX)
 			error("pathname too long (%s/%s)", InboxDir, num);
-		sprintf(inboxfile, "%s/%s", InboxDir, num);
+		snprintf(inboxfile, PATH_MAX, "%s/%s", InboxDir, num);
 		if (access(inboxfile, F_OK) && errno == ENOENT)
 			break;
 	} while (TRUE);
@@ -506,17 +527,16 @@ open_new_inbox_file(int *seq, char inboxfile[])
 {
 	char num[PATH_MAX];
 	int flag = O_EXCL | O_CREAT | O_WRONLY;
-	int mode = S_IRUSR | S_IRGRP | S_IROTH |
-		   S_IWUSR | S_IWGRP | S_IWOTH;
 	int fd;
 	FILE *fp = NULL;
+	char *suffix = (Use_Suffix == YES) ? Suffix : "";
 
 	for (;;) {
-		sprintf(num, "%d", ++*seq);
+		snprintf(num, sizeof(num), "%d%s", ++*seq, suffix);
 		if (strlen(InboxDir) + strlen(num) + 2 > PATH_MAX)
 			error("pathname too long (%s/%s)", InboxDir, num);
-		sprintf(inboxfile, "%s/%s", InboxDir, num);
-		if ((fd = open(inboxfile, flag, mode)) >= 0 ||
+		snprintf(inboxfile, PATH_MAX, "%s/%s", InboxDir, num);
+		if ((fd = open(inboxfile, flag, FileMode)) >= 0 ||
 		    errno != EEXIST)
 			break;
 		usleep(rand() % 199);
@@ -551,7 +571,7 @@ get_from_dir(int seq, char *fromdir, char *backupdir, int backup)
 		if (strlen(fromdir) + strlen(dp->d_name) + 2 > PATH_MAX)
 			error("pathname too long (%s/%s)",
 			      fromdir, dp->d_name);
-		sprintf(mailfile, "%s/%s", fromdir, dp->d_name);
+		snprintf(mailfile, sizeof(mailfile), "%s/%s", fromdir, dp->d_name);
 		if (stat(mailfile, &sb))
 			continue;
 		if (!(S_ISREG(sb.st_mode) && (sb.st_mode & S_IRUSR)))
@@ -562,8 +582,7 @@ get_from_dir(int seq, char *fromdir, char *backupdir, int backup)
 			     realloc(list, sizeof(char *)*listsize)) == NULL)
 				error("realloc");
 		}
-		if ((list[listend++] = strdup(dp->d_name)) == NULL)
-			error("strdup(%s)", dp->d_name);
+		STRDUP(list[listend++], dp->d_name);
 	}
 	closedir(dirp);
 
@@ -575,12 +594,12 @@ get_from_dir(int seq, char *fromdir, char *backupdir, int backup)
 		if (strlen(fromdir) + strlen(list[i]) + 2 > PATH_MAX)
 			error("pathname too long (%s/%s)",
 			      fromdir, list[i]);
-		sprintf(mailfile, "%s/%s", fromdir, list[i]);
+		snprintf(mailfile, sizeof(mailfile), "%s/%s", fromdir, list[i]);
 		if (backup && backupdir != NULL) {
 			if (strlen(backupdir) + strlen(list[i]) + 6 > PATH_MAX)
 				error("pathname too long (%s/%s)",
 				      backupdir, list[i]);
-			sprintf(backupfile, "%s/%s:2,S", backupdir, list[i]);
+			snprintf(backupfile, sizeof(backupfile), "%s/%s:2,S", backupdir, list[i]);
 			movefile(mailfile, inboxfile, backupfile, backup);
 		}
 		else
@@ -647,7 +666,7 @@ process_mbox(int seq)
 
 	if (strlen(Mbox) + strlen(LOCK_SUFFIX) + 1 > PATH_MAX)
 		error("pathname too long (%s%s)", Mbox, LOCK_SUFFIX);
-	sprintf(MboxLock, "%s%s", Mbox, LOCK_SUFFIX);
+	snprintf(MboxLock, sizeof(MboxLock), "%s%s", Mbox, LOCK_SUFFIX);
 	if (lock_mbox(MboxLock))
 		MboxLock[0] = '\0'; /* doesn't need a lockfile, maybe. */
 
@@ -667,7 +686,7 @@ process_mbox(int seq)
 		warning("fdopen(%s) for read", Mbox);  goto rerr;
 	}
 
-	while ((ln = getline(srcfp)) != NULL) {
+	while ((ln = Getline(srcfp)) != NULL) {
 		if (Exit)
 			goto werr;
 		switch (state) {
@@ -676,13 +695,18 @@ process_mbox(int seq)
 				dstfp = open_new_inbox_file(&seq, inboxfile);
 				if (dstfp == NULL)
 					goto rerr;
+				if (PreserveUnixFrom &&
+				    fputs(ln, dstfp) == EOF) {
+					warning("fputs(%s)", inboxfile);
+					goto werr;
+				}
 				state = ST_HEADER;
 			}
 			break;
 		case ST_HEADER:
 			if (strlen(ln) < 3 &&
 			    (ln[0] == '\n' || ln[0] == '\r')) {
-				strcpy(emptyline, ln);
+				STRCPY(emptyline, ln);
 				state = ST_BODY_AFTER_EMPTY_LINE;
 				break;
 			}
@@ -700,12 +724,18 @@ process_mbox(int seq)
 			break;
 		case ST_BODY_AFTER_EMPTY_LINE:
 			if (bytes < 0 && strncmp(ln, "From ", 5) == 0) {
-				fclose(dstfp);
+				if (fclose(dstfp))
+					goto rerr;
 				printf("%d\n", seq);
 
 				dstfp = open_new_inbox_file(&seq, inboxfile);
 				if (dstfp == NULL)
 					goto rerr;
+				if (PreserveUnixFrom &&
+				    fputs(ln, dstfp) == EOF) {
+					warning("fputs(%s)", inboxfile);
+					goto werr;
+				}
 				state = ST_HEADER;
 				break;
 			}
@@ -715,7 +745,7 @@ process_mbox(int seq)
 		case ST_BODY:
 			if (strlen(ln) < 3 &&
 			    (ln[0] == '\n' || ln[0] == '\r')) {
-				strcpy(emptyline, ln);
+				STRCPY(emptyline, ln);
 				state = ST_BODY_AFTER_EMPTY_LINE;
 			}
 			else
@@ -726,7 +756,9 @@ process_mbox(int seq)
 			if (bytes >= 0) {
 				bytes -= strlen(ln);
 				if (bytes <= 0) {
-					fclose(dstfp);  dstfp = NULL;
+					if (fclose(dstfp))
+						goto rerr;
+					dstfp = NULL;
 					printf("%d\n", seq);
 					state = ST_UNKNOWN;
 					bytes = -1;
@@ -738,7 +770,8 @@ process_mbox(int seq)
 		free(ln);
 	}
 	if (dstfp) {
-		fclose(dstfp);
+		if (fclose(dstfp))
+			goto rerr;
 		printf("%d\n", seq);
 	}
 	if (!Backup && ftruncate(srcfd, 0)) {
@@ -772,7 +805,7 @@ process_stdin(int seq)
 	if ((dstfp = open_new_inbox_file(&seq, inboxfile)) == NULL)
 		goto rerr;
 
-	while ((ln = getline(srcfp)) != NULL) {
+	while ((ln = Getline(srcfp)) != NULL) {
 		if (Exit)
 			goto werr;
 		if (fputs(ln, dstfp) == EOF) {
@@ -825,7 +858,7 @@ process(void)
 		return;		/* no new mail */
 	if (strlen(InboxDir) + strlen(MEW_MTIME_FILE) + 1 > PATH_MAX)
 		error("pathname too long (%s%s)", InboxDir, MEW_MTIME_FILE);
-	sprintf(mtimefile, "%s/%s", InboxDir, MEW_MTIME_FILE);
+	snprintf(mtimefile, sizeof(mtimefile), "%s/%s", InboxDir, MEW_MTIME_FILE);
 
 	if ((fp = fopen(mtimefile, FDWRITE)) == NULL)
 		error("can't create file (%s)", mtimefile);
@@ -864,7 +897,7 @@ main(int argc, char **argv)
 	warn_prog = progname;
 	init_env(argc, argv);
 
-	while ((ch = Getopt(argc, argv, "abcd:hi:m:suvp")) != EOF) {
+	while ((ch = Getopt(argc, argv, "abcd:fhi:m:op:suvx:")) != EOF) {
 		switch (ch) {
 		case 'a':
 			GetCur = TRUE;
@@ -879,17 +912,24 @@ main(int argc, char **argv)
 		case 'm':
 			if (strlen(Optarg) + 1 > PATH_MAX)
 				error("pathname too long (%s)", Optarg);
-			sprintf(Mbox, "%s", Optarg);
+			snprintf(Mbox, sizeof(Mbox), "%s", Optarg);
+			break;
+		case 'f':
+			PreserveUnixFrom = TRUE;
 			break;
 		case 'i':
-			if (strlen(Optarg) + 1 > PATH_MAX)
-				error("pathname too long (%s)", Optarg);
-			sprintf(InboxDir, "%s", Optarg);
+			STRDUP(InboxDir, Optarg);
+			break;
+		case 'o':
+			Use_Suffix = TRUE;
+			break;
+		case 'x':
+			STRDUP(Suffix, Optarg);
+			Suffix_len = strlen(Suffix);
+			break;
+		case 'p':
+			sscanf(Optarg, "%i", &FileMode);
 			break;
-		case 'p':	/* for debug */
-			printf("InboxDir: %s\n", InboxDir);
-			printf("Mbox: %s\n", Mbox);
-			exit(EXIT_SUCCESS);
 		case 's':
 			MboxType = T_STDIN;
 			break;
@@ -916,7 +956,7 @@ main(int argc, char **argv)
 }
 
 /* 
- * Copyright (C) 2001-2003 Mew developing team.
+ * Copyright (C) 2001-2005 Mew developing team.
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
