@@ -11,7 +11,7 @@
   '(("B" . mew-base64-encode-string)
     ("Q" . mew-q-encode-string)))
 
-(defsubst mew-header-encode-get-func (b-or-q)
+(defun mew-header-encode-get-func (b-or-q)
   (cdr (mew-assoc-case-equal b-or-q mew-header-encode-switch 0)))
 
 
@@ -25,13 +25,13 @@
      nil
      ,mew-error-invalid-q-encoding)))
 
-(defsubst mew-header-decode-get-func (b-or-q)
+(defun mew-header-decode-get-func (b-or-q)
   (nth 1 (mew-assoc-case-equal b-or-q mew-header-decode-switch 0)))
 
-(defsubst mew-header-decode-get-error-func (b-or-q)
+(defun mew-header-decode-get-error-func (b-or-q)
   (nth 2 (mew-assoc-case-equal b-or-q mew-header-decode-switch 0)))
 
-(defsubst mew-header-decode-get-error-str (b-or-q)
+(defun mew-header-decode-get-error-str (b-or-q)
   (nth 3 (mew-assoc-case-equal b-or-q mew-header-decode-switch 0)))
 
 (defconst mew-header-decode-regex
@@ -143,9 +143,9 @@
 
 (defun mew-q-encode-string (str &optional prefix)
   (let* ((len (length str))
-	 (ret (make-string (* len 3) ?a))
-	 (i 0) (j 0) char)
-    (while (< i len)
+	 (ret (mew-make-string (* len 3)))
+	 (j 0) char)
+    (dotimes (i len)
       (setq char (aref str i))
       (cond
        ((char-equal char mew-sp)
@@ -163,20 +163,20 @@
 	(aset ret j (aref "0123456789ABCDEF" (lsh char -4)))
 	(setq j (1+ j))
 	(aset ret j (aref "0123456789ABCDEF" (logand char 15)))))
-      (setq i (1+ i) j (1+ j)))
+      (setq j (1+ j)))
     (substring ret 0 j)))
 
 (defun mew-samba-encoding (str)
   (setq str (mew-cs-encode-string str mew-cs-samba))
   (let* ((len (length str))
-	 (ret (make-string (* len 3) ?a))
-	 (i 0) (j 0) char type)
+	 (ret (mew-make-string (* len 3)))
+	 (j 0) char type)
     (cond
      ((eq mew-use-samba-encoding-type 'cap)
       (setq type 'cap))
      (t
       (setq type 'hex)))
-    (while (< i len)
+    (dotimes (i len)
       (setq char (aref str i))
       (if (and (eq type 'cap) (< char 128))
 	  (aset ret j char)
@@ -185,7 +185,7 @@
 	(aset ret j (aref "0123456789abcdef" (lsh char -4)))
 	(setq j (1+ j))
 	(aset ret j (aref "0123456789abcdef" (logand char 15))))
-      (setq i (1+ i) j (1+ j)))
+      (setq j (1+ j)))
     (substring ret 0 j)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -193,7 +193,7 @@
 ;;; Quoted-printable decoding
 ;;;
 
-(defsubst mew-hexchar-to-int (hex)
+(defun mew-hexchar-to-int (hex)
   ;; would return nil if error
   (cond
    ((and (<= ?0 hex) (<= hex ?9)) (- hex ?0))
@@ -203,10 +203,10 @@
 (defun mew-q-decode-string (qpstr &optional okey)
   (condition-case nil
       (let* ((len (length qpstr))
-	     (ret (make-string len ?a))
-	     (i 0) (j 0) char key)
+	     (ret (mew-make-string len))
+	     (j 0) char key)
 	(setq key (or okey ?=))
-	(while (< i len)
+	(dotimes (i len)
 	  (setq char (aref qpstr i))
 	  (cond
 	   ((and (not okey) (char-equal char ?_))
@@ -217,7 +217,7 @@
 	    (setq i (+ i 2)))
 	   (t
 	    (aset ret j char)))
-	  (setq i (1+ i) j (1+ j)))
+	  (setq j (1+ j)))
 	(substring ret 0 j))
     (error nil)))
 
@@ -255,7 +255,7 @@
 	(mew-encode-error
 	 (format "Unknown coding system %s in the header" (symbol-name hcs)))
       (setq charset (mew-cs-to-charset hcs))
-      (setq b-or-q (mew-ecsdb-b-or-q ecsdb))
+      (setq b-or-q (mew-ecsdb-get-b-or-q ecsdb))
       (setq fun (mew-header-encode-get-func b-or-q))
       (if (null fun)
 	  (mew-encode-error
@@ -295,10 +295,9 @@
   (let ((str-list (mapcar 'mew-chop (mew-split str ?,))))
     (mew-header-encode-text (car str-list))
     (setq str-list (cdr str-list))
-    (while str-list
+    (dolist (sl str-list)
       (insert ", ") ;; must be fold here
-      (mew-header-encode-text (car str-list))
-      (setq str-list (cdr str-list)))))
+      (mew-header-encode-text sl))))
 
 (defmacro mew-header-encode-cond (c)
   `(cond
@@ -613,9 +612,9 @@
     ;; If each line does not end with ",", unfold it.
     ;; In Page 5 of RFC822 says, "Unfolding is accomplished by
     ;; regarding CRLF immediately followed by a LWSP-char as
-    ;; equivalent to the LWSP-char". However, it also says, 
-    ;; "In structured field bodies, multiple linear space ASCII 
-    ;; characters (namely HTABs and SPACEs) are treated as single 
+    ;; equivalent to the LWSP-char". However, it also says,
+    ;; "In structured field bodies, multiple linear space ASCII
+    ;; characters (namely HTABs and SPACEs) are treated as single
     ;; spaces and may freely surround any symbol." So, remove
     ;; continuous white spaces.
     (when (and mew-decode-broken (looking-at "\n[ \t]+"))
@@ -635,8 +634,8 @@
    (t ;; unstruct
     )))
 
-(defsubst mew-header-decode-string (key)
-  (let ((beg (match-beginning 0)) 
+(defun mew-header-decode-string (key)
+  (let ((beg (match-beginning 0))
 	(end (match-end 0))
 	(css (mew-header-decode (mew-match-string 1)
 				(mew-match-string 2)
@@ -646,17 +645,17 @@
     (delete-region beg end)
     (insert css)))
 
-(defsubst mew-header-decode-in-quotedp ()
+(defun mew-header-decode-in-quotedp ()
   (save-excursion
     (beginning-of-line)
     (char-equal (following-char) ?\")))
 
-(defsubst mew-header-decode-in-addr-spec ()
+(defun mew-header-decode-in-addr-spec ()
   (eq (get-text-property (match-beginning 0) 'mew-addr-spec) t))
 
 (defun mew-decode-warning-body (err msg)
   (mew-xinfo-set-warning
-   (cons 
+   (cons
     (cond
      ((eq err 'unknown-charset)
       (format "Charset (%s) for body is not supported.\n" msg))
@@ -725,9 +724,9 @@ That is, each line may be more than 75."
       ;;
       (goto-char (point-min))
       (when (re-search-forward mew-header-decode-regex nil t)
-	;; In Page 10 of RFC 2047 says, "When displaying a particular 
-	;; header field that contains multiple 'encoded-word's, any 
-	;; 'linear-white-space' that separates a pair of adjacent 
+	;; In Page 10 of RFC 2047 says, "When displaying a particular
+	;; header field that contains multiple 'encoded-word's, any
+	;; 'linear-white-space' that separates a pair of adjacent
 	;; 'encoded-word's is ignored". So, use looking-at.
 	(cond
 	 ((memq type '(text comma-text))
@@ -746,7 +745,7 @@ That is, each line may be more than 75."
 	  ;; encoded-word in quoted-string should not be decoded
 	  ;; according to RFC 2047. However, if users wish
 	  ;; (ie mew-decode-broken is *non-nil*), decode it.
-	  ;; Regular expression cannot express a quoted string in 
+	  ;; Regular expression cannot express a quoted string in
 	  ;; general case. So, canonicalize as follows:
 	  ;;     Not quoted line (may be null line)
           ;;     "quoted line"
@@ -828,9 +827,9 @@ That is, each line may be more than 75."
 	 (charset (mew-cs-to-charset hcs))
          (estr (mew-cs-encode-string str hcs))
 	 (len (length estr))
-	 (ret (make-string (* len 3) ?a))
-	 (i 0) (j 0) char)
-    (while (< i len)
+	 (ret (mew-make-string (* len 3)))
+	 (j 0) char)
+    (dotimes (i len)
       (setq char (aref estr i))
       (if (or (and (<= ?0 char) (<= char ?9))
 	      (and (<= ?a char) (<= char ?z))
@@ -841,7 +840,7 @@ That is, each line may be more than 75."
 	(aset ret j (aref "0123456789ABCDEF" (lsh char -4)))
 	(setq j (1+ j))
 	(aset ret j (aref "0123456789ABCDEF" (logand char 15))))
-      (setq i (1+ i) j (1+ j)))
+      (setq j (1+ j)))
     (concat charset "''" (substring ret 0 j))))
 
 (defun mew-param-decode (whole-value)
@@ -849,23 +848,21 @@ That is, each line may be more than 75."
 	 (value (car value-params))
 	 (params (cdr value-params))
 	 (max 0) num
-	 ret ext ea ext-sort entry charset cs
+	 ret ext ext-sort entry charset cs
 	 paramname-value paramname paramvalue)
-    (while params
-      (setq paramname-value (mew-param-analyze (car params)))
+    (dolist (param params)
+      (setq paramname-value (mew-param-analyze param))
       (when paramname-value
 	(setcar paramname-value (downcase (nth 0 paramname-value)))
 	(if (= (length paramname-value) 2)
 	    (setq ret (cons paramname-value ret))
 	  (setq ext (cons paramname-value ext))
 	  (setq num (nth 2 paramname-value))
-	  (if (> num max) (setq max num))))
-      (setq params (cdr params)))
+	  (if (> num max) (setq max num)))))
     (if (null ext)
 	(cons value (nreverse ret)) ;; fast return
       (setq max (1+ max))
-      (while ext
-	(setq ea (car ext))
+      (dolist (ea ext)
 	(mew-set '(paramname paramvalue num) ea)
 	(if (setq entry (assoc paramname ext-sort))
 	    (progn
@@ -877,10 +874,8 @@ That is, each line may be more than 75."
 	  (aset entry num paramvalue)
 	  (setq ext-sort
 		(cons (list paramname entry (nth 3 ea) (nth 4 ea))
-		      ext-sort)))
-	(setq ext (cdr ext)))
-      (while ext-sort
-	(setq ea (car ext-sort))
+		      ext-sort))))
+      (dolist (ea ext-sort)
 	(setq paramvalue nil)
 	(mew-set '(paramname entry charset) ea)
 	(setq num 0)
@@ -905,15 +900,14 @@ That is, each line may be more than 75."
 	  (if (mew-coding-system-p cs)
 	      (setq paramvalue (mew-cs-decode-string paramvalue cs))
 	    (setq paramvalue mew-error-unknown-charset)))
-	(setq ret (cons (list paramname paramvalue) ret))
-	(setq ext-sort (cdr ext-sort)))
+	(setq ret (cons (list paramname paramvalue) ret)))
       (setq ret (mapcar 'mew-param-sanity-check ret))
       (cons value (nreverse ret))))) ;; late return
 
 (defun mew-param-analyze (param)
-  "Return (paramname paramvalue) or 
+  "Return (paramname paramvalue) or
           (paramname paramvalue section charset lang)."
-  (let* (name section asterisk value charset lang)
+  (let (name section asterisk value charset lang)
     (when (string-match mew-header-param-regex param)
       (setq name (match-string 1 param))
       (setq section (match-string 2 param)) ;; *21
@@ -934,12 +928,16 @@ That is, each line may be more than 75."
 	    (if mew-decode-broken
 		(setq value (mew-param-analyze-broken value name))))
 	(when (and (= section 0)
-		   (string-match "^\\([^']*\\)'\\([^']*\\)'\\(.*\\)$" value))
+		   ;; broken UAs quote extended-value, sigh
+		   (string-match "^\"?\\([^']*\\)'\\([^']*\\)'\\([^\"]*\\)\"?$" value))
 	  (setq charset (match-string 1 value))
 	  (setq lang (match-string 2 value))
 	  (if (string= lang "") (setq lang nil))
 	  (setq value (match-string 3 value))
 	  (if (string= value "") (setq value nil)))
+	;; broken UAs quote extended-value, sigh
+	(if (string-match "^\"\\([^\"]*\\)\"$" value)
+	    (setq value (mew-match-string 1 value)))
 	(setq value (mew-q-decode-string value ?%)))
       (if section
 	  (list name value section charset lang)
@@ -1026,16 +1024,13 @@ That is, each line may be more than 75."
 	 (bias mew-puny-initial-bias)
 	 (len (length input))
 	 (in 0) (out 0)
-	 (i 0) (j 0) (b 0)
+	 (i 0) (b 0)
 	 digit thr oldi w k output ret)
-    (while (< j len)
-      (if (= (aref input j) mew-puny-delimiter) (setq b j))
-      (setq j (1+ j)))
-    (setq j 0)
-    (while (< j b)
+    (dotimes (j len)
+      (if (= (aref input j) mew-puny-delimiter) (setq b j)))
+    (dotimes (j b)
       (setq output (cons (aref input j) output))
-      (setq out (1+ out))
-      (setq j (1+ j)))
+      (setq out (1+ out)))
     (setq output (nreverse output))
     (if (> b 0) (setq in (1+ b)) (setq in 0))
     (while (< in len)
@@ -1049,7 +1044,7 @@ That is, each line may be more than 75."
 	  (if (>= digit mew-puny-base) (error "punycode bad input"))
 	  (setq in (1+ in))
 	  (setq i (+ i (* digit w)))
-	  (if (<= k bias) 
+	  (if (<= k bias)
 	      (setq thr mew-puny-tmin)
 	    (if (>= k (+ bias mew-puny-tmax))
 		(setq thr mew-puny-tmax)
@@ -1065,14 +1060,13 @@ That is, each line may be more than 75."
 	  (setq output (cons n (nthcdr i output)))
 	(setcdr (nthcdr (1- i) output) (cons n (nthcdr i output))))
       (setq i (1+ i)))
-    (setq ret (make-string (* out 2) 0))
-    (setq j 0)
-    (while output
-      (aset ret j (/ (car output) 256))
-      (setq j (1+ j))
-      (aset ret j (% (car output) 256))
-      (setq j (1+ j))
-      (setq output (cdr output)))
+    (setq ret (mew-make-string (* out 2)))
+    (let ((j 0))
+      (dolist (op output)
+	(aset ret j (/ op 256))
+	(setq j (1+ j))
+	(aset ret j (% op 256))
+	(setq j (1+ j))))
     ret))
 
 (defun mew-puny-encode (input)
@@ -1087,13 +1081,12 @@ That is, each line may be more than 75."
 	 (n mew-puny-initial-n)
 	 (bias mew-puny-initial-bias)
 	 (delta 0) (out 0)
-	 (output (make-string (* len 4) 0))
-	 (j 0) h b m q k thr uni)
-    (while (< j len)
+	 (output (mew-make-string (* len 4)))
+	 h b m q k thr uni)
+    (dotimes (j len)
       (setq uni (aref input j))
       (setq j (1+ j))
       (setq uni (+ (* uni 256) (aref input j)))
-      (setq j (1+ j))
       (when (< uni 128) ;; basic
 	(aset output out uni)
 	(setq out (1+ out))))
@@ -1104,21 +1097,17 @@ That is, each line may be more than 75."
       (setq out (1+ out)))
     (while (< h h-len)
       (setq m 65536) ;; 17bits
-      (setq j 0)
-      (while (< j len)
+      (dotimes (j len)
 	(setq uni (aref input j))
 	(setq j (1+ j))
 	(setq uni (+ (* uni 256) (aref input j)))
-	(setq j (1+ j))
 	(if (and (>= uni n) (< uni m)) (setq m uni)))
       (setq delta (+ delta (* (- m n) (1+ h))))
       (setq n m)
-      (setq j 0)
-      (while (< j len)
+      (dotimes (j len)
 	(setq uni (aref input j))
 	(setq j (1+ j))
 	(setq uni (+ (* uni 256) (aref input j)))
-	(setq j (1+ j))
 	(when (< uni n)
 	  (setq delta (1+ delta))
 	  (if (= delta 0) (error "punycode overflow")))
@@ -1160,7 +1149,7 @@ That is, each line may be more than 75."
       (save-restriction
 	(narrow-to-region beg end)
 	(goto-char (point-min))
-	(while (re-search-forward "[^.]?[^. -][^.]*" nil t)
+	(while (re-search-forward "[^.]?[^.\000-\177][^.]*" nil t)
 	  (setq idn (mew-match-string 0))
 	  (delete-region (match-beginning 0) (match-end 0))
 	  (insert (mew-puny-encode idn))))
@@ -1170,13 +1159,13 @@ That is, each line may be more than 75."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2007 Mew developing team.
+;; Copyright (C) 1997-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1185,7 +1174,7 @@ That is, each line may be more than 75."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
