@@ -14,7 +14,7 @@
 
 (defun mew-draft-on-field-p ()
   (if (bolp)
-      (if (bobp) 
+      (if (bobp)
 	  t
 	(save-excursion
 	  (forward-line -1)
@@ -25,7 +25,7 @@
 	(if (looking-at mew-lwsp)
 	    nil
 	  (if (search-forward ":" pos t) nil t))))))
-      
+
 (defun mew-draft-on-value-p (switch)
   (save-excursion
     (beginning-of-line)
@@ -34,7 +34,7 @@
     (if (looking-at "\\([^:]*:\\)")
 	(mew-field-get-func (match-string 1) switch)
       nil))) ;; what a case reaches here?
-  
+
 ;;
 ;; Window management for completion candidates
 ;;
@@ -43,7 +43,7 @@
 
 (defun mew-complete-window-delete (&optional force)
   (when (mew-ainfo-get-win-cfg)
-    ;; (mew-ainfo-get-win-cfg) remains when the last completion  
+    ;; (mew-ainfo-get-win-cfg) remains when the last completion
     ;; finished with multiple candidates.
     ;; (e.g. foo<RET> when foo and foobar are displayed.)
     ;; In this case, this function is called in another
@@ -127,7 +127,7 @@ is inserted before the cursor, the short name is expanded to its address."
   (if (mew-draft-on-field-p)
       (mew-complete-field)
     (let ((func (mew-draft-on-value-p mew-field-completion-switch)))
-      (if func 
+      (if func
 	  (funcall func)
 	(tab-to-tab-stop))))) ;; default keybinding
 
@@ -197,21 +197,60 @@ is inserted before the cursor, the short name is expanded to its address."
 	(insert word)
 	(message "'%s' cannot be expanded" word)))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Completing folders
+;;;
+
+(defmacro mew-complete-proto-folder (sym &rest body)
+  ;; (declare (indent 1))
+  `(if mew-input-folder-search-direction
+       (mew-input-folder-search-complete)
+     (mew-draft-set-completion-ignore-case mew-complete-folder-ignore-case)
+     (let ((,sym (mew-delete-backward-char))
+	   (completion-ignore-case mew-complete-folder-ignore-case)
+	   (mew-inherit-complete-folder t))
+       ,@body)))
+
+(put 'mew-complete-proto-folder 'lisp-indent-function 1)
+
 (defun mew-complete-local-folder ()
   "Local folder complete function."
   (interactive)
-  (if mew-input-folder-search-direction
-      (mew-input-folder-search-complete)
-    (mew-draft-set-completion-ignore-case mew-complete-folder-ignore-case)
-    (let ((word (mew-delete-backward-char))
-	  (completion-ignore-case mew-complete-folder-ignore-case)
-	  (mew-inherit-complete-folder t))
-      (if (null word)
-	  (mew-complete-window-show (list "+"))
-	(if (and (mew-folder-absolutep word)
-		 (not (mew-draft-or-header-p)))
-	    (mew-complete word (mew-complete-directory-alist word) "directory" nil)
-	  (mew-complete word (mew-local-folder-alist) "folder" nil))))))
+  (mew-complete-proto-folder word
+    (if (null word)
+       (mew-complete-window-show (list "+"))
+     (if (and (mew-folder-absolutep word)
+	      (not (mew-draft-or-header-p)))
+	 (mew-complete word (mew-complete-directory-alist word) "directory" nil)
+       (mew-complete word (mew-local-folder-alist) "folder" nil)))))
+
+;; case is specified by mew-inherit-case.
+(defun mew-complete-imap-folder ()
+  "IMAP folder complete function."
+  (interactive)
+  (mew-complete-proto-folder word
+    (if (null word)
+	(mew-complete-window-show (list "%"))
+      (mew-complete
+       word
+       (mew-imap-folder-alist mew-inherit-case) ;; ie mew-sinfo-get-case
+       "mailbox"
+       nil))))
+
+(defun mew-complete-fcc-folder ()
+  "Fcc: folder complete function."
+  (interactive)
+  (mew-complete-proto-folder word
+    (if (null word)
+	(mew-complete-window-show (list "+" "%"))
+      (cond
+       ((and (mew-folder-absolutep word) (not (mew-draft-or-header-p)))
+	(mew-complete word (mew-complete-directory-alist word) "directory" nil))
+       ((mew-folder-imapp word)
+	(mew-complete word (mew-imap-folder-alist (mew-tinfo-get-case)) "mailbox" nil))
+       (t
+	(mew-complete word (mew-local-folder-alist) "folder" nil))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -293,24 +332,6 @@ is inserted before the cursor, the short name is expanded to its address."
 	 "case"
 	 nil)))))))
 
-;; called by mew-input-refile-folders.
-;; case is specified by mew-inherit-case.
-(defun mew-complete-imap-folder ()
-  "IMAP folder complete function."
-  (interactive)
-  (if mew-input-folder-search-direction
-      (mew-input-folder-search-complete)
-    (let ((word (mew-delete-backward-char))
-	  (completion-ignore-case mew-complete-folder-ignore-case)
-	  (mew-inherit-complete-folder t))
-      (if (null word)
-	  (mew-complete-window-show (list "%"))
-	(mew-complete
-	 word
-	 (mew-imap-folder-alist mew-inherit-case) ;; ie mew-sinfo-get-case
-	 "mailbox"
-	 nil)))))
-
 (defun mew-complete-case ()
   "Complete function for cases."
   (interactive)
@@ -338,8 +359,8 @@ is inserted before the cursor, the short name is expanded to its address."
 (defun mew-circular-complete-domain ()
   "Circular completion of domains for To:, Cc:, etc.
 If the @ character does not exist, the first value of
-mew-mail-domain-list is inserted. If exists, the next value of 
-mew-mail-domain-list concerned with the string between @ and 
+mew-mail-domain-list is inserted. If exists, the next value of
+mew-mail-domain-list concerned with the string between @ and
 the cursor is inserted."
   (interactive)
   (mew-draft-set-completion-ignore-case
@@ -520,7 +541,7 @@ the name exists."
 	(progn
 	  ;; If WORD contains ':', change alist for completion.
 	  (setq field (car (mew-split word ?:)))
-	  (setq alist 
+	  (setq alist
 		(mapcar (lambda (str) (list (concat field ":" str))) mew-sort-modes)))
       ;; Otherwise, alist is mew-sort-key-alist itself.
       (setq alist mew-sort-key-alist))
@@ -657,9 +678,7 @@ the name exists."
       (delete-region savemax (point-max))
       (when quit-flag
 	(setq quit-flag nil)
-	(setq unread-command-events (list (if mew-xemacs-p
-					      (character-to-event 7)
-					    7))))))) ;; 7 == C-g
+	(setq unread-command-events (list 7)))))) ;; 7 == C-g
 
 ;;
 ;; Extracting completion key
@@ -696,7 +715,7 @@ the name exists."
 
 (defun mew-delete-pattern ()
   (let ((pos (point)))
-    (if (re-search-backward " \\|(\\|&\\||\\|!" nil t)
+    (if (re-search-backward " \\|(\\|&\\||\\|!\\|," nil t)
 	(forward-char 1)
       (beginning-of-line))
     (prog1
@@ -750,7 +769,7 @@ the name exists."
   (let ((len (length x))
 	(ret nil)
 	(first (car x)))
-    (cond 
+    (cond
      ((= len 0) nil)
      ((= len 1) (list (cons first first)))
      (t
@@ -764,13 +783,13 @@ the name exists."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2007 Mew developing team.
+;; Copyright (C) 1997-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -779,7 +798,7 @@ the name exists."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
