@@ -7,7 +7,7 @@
 
 (require 'mew)
 
-(defsubst mew-case-default-p (case)
+(defun mew-case-default-p (case)
   (or (null case) (string= case mew-case-default)))
 
 (defvar mew-generate-mail-address-list nil)
@@ -18,20 +18,29 @@
 ;;
 ;;
 
-(defun mew-cfent-value (case key def &optional symbol)
+(defun mew-cfent-value (case key def &optional type)
   (let ((cases (and case (mapcar 'mew-chop (mew-split case ?,))))
-	ent ret)
+	ent ret cfent)
     (or (member mew-case-default cases)
 	(setq cases (cons mew-case-default cases)))
     (setq cases (nreverse cases))
     (catch 'loop
-      (while cases
-	(setq ent (assoc key (mew-cfent-by-case (car cases))))
+      (dolist (cs cases)
+	(setq cfent (mew-cfent-by-case cs))
+	(setq ent (or (assoc key cfent) ;; old
+		      (assoc (intern key) cfent))) ;; new
 	(when ent
-	  (setq ret (cdr ent))
-	  (if symbol (setq ret (symbol-value ret)))
-	  (if ent (throw 'loop nil)))
-	(setq cases (cdr cases)))
+	  (setq ret (cdr ent)) ;; old
+	  (cond
+	   ((eq type 'symbol)
+	    (if (consp ret) (setq ret (car ret)))
+	    (setq ret (symbol-value ret)))
+	   ((eq type 'list)
+	    (if (and (consp (car ret)) (null (cdr ret)))
+		(setq ret (car ret)))) ;; new
+	   (t
+	    (if (consp ret) (setq ret (car ret))))) ;; new
+	  (if ent (throw 'loop nil))))
       (setq ret def))
     ret))
 
@@ -46,6 +55,12 @@
   (if (null mew-mail-domain-list) (setq mew-generate-mail-domain-list t))
   (if (null mew-from-list) (setq mew-generate-from-list t)))
 
+(defun mew-config-case-name (ent)
+  (let ((case (car ent)))
+    (if (symbolp case)
+	(symbol-name case)
+      case)))
+
 (defun mew-config-setup ()
   (if mew-generate-mail-address-list (setq mew-mail-address-list nil))
   (if mew-generate-mail-domain-list (setq mew-mail-domain-list nil))
@@ -53,17 +68,15 @@
   (setq mew-inbox-folders nil)
   (setq mew-queue-folders nil)
   (setq mew-postq-folders nil)
-  (setq mew-config-cases (mapcar 'car mew-config-alist))
+  (setq mew-config-cases (mapcar 'mew-config-case-name mew-config-alist))
   (or (member mew-case-default mew-config-cases)
       (setq mew-config-cases (cons mew-case-default mew-config-cases)))
   (setq mew-config-cases2
 	(append mew-folder-prefixes
 		(mapcar (lambda (x) (concat x ":")) mew-config-cases)))
   (let ((cases (reverse mew-config-cases)) ;; must copy
-	case val) 
-    (while cases
-      (setq case (car cases))
-      (setq cases (cdr cases))
+	val)
+    (dolist (case cases)
       (setq val (mew-inbox-folder case))
       (or (mew-folder-inboxp val)
 	  (setq mew-inbox-folders (cons val mew-inbox-folders)))
@@ -173,7 +186,7 @@
       (mew-mail-address case)))
 
 (defun mew-smtp-auth-list (&optional case)
-  (mew-cfent-value case "smtp-auth-list" mew-smtp-auth-list))
+  (mew-cfent-value case "smtp-auth-list" mew-smtp-auth-list 'list))
 
 (defun mew-smtp-mail-from (&optional case)
   (mew-cfent-value case "smtp-mail-from" mew-smtp-mail-from))
@@ -242,7 +255,7 @@
   (mew-cfent-value case "pop-user" mew-pop-user))
 
 (defun mew-pop-auth-list (&optional case)
-  (mew-cfent-value case "pop-auth-list" mew-pop-auth-list))
+  (mew-cfent-value case "pop-auth-list" mew-pop-auth-list 'list))
 
 (defun mew-pop-size (&optional case)
   (mew-cfent-value case "pop-size" mew-pop-size))
@@ -286,7 +299,7 @@
   (mew-cfent-value case "imap-user" mew-imap-user))
 
 (defun mew-imap-auth-list (&optional case)
-  (mew-cfent-value case "imap-auth-list" mew-imap-auth-list))
+  (mew-cfent-value case "imap-auth-list" mew-imap-auth-list 'list))
 
 (defun mew-imap-size (&optional case)
   (mew-cfent-value case "imap-size" mew-imap-size))
@@ -298,7 +311,7 @@
   (mew-cfent-value case "imap-header-only" mew-imap-header-only))
 
 (defun mew-imap-prefix-list (&optional case)
-  (mew-cfent-value case "imap-prefix-list" mew-imap-prefix-list))
+  (mew-cfent-value case "imap-prefix-list" mew-imap-prefix-list 'list))
 
 ;;
 
@@ -367,10 +380,10 @@
   (mew-cfent-value case "imap-trash-folder" mew-imap-trash-folder))
 
 (defun mew-imap-trash-folder-list (&optional case)
-  (mew-cfent-value case "imap-trash-folder-list" mew-imap-trash-folder-list))
+  (mew-cfent-value case "imap-trash-folder-list" mew-imap-trash-folder-list 'list))
 
 (defun mew-header-alist (&optional case)
-  (mew-cfent-value case "header-alist" mew-header-alist))
+  (mew-cfent-value case "header-alist" mew-header-alist 'list))
 
 (defun mew-signature-file (&optional case)
   (mew-cfent-value case "signature-file" mew-signature-file))
@@ -445,13 +458,39 @@
 ;;
 
 (defun mew-inbox-action-alist (&optional case)
-  (mew-cfent-value case "inbox-action-alist" mew-inbox-action-alist))
+  (mew-cfent-value case "inbox-action-alist" mew-inbox-action-alist 'list))
 
 ;;
 
 (defun mew-draft-privacy-method (&optional case)
   (mew-cfent-value case "draft-privacy-method" mew-draft-privacy-method))
 
+(defun mew-protect-privacy-always (&optional case)
+  (mew-cfent-value case "protect-privacy-always" mew-protect-privacy-always))
+
+(defun mew-protect-privacy-always-type (&optional case)
+  (mew-cfent-value case "protect-privacy-always-type" mew-protect-privacy-always-type))
+
+(defun mew-protect-privacy-encrypted (&optional case)
+  (mew-cfent-value case "protect-privacy-encrypted" mew-protect-privacy-encrypted))
+
+(defun mew-protect-privacy-encrypted-type (&optional case)
+  (mew-cfent-value case "protect-privacy-encrypted-type" mew-protect-privacy-encrypted-type))
+
+(defun mew-protect-privacy-with-old-pgp-signature (&optional case)
+  (mew-cfent-value case "protect-privacy-with-old-pgp-signature" mew-protect-privacy-with-old-pgp-signature))
+
+;;
+
+(defun mew-pgp-signer (&optional case)
+  (mew-cfent-value case "pgp-signer" mew-pgp-signer))
+
+(defun mew-smime-signer (&optional case)
+  (mew-cfent-value case "smime-signer" mew-smime-signer))
+
+(defun mew-use-x-mailer (&optional case)
+  (mew-cfent-value case "use-x-mailer" mew-use-x-mailer))
+
 ;;;
 ;;; Setting case
 ;;;
@@ -466,25 +505,21 @@
 (defun mew-summary-set-case ()
   "Set the case."
   (interactive)
-  (let (case inbox)
-    (setq case (mew-case-set))
+  (let ((case (mew-case-set))) ;; side effect
     (save-excursion
-      (let ((bufs mew-buffers))
-	(while bufs
-	  (if (get-buffer (car bufs))
-	      (progn
-		(set-buffer (car bufs))
-		(cond
-		 ((mew-summary-p)
-		  (mew-summary-mode-name mew-mode-name-summary))
-		 ((mew-virtual-p)
-		  (mew-summary-mode-name mew-mode-name-virtual)))))
-	  (setq bufs (cdr bufs)))))
+      (dolist (buf mew-buffers)
+	(when (get-buffer buf)
+	  (set-buffer buf)
+	  (cond
+	   ((mew-summary-p)
+	    (mew-summary-mode-name mew-mode-name-summary))
+	   ((mew-virtual-p)
+	    (mew-summary-mode-name mew-mode-name-virtual))))))
     (when mew-visit-inbox-after-setting-case
-      (setq inbox (mew-case-folder
-		   case
-		   (mew-proto-inbox-folder (mew-proto case) case)))
-      (mew-summary-visit-folder inbox))))
+      (let ((inbox (mew-case-folder
+		    case
+		    (mew-proto-inbox-folder (mew-proto case) case))))
+	(mew-summary-visit-folder inbox)))))
 
 (defun mew-draft-set-case (&optional arg)
   "Guess case and set the case for output to it. The value is
@@ -532,10 +567,11 @@ fields in the header according to the new value."
       (mew-header-replace-value fcc (mew-fcc new-case))
       (mew-header-replace-value mew-reply-to: (mew-reply-to new-case))
       (mew-header-replace-value mew-organization: (mew-organization new-case))
-      (mew-header-delete-lines (mapcar 'car (mew-header-alist old-case)))
-      (mew-header-delete-lines (mapcar 'car (mew-header-alist new-case)))
-      (mew-header-delete-lines (list mew-x-mailer:))
+      (mew-header-delete-lines (mapcar 'mew-alist-get-key (mew-header-alist old-case)))
+      (mew-header-delete-lines (mapcar 'mew-alist-get-key (mew-header-alist new-case)))
+      (mew-header-delete-lines (list mew-x-face: mew-x-mailer:))
       (mew-header-goto-end)
+      (mew-draft-header-insert-xface)
       (mew-draft-header-insert-alist (mew-header-alist new-case))
       ;; X-Mailer: must be the last
       (mew-draft-header-insert mew-x-mailer: mew-x-mailer))))
@@ -568,13 +604,13 @@ fields in the header according to the new value."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2007 Mew developing team.
+;; Copyright (C) 1999-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -583,7 +619,7 @@ fields in the header according to the new value."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
