@@ -62,7 +62,7 @@
     (,mew-xg  "-d" "-g" "-t")
     (,mew-xuu "-d" "-u")))
 
-(defsubst mew-prog-mime-decode-get-opt (cte switch)
+(defun mew-prog-mime-decode-get-opt (cte switch)
   (cdr (mew-assoc-case-equal cte switch 0)))
 
 (defvar mew-decode-multipart-encrypted-switch
@@ -75,13 +75,13 @@
 
 ;;
 
-(defsubst mew-decode-get-security-func (proto switch)
+(defun mew-decode-get-security-func (proto switch)
   (nth 1 (mew-assoc-case-equal proto switch 0)))
 
-(defsubst mew-decode-get-security-existence (proto switch)
+(defun mew-decode-get-security-existence (proto switch)
   (symbol-value (nth 2 (mew-assoc-case-equal proto switch 0))))
 
-(defsubst mew-decode-get-security-prog (proto switch)
+(defun mew-decode-get-security-prog (proto switch)
   (symbol-value (nth 3 (mew-assoc-case-equal proto switch 0))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -89,7 +89,7 @@
 ;;; MIME decoder
 ;;;
 
-(defsubst mew-decode-error (error-msg)
+(defun mew-decode-error (error-msg)
   (mew-xinfo-set-decode-err error-msg)
   (error ""))
 
@@ -168,7 +168,7 @@ This commands toggles visibility of these lines."
 	     (setq vs-end (next-single-property-change vs-beg 'mew-visible)))
 	    (mew-field-other-visible
 	     (setq vs-beg (next-single-property-change (point-min) 'mew-others))
-	     (if vs-beg 
+	     (if vs-beg
 		 (setq vs-end (next-single-property-change vs-beg 'mew-others))
 	       (setq vs-beg ch-beg)
 	       (setq vs-end ch-beg)))
@@ -192,8 +192,7 @@ This commands toggles visibility of these lines."
 	     ;; invisible
 	     (goto-char vs-beg)
 	     (setq cbeg (point))
-	     (insert contents)
-	     (put-text-property cbeg (point) 'mew-visible nil))) ;; for XEmacs
+	     (insert contents)))
 	   (goto-char (point-max)))
 	 (mew-decode-syntax-insert-privacy)
 	 (mew-decode-syntax-insert-warning)
@@ -226,10 +225,8 @@ This commands toggles visibility of these lines."
 	     (setq med (match-end 0))
 	     (forward-line)
 	     (setq nspec (mew-nspec-by-key key))
-	     (setq key-face (or (mew-nspec-keyface nspec)
-				'mew-face-header-key))
-	     (setq val-face (or (mew-nspec-valface nspec)
-				'mew-face-header-marginal))
+	     (setq key-face (mew-key-face key nspec))
+	     (setq val-face (mew-val-face key nspec))
 	     (put-text-property beg med 'face key-face)
 	     (put-text-property med (1- (point)) 'face val-face)
 	     (while (looking-at mew-lwsp+)
@@ -273,7 +270,7 @@ Return the existence MIME-Version: and the value of Subject:."
 	   (setq size (mew-scan-uid-size (mew-buffer-substring med (1- (point)))))
 	   (when (and (eq cnt 1) (mew-msg-truncatedp size))
 	     (mew-xinfo-set-action
-	      (mew-substitute-for-summary 
+	      (mew-substitute-for-summary
 	       "Too large, truncated (the 'T' mark). To get the entire message, type '\\[mew-summary-retrieve-message]'"))))
 	  ((string-match "^Content-" key)
 	   ;; Due to PGP/MIME, properties are not put here.
@@ -300,10 +297,8 @@ Return the existence MIME-Version: and the value of Subject:."
 			  (mew-addrstr-parse-value
 			   (mew-buffer-substring med (point)))))))
 	   (when prop
-	     (setq key-face (or (mew-nspec-keyface nspec)
-				'mew-face-header-key))
-	     (setq val-face (or (mew-nspec-valface nspec)
-				'mew-face-header-marginal))
+	     (setq key-face (mew-key-face key nspec))
+	     (setq val-face (mew-val-face key nspec))
 	     (put-text-property beg med 'face key-face)
 	     (goto-char med)
 	     (forward-line)
@@ -315,7 +310,7 @@ Return the existence MIME-Version: and the value of Subject:."
 	     (when (setq ext (mew-nspec-extraface nspec))
 	       (save-restriction
 		 (narrow-to-region med (point))
-		 (while ext
+		 (while ext ;; cannot use dolist
 		   (goto-char (point-min))
 		   (setq ext-str (car ext))
 		   (setq ext (cdr ext))
@@ -430,11 +425,11 @@ Return a part syntax after moving the beginning of the content body."
 	(setq mew-decode-syntax
 	      (mew-decode-message (mew-decode-syntax-rfc822-head) 0)))
     (condition-case nil
-	(setq mew-decode-syntax 
+	(setq mew-decode-syntax
 	      (mew-decode-message
 	       ;; Call internalform with VIRTUAL content header
 	       ;;     CT: message/rfc822 (virtual)
-	       ;; 
+	       ;;
 	       ;;     Header(RFC822 header + content header)
 	       ;;
 	       ;;     Body(content body)
@@ -449,7 +444,7 @@ Return a part syntax after moving the beginning of the content body."
   ;; Called on the beginning of the RFC822 header in the narrowed region
   ;; hbeg is certainly the beginning of the VIRTUAL content body(i.e. min).
   ;; hend will have to set to the end of PHYSICAL content header(i.e. end)
-  ;; after analyzing the physical content header and body since CD:'s 
+  ;; after analyzing the physical content header and body since CD:'s
   ;; length in the physical content header will change(no need to say
   ;; about the end of the physical content header).
   ;;
@@ -484,12 +479,12 @@ Return a part syntax after moving the beginning of the content body."
 	   (mew-inherit-decode-signer (nth 2 msf)) ;; signature verification
 	   part)
       ;; the beginning of the physical content header (cur)
-      (cond 
+      (cond
        (mimep ;; MIME
 	(save-restriction
 	  (narrow-to-region (point) (point-max))
 	  (setq part (mew-decode-singlepart cnt nil 'message))
-	  ;; hend is always 1 char smaller than the beginning of 
+	  ;; hend is always 1 char smaller than the beginning of
 	  ;; the physical content body.
 	  ;; If multipart/alternative mew-syntax-get-begin2 gets
 	  ;; correct value like other cases.
@@ -525,9 +520,8 @@ Return a part syntax after moving the beginning of the content body."
     (unless (or (null cte) (mew-cte-composite-p cte))
       (when (and (mew-case-equal cte mew-b64) (fboundp 'base64-decode-region))
 	(condition-case nil
-	    ;; Emacs 20
 	    (setq decoded (base64-decode-region beg (point-max)))
-	  (error (setq decoded nil))) ;; Emacs 21
+	  (error (setq decoded nil)))
 	(when (and decoded linebasep)
 	  (goto-char beg)
 	  (mew-crlf-to-lf)))
@@ -545,16 +539,14 @@ Return a part syntax after moving the beginning of the content body."
 	    ;; Never reach here when decoding.
 	    (mew-decode-error (concat "Unknown CTE: " cte))
 	  (setq file (mew-make-temp-name))
-	  (mew-frwlet
-	   mew-cs-dummy mew-cs-text-for-write
-	   ;; NEVER use call-process-region for privacy reasons
-	   (write-region beg (point-max) file nil 'no-msg))
+	  (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	    ;; NEVER use call-process-region for privacy reasons
+	    (write-region beg (point-max) file nil 'no-msg))
 	  (delete-region beg (point-max))
-	  (mew-piolet
-	   mew-cs-binary mew-cs-dummy
-	   ;; mew-prog-mime-decode converts CRLF to LF, so
-	   ;; read input as binary.
-	   (apply 'call-process mew-prog-mime-decode file '(t nil) nil opt))
+	  (mew-piolet mew-cs-binary mew-cs-dummy
+	    ;; mew-prog-mime-decode converts CRLF to LF, so
+	    ;; read input as binary.
+	    (apply 'call-process mew-prog-mime-decode file '(t nil) nil opt))
 	  (mew-delete-file file)))
        (t
 	(mew-decode-error (concat mew-prog-mime-decode " does not exist")))))
@@ -758,10 +750,10 @@ Return a part syntax after moving the beginning of the content body."
 	(mew-decode-error "Boundary has invalid charactor")))
     (concat "^--" (regexp-quote mboundary) "\\(--\\|\\)[ \t]*$")))
 
-(defsubst mew-decode-multipart-boundary-cont ()
+(defun mew-decode-multipart-boundary-cont ()
   (string= (mew-match-string 1) ""))
 
-(defsubst mew-decode-multipart-boundary-end ()
+(defun mew-decode-multipart-boundary-end ()
   (string= (mew-match-string 1) "--"))
 
 (defun mew-decode-multipart (syntax cnt &optional dct)
@@ -776,8 +768,8 @@ Return a part syntax after moving the beginning of the content body."
     (unless boundary
       (mew-decode-error "No boundary parameter for multipart"))
     (mew-syntax-set-key syntax 'multi)
-    ;; RFC 2046 says, "An exact match of the entire candidate 
-    ;; line is not required; it is sufficient that the boundary 
+    ;; RFC 2046 says, "An exact match of the entire candidate
+    ;; line is not required; it is sufficient that the boundary
     ;; appear in its entirety following the CRLF".
     (setq bregex (mew-decode-multipart-boundary-regex boundary))
     (unless (and (re-search-forward bregex nil t)
@@ -846,6 +838,8 @@ Return a part syntax after moving the beginning of the content body."
 ;;; Privacy services
 ;;;
 
+(mew-defstruct privacy-dinfo ct proto result)
+
 (defun mew-decode-multipart-encrypted (syntax cnt)
   ;; called in narrowed region
   ;;
@@ -884,7 +878,7 @@ Return a part syntax after moving the beginning of the content body."
 	(if existp (setq file1 (mew-save-decode-form syntax1)))
       (setq unknown t))
     (forward-line) ;; the beginning of the encrypted part
-    (setq start (point)) 
+    (setq start (point))
     ;;
     (unless (and (re-search-forward bregex nil t)
 		 (mew-decode-multipart-boundary-end))
@@ -895,7 +889,7 @@ Return a part syntax after moving the beginning of the content body."
 		     (mew-decode-security-singlepart start (1- (point))))))
     ;;
     (delete-region (point-min) (point-max))
-    ;; 
+    ;;
     ;; Call protocol function
     (cond
      (unknown
@@ -908,7 +902,7 @@ Return a part syntax after moving the beginning of the content body."
       (mew-set '(file3 result) file3result)))
     ;;
     (if (and func existp (file-exists-p file3))
-	(mew-flet 
+	(mew-flet
 	 (mew-insert-file-contents file3)
 	 (put-text-property (point-min) (point-max) 'mew-noncontents nil)
 	 ;; because of RICH functionality of RFC1847... Gee dirty!
@@ -926,7 +920,7 @@ Return a part syntax after moving the beginning of the content body."
     (setq privacy (mew-syntax-get-privacy syntax3))
     (if privacy (setq result (concat result "\n\t")))
     (mew-syntax-set-privacy
-     syntax3 (cons (list mew-ct-mle proto result) privacy))
+     syntax3 (cons (mew-make-privacy-dinfo :ct mew-ct-mle :proto proto :result result) privacy))
     syntax3))
 
 (defun mew-decode-multipart-signed (syntax cnt)
@@ -959,10 +953,10 @@ Return a part syntax after moving the beginning of the content body."
     (unless (and (re-search-forward bregex nil t)
 		 (mew-decode-multipart-boundary-cont))
       (mew-decode-error "No second boundary for Multipart/Signed"))
-    (beginning-of-line) 
+    (beginning-of-line)
     (setq end1 (1- (point))) ;; the end of the signed part
     (forward-line) ;; the beginning of the key part
-    (setq start2 (point)) 
+    (setq start2 (point))
     ;;
     (unless (and (re-search-forward bregex nil t)
 		 (mew-decode-multipart-boundary-end))
@@ -998,7 +992,7 @@ Return a part syntax after moving the beginning of the content body."
     (setq privacy (mew-syntax-get-privacy syntax3))
     (if privacy (setq result (concat result "\n\t")))
     (mew-syntax-set-privacy
-     syntax3 (cons (list mew-ct-mls proto result) privacy))
+     syntax3 (cons (mew-make-privacy-dinfo :ct mew-ct-mls :proto proto :result result) privacy))
     syntax3))
 
 (defun mew-decode-security-singlepart (beg end)
@@ -1023,7 +1017,7 @@ Return a part syntax after moving the beginning of the content body."
     (save-excursion
       (goto-char (point-min))
       (catch 'header
-	(while (re-search-forward 
+	(while (re-search-forward
 		"^\r?$\\|^Content-Transfer-Encoding:[ \t]*" nil t)
 	  (setq key (mew-match-string 0))
 	  (setq start (match-end 0))
@@ -1044,13 +1038,13 @@ Return a part syntax after moving the beginning of the content body."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2007 Mew developing team.
+;; Copyright (C) 1996-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1059,7 +1053,7 @@ Return a part syntax after moving the beginning of the content body."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
