@@ -17,7 +17,7 @@
     (,mew-qp  "-q")
     (,mew-xg  "-g""-t")))
 
-(defsubst mew-prog-mime-encode-get-opt (cte switch)
+(defun mew-prog-mime-encode-get-opt (cte switch)
   (cdr (mew-assoc-case-equal cte switch 0)))
 
 (defvar mew-encode-multipart-encrypted-switch
@@ -29,10 +29,10 @@
 
 ;;
 
-(defsubst mew-encode-get-security-func (proto switch)
+(defun mew-encode-get-security-func (proto switch)
   (nth 1 (mew-assoc-case-equal proto switch 0)))
 
-(defsubst mew-encode-get-canonicalize-func (proto switch)
+(defun mew-encode-get-canonicalize-func (proto switch)
   (nth 2 (mew-assoc-case-equal proto switch 0)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -40,7 +40,7 @@
 ;;; MIME encoder
 ;;;
 
-(defsubst mew-encode-error (error-msg)
+(defun mew-encode-error (error-msg)
   (mew-tinfo-set-encode-err error-msg)
   (error ""))
 
@@ -73,7 +73,7 @@
   (list mew-subj: mew-in-reply-to: mew-references:))
 
 (defconst mew-multiple-field-combine-list
-  (list mew-from: mew-reply-to: mew-to: mew-cc:)) 
+  (list mew-from: mew-reply-to: mew-to: mew-cc:))
 
 (defun mew-header-count-field (field)
   (let ((case-fold-search t)
@@ -116,11 +116,9 @@
     (save-excursion
       (save-restriction
 	(narrow-to-region (point-min) (mew-header-end))
-	(let ((fields mew-multiple-field-error-list))
-	  (while fields
-	    (if (> (mew-header-count-field (car fields)) 1)
-		(mew-encode-error (format "Multiple %s!" (car fields))))
-	    (setq fields (cdr fields))))
+	(dolist (field mew-multiple-field-error-list)
+	  (if (> (mew-header-count-field field) 1)
+	      (mew-encode-error (format "Multiple %s!" field))))
 	(mapcar 'mew-header-combine-field mew-multiple-field-combine-list)))))
 
 (defun mew-encode-ask-subject ()
@@ -129,23 +127,23 @@
     (let ((subj (read-string (concat mew-subj: " "))))
       (mew-header-replace-value mew-subj: subj))))
 
+(defun mew-encode-fcc-folder-check (fld)
+  (let ((folder (mew-canonicalize-folder fld)))
+    (when (and mew-ask-fcc
+	       (mew-folder-localp folder)
+	       (not (file-directory-p (mew-expand-folder folder))))
+      (if (y-or-n-p (format "%s does not exist. Create it? " folder))
+	  (mew-local-folder-check folder)
+	(mew-encode-error "Edit Fcc:")))
+    folder))
+
 (defun mew-encode-ask-fcc (resentp)
   (let* ((target (if resentp mew-resent-fcc: mew-fcc:))
-	 (folders (mew-header-get-value target))
-	 folder ret)
+	 (folders (mew-header-get-value target)))
     (when folders
-      (setq folders (mew-addrstr-parse-value-list2 folders))
-      (while folders
-	(setq folder (mew-canonicalize-folder (car folders)))
-	(setq folders (cdr folders))
-	(when (and mew-ask-fcc
-		   (mew-folder-localp folder)
-		   (not (file-directory-p (mew-expand-folder folder))))
-	  (if (y-or-n-p (format "%s does not exist. Create it? " folder))
-	      (mew-local-folder-check folder)
-	    (mew-encode-error "Edit Fcc:")))
-	(setq ret (cons folder ret))))
-    (nreverse ret)))
+      (mapcar
+       'mew-encode-fcc-folder-check
+       (mew-addrstr-parse-value-list2 folders)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -183,9 +181,7 @@
 	    (backward-char 1)
 	    ;; single quote is allowed for local-part
 	    (setq vals (mapcar 'mew-chop (mew-split-quoted val ?, ?: ?\; 'no-single)))
-	    (while vals
-	      (setq val (car vals))
-	      (setq vals (cdr vals))
+	    (dolist (val vals)
 	      (setq ins nil addrs nil)
 	      (cond
 	       ;; phrase:addr1,addr2;
@@ -213,7 +209,7 @@
 		    (setq ins (copy-sequence addrs))))))
 	      (setq insl (nconc insl ins))
 	      (unless (string-match "bcc" key) ;; Removing Bcc:
-		(setq ret (nconc ret addrs)))) ;; end of while
+		(setq ret (nconc ret addrs)))) ;; end of dolist
 	    (insert " " (mapconcat 'identity insl ", "))
 	    (setq insl nil)
 	    (forward-line))))
@@ -302,7 +298,7 @@
 ;;; Misc
 ;;;
 
-(defun mew-encode-set-privacy (pnm privacy)
+(defun mew-encode-set-privacy (pnm privacy case)
   (unless (or (mew-syntax-get-privacy mew-encode-syntax) ;; specified
 	      ;; encryption previously failed, so bypass.
 	      (and (null privacy) (mew-tinfo-get-privacy-err)))
@@ -312,10 +308,10 @@
 	(setq type privacy))
        ((mew-tinfo-get-privacy-type)
 	(setq type (mew-tinfo-get-privacy-type)))
-       ((and (mew-tinfo-get-encrypted-p) mew-protect-privacy-encrypted)
-	(setq type mew-protect-privacy-encrypted-type))
-       (mew-protect-privacy-always
-	(setq type mew-protect-privacy-always-type)))
+       ((and (mew-tinfo-get-encrypted-p) (mew-protect-privacy-encrypted case))
+	(setq type (mew-protect-privacy-encrypted-type case)))
+       ((mew-protect-privacy-always case)
+	(setq type (mew-protect-privacy-always-type case))))
       (mew-syntax-set-privacy
        mew-encode-syntax
        (mew-pcdb-ct (mew-pcdb-by-service type)))
@@ -349,12 +345,11 @@
   (mew-highlight-header-region (point-min) (point)))
 
 (defun mew-encode-save-draft ()
-  (mew-frwlet
-   mew-cs-dummy mew-cs-text-for-write
-   (write-region (point-min) (point-max) (buffer-file-name) nil 'no-msg)
-   (clear-visited-file-modtime)
-   (mew-delete-file buffer-auto-save-file-name)
-   (set-buffer-modified-p nil)))
+  (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+    (write-region (point-min) (point-max) (buffer-file-name) nil 'no-msg)
+    (clear-visited-file-modtime)
+    (mew-delete-file buffer-auto-save-file-name)
+    (set-buffer-modified-p nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -382,7 +377,7 @@
   (mew-set-buffer-multibyte t)
   (if (buffer-modified-p) (save-buffer)) ;; to make backup
   (widen)
-  (let (multip recipients msgid-logtime)
+  (let (multip recipients msgid logtime)
     (mew-smtp-set-raw-header
      pnm (mew-buffer-substring (point-min) (mew-header-end)))
     (unless headerp
@@ -417,9 +412,9 @@
     (mew-encode-check-sender resentp)
     (mew-encode-from case resentp)
     ;;
-    (setq msgid-logtime (mew-encode-id-date pnm (mew-smtp-message-id case) resentp))
-    (mew-smtp-set-msgid pnm (nth 0 msgid-logtime))
-    (mew-smtp-set-logtime pnm (nth 1 msgid-logtime))
+    (mew-set '(msgid logtime) (mew-encode-id-date pnm (mew-smtp-message-id case) resentp))
+    (mew-smtp-set-msgid pnm msgid)
+    (mew-smtp-set-logtime pnm logtime)
     ;;
     (goto-char (mew-header-end))
     (forward-line) ;; necessary for PGP
@@ -427,8 +422,16 @@
     (message "Making a message...")
     ;; save syntax before setting privacy
     (unless headerp
-      (mew-encode-set-privacy pnm privacy)
-      (let ((mew-inherit-encode-signer (or signer (mew-get-my-address))))
+      (mew-encode-set-privacy pnm privacy case)
+      (let ((mew-inherit-encode-signer signer)
+	    (method (mew-draft-privacy-method (mew-tinfo-get-case))))
+	(unless mew-inherit-encode-signer
+	  (setq mew-inherit-encode-signer
+		(cond
+		 ((eq method 'pgp)   (mew-pgp-signer   (mew-tinfo-get-case)))
+		 ((eq method 'smime) (mew-smime-signer (mew-tinfo-get-case)))))
+	  (or mew-inherit-encode-signer
+	      (setq mew-inherit-encode-signer (mew-get-my-address))))
 	(goto-char (mew-header-end)) ;; due to invalid null lines in the header
 	(forward-line)
 	(if multip
@@ -462,7 +465,7 @@
   (mew-set-buffer-multibyte t)
   (if (buffer-modified-p) (save-buffer)) ;; to make backup
   (widen)
-  (let (multip newsgroups msgid-logtime)
+  (let (multip newsgroups msgid logtime)
     (mew-nntp2-set-raw-header
      pnm (mew-buffer-substring (point-min) (mew-header-end)))
     ;; Let's backup
@@ -487,9 +490,9 @@
     ;;
     (mew-encode-from case nil)
     ;;
-    (setq msgid-logtime (mew-encode-id-date pnm (mew-nntp-message-id case)))
-    (mew-nntp2-set-msgid pnm (nth 0 msgid-logtime))
-    (mew-nntp2-set-logtime pnm (nth 1 msgid-logtime))
+    (mew-set '(msgid logtime) (mew-encode-id-date pnm (mew-nntp-message-id case)))
+    (mew-nntp2-set-msgid pnm msgid)
+    (mew-nntp2-set-logtime pnm logtime)
     ;;
     (goto-char (mew-header-end))
     (forward-line) ;; necessary for PGP
@@ -497,7 +500,7 @@
     (message "Making a message...")
     ;; save syntax before setting privacy
     (unless headerp
-      (mew-encode-set-privacy pnm privacy)
+      (mew-encode-set-privacy pnm privacy case)
       (let ((mew-inherit-encode-signer (or signer (mew-get-my-address))))
 	(goto-char (mew-header-end)) ;; due to invalid null lines in the header
 	(forward-line)
@@ -509,7 +512,7 @@
     (mew-overlay-delete-buffer)
     (message "Making a message...done")))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Encoding singlepart
 ;;;
@@ -533,7 +536,7 @@
   (if (and (mew-which-exec mew-prog-8bit) (file-readable-p file))
       (eq (call-process mew-prog-8bit file nil nil "-8") 1)))
 
-(defsubst mew-encode-limit-7bitp (privacy)
+(defun mew-encode-limit-7bitp (privacy)
   ;; Only parts to be signed FIRST are limited to 7bit.
   (and privacy (mew-case-equal mew-ct-mls (car (car privacy)))))
 
@@ -572,9 +575,8 @@
 	    (mew-encode-error
 	     (format "Unknown coding system %s in the body"
 		     (symbol-name cs)))))
-	(mew-frwlet
-	 (or cs mew-cs-autoconv) mew-cs-dummy
-	 (mew-insert-file-contents file)))
+	(mew-frwlet (or cs mew-cs-autoconv) mew-cs-dummy
+	  (mew-insert-file-contents file)))
       ;; A user may specify charset to convey JIS X 0201 Katakana.
       ;; So, we need to avoid the sanity check.
       (unless charset
@@ -635,21 +637,19 @@
      (t
       (if (and no-encoding cte (mew-case-equal cte mew-b64))
 	  (setq cte mew-bin))
-      ;; There are 7bit ascii bodies such as 
+      ;; There are 7bit ascii bodies such as
       ;; application/pgp-encrypted and message/external-body.
       ;; If 7bit or 8bit, it should be linebase.
       (if (null cte) (setq cte mew-7bit))))
     (cond
      ((mew-case-equal cte mew-bin)
-      (mew-frwlet
-       mew-cs-binary mew-cs-dummy
-       (mew-insert-file-contents file)))
+      (mew-frwlet mew-cs-binary mew-cs-dummy
+	(mew-insert-file-contents file)))
      ((or (mew-case-equal cte mew-7bit) (mew-case-equal cte mew-8bit))
       ;; Certainly linebase here.
       (unless textp
-	(mew-frwlet
-	 mew-cs-text-for-read mew-cs-dummy
-	 (mew-insert-file-contents file)))
+	(mew-frwlet mew-cs-text-for-read mew-cs-dummy
+	  (mew-insert-file-contents file)))
       (when (and (string= ct mew-ct-msg)
 		 (or mew-inherit-7bit (not mew-use-8bit))
 		 (mew-case-equal cte mew-8bit))
@@ -659,10 +659,8 @@
 	  (mew-convert-message))))
      ((and (mew-case-equal cte mew-b64) (fboundp 'base64-encode-region))
       (unless textp
-	(mew-frwlet
-	 (if linebasep mew-cs-text-for-read mew-cs-binary)
-	 mew-cs-dummy
-	 (mew-insert-file-contents file)))
+	(mew-frwlet (if linebasep mew-cs-text-for-read mew-cs-binary) mew-cs-dummy
+	  (mew-insert-file-contents file)))
       (when linebasep
 	(goto-char beg)
 	(mew-lf-to-crlf))
@@ -675,14 +673,12 @@
 	  (mew-encode-error (concat "Unknown CTE: " cte))
 	(when textp
 	  (setq file (mew-make-temp-name))
-	  (mew-frwlet
-	   mew-cs-dummy mew-cs-text-for-write
-	   ;; NEVER use call-process-region for privacy reasons
-	   (write-region beg (point-max) file nil 'no-msg))
+	  (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	    ;; NEVER use call-process-region for privacy reasons
+	    (write-region beg (point-max) file nil 'no-msg))
 	  (delete-region beg (point-max)))
-	(mew-piolet
-	 mew-cs-text-for-read mew-cs-dummy
-	 (apply 'call-process mew-prog-mime-encode file t nil opt))
+	(mew-piolet mew-cs-text-for-read mew-cs-dummy
+	  (apply 'call-process mew-prog-mime-encode file t nil opt))
 	(if textp (mew-delete-file file))))
      (t
       (mew-encode-error (concat mew-prog-mime-encode " does not exist"))))
@@ -803,7 +799,7 @@
 	  (setq first (mew-syntax-get-part mew-encode-syntax))
 	  (setq ct (mew-syntax-get-value (mew-syntax-get-ct first) 'cap))
 	  (if (not (mew-ct-textp ct))
-	      (setq multip t)		   
+	      (setq multip t)		
 	    (setq path (expand-file-name
 			(mew-syntax-get-file mew-encode-syntax) path))
 	    (setq privacy (mew-syntax-get-privacy mew-encode-syntax))
@@ -847,7 +843,7 @@
     (while (< cnt len)
       (insert "\n--" boundary "\n")
       (if (mew-syntax-multipart-p (aref syntax cnt))
-	  (setq 8bitp (mew-encode-multipart 
+	  (setq 8bitp (mew-encode-multipart
 		       (aref syntax cnt)
 		       fullname
 		       (1+ depth)))
@@ -866,7 +862,7 @@
     (save-excursion
       (goto-char cte-pos)
       (mew-header-insert mew-cte: (if (> 8bit-cnt 0) mew-8bit mew-7bit)))
-    (when privacy 
+    (when privacy
       (mew-encode-security-multipart
        beg privacy depth (mew-syntax-get-decrypters syntax) cte))
     (goto-char (point-max))
@@ -881,12 +877,11 @@
   (save-restriction
     (narrow-to-region beg (point-max))
     (let (proto ct)
-      (while privacy
-	(goto-char (point-min)) 
-	(mew-set '(ct proto) (car privacy))
+      (dolist (ent privacy)
+	(goto-char (point-min))
+	(mew-set '(ct proto) ent)
 	(setq ct (mew-capitalize ct))
-	(setq privacy (cdr privacy))
-	(cond 
+	(cond
 	 ((string= ct mew-ct-mle)
 	  (mew-encode-multipart-encrypted ct proto depth decrypters cte))
 	 ((string= ct mew-ct-mls)
@@ -926,9 +921,8 @@
 	(mew-lf-to-crlf)))
     (setq send (point-max))
     (setq file (mew-make-temp-name))
-    (mew-frwlet
-     mew-cs-dummy ocs
-     (write-region sbeg send file nil 'no-msg))
+    (mew-frwlet mew-cs-dummy ocs
+      (write-region sbeg send file nil 'no-msg))
     (if retain
 	(mew-remove-buffer tmpbuf)
       (delete-region sbeg send))
@@ -953,7 +947,7 @@
       (error
        (mew-delete-file file1)
        (mew-encode-error
-	(format "unknown error for %s. Check %s, anyway" 
+	(format "unknown error for %s. Check %s, anyway"
 		mew-ct-mle mew-temp-dir))))
     (mew-set '(file2 cte2 file3 cte3 errmsg) fc)
     (if errmsg
@@ -972,14 +966,14 @@
       (mew-header-insert mew-cte: mew-7bit)
       (insert (format "\n--%s\n" boundary))
       ;; Insert control keys
-      (mew-encode-singlepart 
+      (mew-encode-singlepart
        (mew-encode-syntax-single file2 (list proto) cte2))
       (insert (format "\n--%s\n" boundary))
       ;; Insert encrypted body
-      (mew-encode-singlepart 
+      (mew-encode-singlepart
        (mew-encode-syntax-single file3 mew-type-apo cte3))
       (insert (format "\n--%s--\n" boundary))
-      ;; Throw away the garbage 
+      ;; Throw away the garbage
       (mew-delete-file file1)
       (mew-delete-file file2)
       (mew-delete-file file3))))
@@ -1000,7 +994,7 @@
       (error
        (mew-delete-file file1)
        (mew-encode-error
-	(format "unknown error for %s. Check %s, anyway" 
+	(format "unknown error for %s. Check %s, anyway"
 		mew-ct-mls mew-temp-dir))))
     (mew-set '(file2 cte2 micalg errmsg ct2 cdp2) fmc)
     (if errmsg
@@ -1023,10 +1017,10 @@
       (goto-char (point-max))
       ;; After the signed part
       (insert (format "\n--%s\n" boundary))
-      (mew-encode-singlepart 
+      (mew-encode-singlepart
        (mew-encode-syntax-single file2 (or ct2 (list proto)) cte2 nil nil cdp2))
       (insert (format "\n--%s--\n" boundary))
-      ;; Throw away the garbage 
+      ;; Throw away the garbage
       (mew-delete-file file1)
       (mew-delete-file file2))))
 
@@ -1055,15 +1049,12 @@
 	(if (null opt)
 	    (mew-encode-error (concat "Unknown CTE: " cte))
 	  (setq file (mew-make-temp-name))
-	  (mew-frwlet
-	   mew-cs-dummy
-	   (if linebasep mew-cs-text-for-write mew-cs-binary)
-	   ;; NEVER use call-process-region for privacy reasons
-	   (write-region (point-min) (point-max) file nil 'no-msg)
-	   (delete-region (point-min) (point-max)))
-	  (mew-piolet
-	   mew-cs-text-for-read mew-cs-dummy
-	   (apply 'call-process mew-prog-mime-encode file t nil opt))
+	  (mew-frwlet mew-cs-dummy (if linebasep mew-cs-text-for-write mew-cs-binary)
+	    ;; NEVER use call-process-region for privacy reasons
+	    (write-region (point-min) (point-max) file nil 'no-msg)
+	    (delete-region (point-min) (point-max)))
+	  (mew-piolet mew-cs-text-for-read mew-cs-dummy
+	    (apply 'call-process mew-prog-mime-encode file t nil opt))
 	  (mew-delete-file file)))
        (t
 	(mew-encode-error (concat mew-prog-mime-encode " does not exist")))))))
@@ -1110,7 +1101,7 @@
   ;; called on the beginning of a content-header
   (let ((case-fold-search t)
 	cthd-end ctbody-beg
-	ct-val cte-val 
+	ct-val cte-val
 	ctl ct cte charset)
     (unless (re-search-forward mew-eoh nil t)
       (goto-char (point-max)))
@@ -1212,9 +1203,8 @@
   (let* ((file (buffer-file-name))
 	 (back (mew-prepend-prefix file mew-backup-prefix))
 	 (info (concat file mew-draft-info-suffix)))
-    (mew-frwlet
-     mew-cs-dummy mew-cs-m17n
-     (write-region (point-min) (point-max) back nil 'no-msg))
+    (mew-frwlet mew-cs-dummy mew-cs-m17n
+      (write-region (point-min) (point-max) back nil 'no-msg))
     (mew-lisp-save
      info
      (list
@@ -1242,9 +1232,8 @@
   (let* ((file (buffer-file-name))
 	 (back (mew-prepend-prefix file mew-backup-prefix)))
     (when (file-exists-p back)
-      (mew-frwlet
-       mew-cs-m17n mew-cs-dummy
-       (mew-insert-file-contents back)))))
+      (mew-frwlet mew-cs-m17n mew-cs-dummy
+	(mew-insert-file-contents back)))))
 
 (defun mew-encode-undo ()
   "Get back to the draft before making MIME message."
@@ -1266,13 +1255,13 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2007 Mew developing team.
+;; Copyright (C) 1996-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1281,7 +1270,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
