@@ -13,22 +13,41 @@
 ;;;
 
 (defvar mew-port-db
-  '(("smtp"  .  "25")
+  '(("smtp"        25)
     ;; This is officially assigned to another service (urd) by IANA.
     ;; IESG will never assign a port number to SMTP over SSL...
-    ("smtps" . "465")
-    ("pop3"  . "110")
-    ("pop3s" . "995")
-    ("nntp"  . "119")
-    ("nntps" . "563")
-    ("imap"  . "143")
-    ("imaps" . "993")
-    ("submission" . "587")))
+    ("smtps"      465)
+    ("pop3"       110)
+    ("pop3s"      995)
+    ("nntp"       119)
+    ("nntps"      563)
+    ("imap"       143)
+    ("imaps"      993)
+    ("submission" 587)))
 
 (defun mew-serv-to-port (serv)
-  (if (string-match "^[0-9]+$" serv)
-      serv
-    (or (cdr (assoc serv mew-port-db)) "0")))
+  (cond
+   ((integerp serv) serv)
+   ((string-match "^[0-9]+$" serv)
+    (string-to-number serv))
+   (t
+    (or (mew-alist-get-value (assoc serv mew-port-db)) 0))))
+
+(defun mew-*-to-port (port)
+  (cond
+   ((integerp port) port)
+   ((string-match "^[0-9]+$" port)
+    (string-to-number port))
+   (t port)))
+
+(defun mew-*-to-string (any)
+  (cond
+   ((integerp any)
+    (number-to-string any))
+   (t any)))
+
+(defun mew-port-equal (port1 port2)
+  (string= (mew-*-to-string port1) (mew-*-to-string port2)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -61,7 +80,7 @@
   (setq mew-summary-buffer-process key)
   (setq mew-summary-buffer-process-status (format " %s" status))
   (setq mew-summary-buffer-secure-process secure))
-  
+
 
 (defun mew-summary-unlock ()
   (setq mew-summary-buffer-process nil)
@@ -100,7 +119,7 @@
       (let ((msiz (string-to-number siz))
 	    (bsiz (if zero 0 (buffer-size)))
 	    percent substatus)
-	(if (and (numberp mew-net-status-percent-size)
+	(if (and (integerp mew-net-status-percent-size)
 		 (< msiz mew-net-status-percent-size))
 	    (progn
 	      (setq substatus (format "%d/%d" rcnt rttl))
@@ -118,8 +137,8 @@
 (defun mew-net-status3 (buf rttl rcnt)
   (when mew-use-net-status
     (if rttl
-	(if (or (not (numberp mew-local-status-unit))
-		(not (numberp mew-local-status-threshold))
+	(if (or (not (integerp mew-local-status-unit))
+		(not (integerp mew-local-status-threshold))
 		(< (- rttl rcnt) mew-local-status-threshold)
 		(= 0 (% rcnt mew-local-status-unit)))
 	    (mew-net-status buf "Scanning" (format "%d/%d" rcnt rttl)))
@@ -183,19 +202,19 @@
 ;;; Keep time
 ;;;
 
-(defsubst mew-time-diff (t1 t2)
+(defun mew-time-diff (t1 t2)
   (/ (+ (* (- (nth 0 t2) (nth 0 t1)) 65536)
 	(- (nth 1 t2) (nth 1 t1)))
      86400.0)) ;; one day (* 60 60 24)
 
-(defsubst mew-expired-p (time keep)
+(defun mew-expired-p (time keep)
   (cond
    ((and (consp keep)
 	 (stringp (nth 0 keep)) (file-exists-p (nth 0 keep))
-	 (numberp (nth 1 keep)))
+	 (integerp (nth 1 keep)))
     (if (>= (mew-time-diff time (mew-file-get-time (nth 0 keep))) (nth 1 keep))
 	t))
-   ((numberp keep)
+   ((integerp keep)
     (if (>= (mew-time-diff time (current-time)) keep) t))
    ;; ((eq keep t) t)
    ;; This case MUST not be included because messages marked with 'T'
@@ -210,10 +229,10 @@
 (defvar mew-net-uidl-file ".mew-uidl")
 (defvar mew-net-uidl-db nil)
 
-(defsubst mew-net-uidl-db-get (tag)
+(defun mew-net-uidl-db-get (tag)
   (cdr (assoc tag mew-net-uidl-db)))
 
-(defsubst mew-net-uidl-db-set (tag uidl)
+(defun mew-net-uidl-db-set (tag uidl)
   (let* ((ent (assoc tag mew-net-uidl-db)))
     (if ent
 	(setcdr ent uidl)
@@ -302,27 +321,25 @@
 	 (work (concat file mew-queue-work-suffix)) ;; 1.wrk
 	 (info (concat file mew-queue-info-suffix)) ;; 1.mqi
 	 (iwrk (concat info mew-queue-work-suffix)) ;; 1.mqi.wrk
-	 (i 0) buf data)
+	 buf data)
     (when (and (file-readable-p file) (file-readable-p info))
       (rename-file file work 'override)
       ;; If an old buffer exists by accident, we MUST remove the buffer.
       (if (setq buf (get-file-buffer work)) (mew-remove-buffer buf))
-      (mew-frwlet
-       mew-cs-text-for-read mew-cs-dummy
-       (set-buffer (mew-find-file-noselect work)))
+      (mew-frwlet mew-cs-text-for-read mew-cs-dummy
+	(set-buffer (mew-find-file-noselect work)))
       (setq data (mew-lisp-load info))
-      (while (< i n)
-	(aset (mew-info pnm) i (aref data i))
-	(setq i (1+ i)))
+      (dotimes (i n)
+	(aset (mew-info pnm) i (aref data i)))
       (rename-file info iwrk 'override)
       t)))
 
 (defun mew-queue-get-next (pnm qfld msgs n func)
-  (let* (msg flushp)
-    (while (and msgs (not flushp))
-      (setq msg (car msgs))
-      (setq msgs (cdr msgs))
-      (if (mew-queue-insert-file pnm n qfld msg) (setq flushp t)))
+  (let (flushp)
+    (catch 'loop
+      (dolist (msg msgs)
+	(if (mew-queue-insert-file pnm n qfld msg)
+	    (throw 'loop (setq flushp t)))))
     (if (fboundp func) (funcall func pnm msgs)) ;; set-messages
     flushp))
 
@@ -343,15 +360,14 @@
 
 (defun mew-net-fcc-message (case fcc file)
   (let (flds fcc-file link fld imapp)
-    (while fcc
-      (if (mew-folder-imapp (car fcc))
+    (dolist (fc fcc)
+      (if (mew-folder-imapp fc)
 	  (progn
-	    (setq flds (cons (mew-folder-imap-to-fcc case (car fcc)) flds))
+	    (setq flds (cons (mew-folder-imap-to-fcc case fc) flds))
 	    (setq imapp t))
-	(setq flds (cons (car fcc) flds)))
-      (setq fcc (cdr fcc)))
+	(setq flds (cons fc flds))))
     (catch 'loop
-      (while flds
+      (while flds ;; cannot use dolist
 	(setq fld (car flds))
 	(mew-local-folder-check fld)
 	(setq flds (cdr flds))
@@ -361,10 +377,8 @@
 	  (mew-set-file-modes fcc-file)
 	  (mew-touch-folder fld)
 	  (throw 'loop nil))))
-    (while flds
-      (setq fld (car flds))
+    (dolist (fld flds)
       (mew-local-folder-check fld)
-      (setq flds (cdr flds))
       (setq link (mew-folder-new-message fld))
       (when link
 	(mew-link fcc-file link)
@@ -443,30 +457,33 @@
       (mew-local-retrieve 'scan folder range)
       (mew-rendezvous mew-summary-buffer-process))))
 
+(defun mew-summary-case-proto ()
+  (let (case:folder case proto)
+    (cond
+     ((mew-virtual-for-one-summary)
+      (setq case:folder (mew-summary-physical-folder))
+      (setq case (or (mew-case:folder-case case:folder)
+		     mew-case))
+      (setq proto (mew-folder-prefix (mew-case:folder-folder case:folder))))
+     ((mew-virtual-p)
+      (setq case (or mew-case mew-case-default))
+      (setq proto (mew-proto case)))
+     (t ;; Summary
+      (setq case (or (mew-sinfo-get-case)   ;; remote
+		     mew-case))	            ;; local
+      (setq proto (mew-folder-prefix (mew-sinfo-get-folder)))))
+    (list case proto)))
+
 (defun mew-summary-retrieve (&optional no-flush)
-  "In local folders, retrieve messages to +inbox asynchronously 
-according to 'mew-mailbox-type' and 'mew-case'. 
+  "In local folders, retrieve messages to +inbox asynchronously
+according to 'mew-mailbox-type' and 'mew-case'.
 If 'mew-auto-flush-queue' is non-nil, +queue is flushed.
 If called with '\\[universal-argument]', +queue is not flushed.
 
 In remote folders, visit an inbox folder and scan with 'update."
   (interactive "P")
-  (let (proto case inbox case:inbox mailbox case:folder)
-    (cond
-     ((mew-virtual-p)
-      (cond
-       ((setq case:folder (or (mew-vinfo-get-parent-folder) ;; Thread
-			      (car (mew-vinfo-get-flds)))) ;; Virtual by "/"
-	(setq proto (mew-case:folder-folder case:folder))
-	(setq case (or (mew-case:folder-case case:folder)
-                       mew-case)))
-       (t ;; Virtual by "k/"
-	(setq proto (mew-proto mew-case))
-	(setq case mew-case))))
-     (t
-      (setq proto (mew-sinfo-get-folder)) ;; folder is proto
-      (setq case (or (mew-sinfo-get-case) ;; remote
-		     mew-case)))) ;; local
+  (let (case proto inbox case:inbox mailbox)
+    (mew-set '(case proto) (mew-summary-case-proto))
     (setq inbox (mew-proto-inbox-folder proto case))
     (cond
      ((mew-folder-remotep proto)
@@ -572,19 +589,15 @@ In remote folders, visit an inbox folder and scan with 'update."
 	(setq uid (mew-header-get-value mew-x-mew-uidl:))))
     (mew-scan-uid-case uid)))
 
-(defun mew-net-thread-info (msgs)
-  (let ((virtual (mew-summary-folder-name 'ext))
-	(summary (mew-summary-folder-name)))
-    (list virtual summary msgs)))
+(mew-defstruct net-virtual-info virtual summary msgs threadp)
 
-(defun mew-net-thread-info-get-virtual (thread-info)
-  (nth 0 thread-info))
+(defun mew-net-virtual-info (msgs)
+  (mew-make-net-virtual-info :virtual (mew-summary-folder-name 'ext)
+			     :summary (mew-summary-folder-name)
+			     :msgs    msgs
+			     :threadp (mew-thread-p)))
 
-(defun mew-net-thread-info-get-summary (thread-info)
-  (nth 1 thread-info))
-
-(defun mew-net-thread-info-get-msg (thread-info)
-  (nth 2 thread-info))
+(mew-defstruct net-disp-info list fid buf fld msg)
 
 (defun mew-net-disp-info ()
   (let* ((fid (mew-frame-id))
@@ -592,26 +605,14 @@ In remote folders, visit an inbox folder and scan with 'update."
 	 (fld (mew-current-get-fld fid))
 	 (msg (mew-current-get-msg fid))
 	 (disp (mew-sinfo-get-disp-msg)))
-    (if disp (list fid buf fld msg))))
-
-(defun mew-net-disp-info-fid (disp-info)
-  (nth 0 disp-info))
-
-(defun mew-net-disp-info-buf (disp-info)
-  (nth 1 disp-info))
-
-(defun mew-net-disp-info-fld (disp-info)
-  (nth 2 disp-info))
-
-(defun mew-net-disp-info-msg (disp-info)
-  (nth 3 disp-info))
+    (if disp (mew-make-net-disp-info :fid fid :buf buf :fld fld :msg msg))))
 
 (defun mew-net-disp-info-display (disp-info)
   (when disp-info
-    (let ((fid (mew-net-disp-info-fid disp-info))
-	  (buf (mew-net-disp-info-buf disp-info))
-	  (fld (mew-net-disp-info-fld disp-info))
-	  (msg (mew-net-disp-info-msg disp-info))
+    (let ((fid (mew-net-disp-info-get-fid disp-info))
+	  (buf (mew-net-disp-info-get-buf disp-info))
+	  (fld (mew-net-disp-info-get-fld disp-info))
+	  (msg (mew-net-disp-info-get-msg disp-info))
 	  (nfid (mew-frame-id))
 	  m)
       (when (and (equal fid nfid)
@@ -631,7 +632,7 @@ In remote folders, visit an inbox folder and scan with 'update."
   "Retrieve the rest of a truncated('T') message.
 
 In a LOCAL folder: a method to retrieve the message is determined
-by 'mew-case' and 'mew-mailbox-type'. 
+by 'mew-case' and 'mew-mailbox-type'.
 If 'mew-pop-delete'/'mew-imap-delete' is non-nil, delete the message
 from the mailbox. When executed with '\\[universal-argument]',
 'mew-pop-delete'/'mew-imap-delete' is considered reversed.
@@ -639,91 +640,112 @@ from the mailbox. When executed with '\\[universal-argument]',
 In a REMOTE folder: case and protocol are determined by the folder.
 The message in the server side is always retained."
   (interactive "P")
-  (mew-summary-msg
-   (when (mew-sumsyn-match mew-regex-sumsyn-long)
-     (let* ((msg (mew-sumsyn-message-number))
-	    (uid (mew-sumsyn-message-uid))
-	    (siz (mew-sumsyn-message-size))
-	    (bnm (mew-summary-folder-name))
-	    (disp-info (mew-net-disp-info))
-	    (thread-info (if (mew-thread-p) (mew-net-thread-info (list msg))))
-	    folder case del rtr rtrs mailbox)
-       (save-excursion
-	 (set-buffer bnm)
-	 (when (mew-summary-exclusive-p)
-	   (setq folder (mew-sinfo-get-folder))
-	   (setq case (mew-sinfo-get-case))
-	   (when (mew-folder-localp folder)
-	     (setq case (or (mew-summary-get-inherit-case folder msg) mew-case))
-	     (setq mailbox (mew-mailbox-type case))
-	     (cond
-	      ((eq mailbox 'pop)
-	       (setq del (eq (mew-pop-delete case) t))) ;; delete may be number
-	      ((eq mailbox 'imap)
-	       (setq del (eq (mew-imap-delete case) t)))) ;; delete may be number
-	     (if rev-del (setq del (not del))))
-	   (when (and uid (not (string= uid "")))
-	     (setq rtr (list uid siz del (list bnm msg)))
-	     (setq rtrs (list rtr)))
-	   (cond
-	    ((not rtrs)
-	     (message "No message to be retrieved"))
-	    ((not (mew-msg-validp msg))
-	     (message "No valid message to be retrieved"))
-	    ((mew-folder-localp folder)
-	     (cond
-	      ((eq mailbox 'pop)
-	       (mew-pop-retrieve case 'get bnm rtrs thread-info disp-info))
-	      ((eq mailbox 'imap)
-	       (mew-imap-retrieve case 'get bnm rtrs thread-info disp-info))))
-	    ((mew-folder-popp folder)
-	     (mew-pop-retrieve case 'get bnm rtrs thread-info disp-info))
-	    ((mew-folder-imapp folder)
-	     (mew-imap-retrieve case 'get bnm rtrs thread-info disp-info))
-	    ((mew-folder-nntpp folder)
-	     (mew-nntp-retrieve case 'get bnm rtrs thread-info disp-info)))))))))
+  (mew-pickable
+   (mew-summary-msg
+    (when (mew-sumsyn-match mew-regex-sumsyn-long)
+      (let* ((msg (mew-sumsyn-message-number))
+	     (uid (mew-sumsyn-message-uid))
+	     (siz (mew-sumsyn-message-size))
+	     (bnm (mew-summary-folder-name))
+	     (disp-info (mew-net-disp-info))
+	     (virtual-info (if (mew-virtual-for-one-summary)
+			       (mew-net-virtual-info (list msg))))
+	     folder case del rtr rtrs mailbox)
+	(save-excursion
+	  (set-buffer bnm)
+	  (when (mew-summary-exclusive-p)
+	    (setq folder (mew-sinfo-get-folder))
+	    (setq case (mew-sinfo-get-case))
+	    (when (mew-folder-localp folder)
+	      (setq case (or (mew-summary-get-inherit-case folder msg) mew-case))
+	      (setq mailbox (mew-mailbox-type case))
+	      (cond
+	       ((eq mailbox 'pop)
+		(setq del (eq (mew-pop-delete case) t))) ;; delete may be number
+	       ((eq mailbox 'imap)
+		(setq del (eq (mew-imap-delete case) t)))) ;; delete may be number
+	      (if rev-del (setq del (not del))))
+	    (when (and uid (not (string= uid "")))
+	      (setq rtr (mew-make-refileinfo :uid uid
+					     :size siz
+					     :delete del
+					     :folders (list bnm msg)))
+	      (setq rtrs (list rtr)))
+	    (cond
+	     ((not rtrs)
+	      (message "No message to be retrieved"))
+	     ((not (mew-msg-validp msg))
+	      (message "No valid message to be retrieved"))
+	     ((mew-folder-localp folder)
+	      (cond
+	       ((eq mailbox 'pop)
+		(mew-pop-retrieve case 'get bnm rtrs virtual-info disp-info))
+	       ((eq mailbox 'imap)
+		(mew-imap-retrieve case 'get bnm rtrs virtual-info disp-info))))
+	     ((mew-folder-popp folder)
+	      (mew-pop-retrieve case 'get bnm rtrs virtual-info disp-info))
+	     ((mew-folder-imapp folder)
+	      (mew-imap-retrieve case 'get bnm rtrs virtual-info disp-info))
+	     ((mew-folder-nntpp folder)
+	      (mew-nntp-retrieve case 'get bnm rtrs virtual-info disp-info))))))))))
 
-(defun mew-summary-retrieve-message-for-thread (thread-info)
-  (let ((virtual (mew-net-thread-info-get-virtual thread-info))
-	(summary (mew-net-thread-info-get-summary thread-info))
-	(msgs    (mew-net-thread-info-get-msg     thread-info))
-	msg str start beg end idt-str level length)
+(defun mew-summary-retrieve-message-for-virtual-summary (msg)
+  (save-excursion
+    (when (mew-summary-search-msg msg)
+      (let ((start (point)))
+	(forward-line)
+	(buffer-substring start (point))))))
+
+(defun mew-summary-retrieve-message-for-virtual-selection (msg str)
+  (save-excursion
+    (when (mew-summary-search-msg msg)
+      (let ((start (point)))
+	(forward-line)
+	(mew-elet
+	 (delete-region start (point))
+	 (insert str))))))
+
+(defun mew-summary-retrieve-message-for-virtual-thread (msg str)
+  (save-excursion
+    (when (mew-summary-search-msg msg)
+      (let* ((start (point))
+	     (beg (mew-thread-next-property start))
+	     (level (mew-thread-get-property beg))
+	     (length (* mew-thread-indent-length level))
+	     (end (+ beg length))
+	     (idt-str (buffer-substring beg end)))
+	(forward-line)
+	(mew-elet
+	 (delete-region start (point))
+	 (insert str)
+	 (goto-char beg)
+	 (if (= level 0) (setq end (1+ end)))
+	 (insert idt-str)
+	 (mew-thread-put-property beg end level))))))
+
+(defun mew-summary-retrieve-message-for-virtual (virtual-info)
+  (let ((virtual (mew-net-virtual-info-get-virtual virtual-info))
+	(summary (mew-net-virtual-info-get-summary virtual-info))
+	(msgs    (mew-net-virtual-info-get-msgs    virtual-info))
+	(func    (if (mew-net-virtual-info-get-threadp virtual-info)
+		     'mew-summary-retrieve-message-for-virtual-thread
+		   'mew-summary-retrieve-message-for-virtual-selection))
+	str)
     (save-excursion
-      (while msgs
-	(setq msg (car msgs))
-	(setq msgs (cdr msgs))
+      (dolist (msg msgs)
 	(set-buffer summary)
 	;; Summary
-	(save-excursion
-	  (when (mew-summary-search-msg msg)
-	    (setq start (point))
-	    (forward-line)
-	    (setq str (buffer-substring start (point)))))
+	(setq str (mew-summary-retrieve-message-for-virtual-summary msg))
 	(set-buffer virtual)
 	;; Virtual
-	(save-excursion
-	  (when (mew-summary-search-msg msg)
-	    (setq start (point))
-	    (setq beg (mew-thread-next-property start))
-	    (setq level (mew-thread-get-property beg))
-	    (setq length (* mew-thread-indent-length level))
-	    (setq end (+ beg length))
-	    (setq idt-str (buffer-substring beg end))
-	    (forward-line)
-	    (mew-elet
-	     (delete-region start (point))
-	     (insert str)
-	     (goto-char beg)
-	     (if (= level 0) (setq end (1+ end)))
-	     (insert idt-str)
-	     (mew-thread-put-property beg end level)))))
+	(funcall func msg str))
       (mew-summary-unlock))))
 
 (defun mew-summary-mark-retrieve-message (&optional rev-del)
   "Retrieve the rest of truncated('T') messages marked with '*'.
 
 In a LOCAL folder: a method to retrieve the messages is determined by
-'mew-case' and 'mew-mailbox-type'.  
+'mew-case' and 'mew-mailbox-type'.
 If 'mew-pop-delete'/'mew-imap-delete' is non-nil, delete the messages
 from the mailbox.  When executed with '\\[universal-argument]',
 'mew-pop-delete'/'mew-imap-delete' is considered reversed.
@@ -754,7 +776,10 @@ The messages in the server side is always retained."
 		    ((eq mailbox 'imap)
 		     (setq del (eq (mew-imap-delete case) t))))	;; delete may be number
 		   (if rev-del (setq del (not del)))
-		   (setq rtr (list uid siz del (list bnm msg)))
+		   (setq rtr (mew-make-refileinfo :uid uid
+						  :size siz
+						  :delete del
+						  :folders (list bnm msg)))
 		   (if (setq tmp (nth 2 (assoc case case-rtrs)))
 		       (nconc tmp (list rtr))
 		     (setq case-rtrs (cons (list case mailbox (list rtr)) case-rtrs))))
@@ -766,7 +791,10 @@ The messages in the server side is always retained."
 	   (setq msg (mew-sumsyn-message-number))
 	   (setq siz (mew-sumsyn-message-size))
 	   (when (and (mew-msg-validp msg) (mew-msg-truncatedp siz))
-	     (setq rtr (list uid siz del (list bnm msg)))
+	     (setq rtr (mew-make-refileinfo :uid uid
+					    :size siz
+					    :delete del
+					    :folders (list bnm msg)))
 	     (setq rtrs (cons rtr rtrs))))
 	   (forward-line))
 	 (setq rtrs (nreverse rtrs))))
@@ -774,7 +802,7 @@ The messages in the server side is always retained."
 	 (message "No message to be retrieved")
        (cond
 	((mew-folder-localp folder)
-	 (while case-rtrs
+	 (while case-rtrs ;; cannot use dolist
 	   (setq case (nth 0 (car case-rtrs)))
 	   (setq mailbox (nth 1 (car case-rtrs)))
 	   (setq rtrs (nth 2 (car case-rtrs)))
@@ -855,13 +883,13 @@ The messages in the server side is always retained."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2007 Mew developing team.
+;; Copyright (C) 2002-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -870,7 +898,7 @@ The messages in the server side is always retained."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
