@@ -30,7 +30,7 @@
     "size" "get-body" "no-msg" "case" "msgdb" "done" "dispatched" "error"
     "max"
     "newsgroup" "msgid" "truncated"
-    "thread-info" "disp-info" "status-buf"))
+    "virtual-info" "disp-info" "status-buf"))
 
 (mew-info-defun "mew-nntp-" mew-nntp-info-list)
 
@@ -51,13 +51,13 @@
     ("pre-quit"    nil (t        . "quit2"))
     ("quit"        nil ("205"    . "noop"))))
 
-(defsubst mew-nntp-fsm-by-status (status)
+(defun mew-nntp-fsm-by-status (status)
   (assoc status mew-nntp-fsm))
 
-(defsubst mew-nntp-fsm-next (status code)
+(defun mew-nntp-fsm-next (status code)
   (cdr (mew-assoc-match2 code (nthcdr 2 (mew-nntp-fsm-by-status status)) 0)))
 
-(defsubst mew-nntp-fsm-reply (status)
+(defun mew-nntp-fsm-reply (status)
   (nth 1 (mew-nntp-fsm-by-status status)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -129,7 +129,7 @@
 	      ;; reversed
 	      (setq max (car max))
 	      (setq max (1+ (string-to-number max))))
-	     ((and (eq directive 'scan) (numberp range))
+	     ((and (eq directive 'scan) (integerp range))
 	      (setq max (1+ (- last range)))
 	      (if (< max first) (setq max first)))
 	     (t
@@ -159,14 +159,13 @@
        ((eq directive 'scan)
 	(if (and uid (or range ;; all, last:n
 			 (> (string-to-number uid) max))) ;; update
-	    (setq rtrs (cons (list uid siz) rtrs))))
+	    (setq rtrs (cons (mew-make-refileinfo :uid uid :size siz) rtrs))))
        ((eq directive 'sync)
 	(if uid (setq hlds (cons uid hlds))))))
-    (mew-nntp-set-msgid pnm (nth 0 (car rtrs))) ;; 'scan
+    (mew-nntp-set-msgid pnm (mew-refileinfo-get-uid (car rtrs))) ;; 'scan
     ;; last:n xxx
-;;    (when (and (eq directive 'scan) (numberp range))
-;;      (if (> (length rtrs) range)
-;;	  (setcdr (nthcdr (1- range) rtrs) nil)))
+    ;;    (when (and (eq directive 'scan) (integerp range))
+    ;;      (mew-ntake range rtrs))
     (setq rtrs (nreverse rtrs))
     (setq hlds (nreverse hlds))
     (mew-nntp-command-dispatch pro pnm directive rtrs hlds)))
@@ -198,14 +197,14 @@
   (mew-net-status2 (mew-nntp-get-status-buf pnm)
 		   (mew-nntp-get-rttl pnm)
 		   (mew-nntp-get-rcnt pnm)
-		   (nth 1 (car (mew-nntp-get-rtrs pnm)))
+		   (mew-refileinfo-get-size (car (mew-nntp-get-rtrs pnm)))
 		   'zero
 		   (mew-nntp-secure-p pnm))
   (let* ((directive (mew-nntp-get-directive pnm))
 	 (rtrs (mew-nntp-get-rtrs pnm))
 	 (rtr (car rtrs))
-	 (uid (nth 0 rtr))
-	 (siz (nth 1 rtr))
+	 (uid (mew-refileinfo-get-uid rtr))
+	 (siz (mew-refileinfo-get-size rtr))
 	 (lim (mew-nntp-get-size pnm))
 	 (get-body (mew-nntp-get-get-body pnm)))
     (cond
@@ -231,9 +230,9 @@
 	 (width (1- (mew-scan-width)))
 	 (rtrs (mew-nntp-get-rtrs pnm))
 	 (rtr (car rtrs))
-	 (uid (nth 0 rtr))
-	 (siz (nth 1 rtr))
-	 (fld-msg (nth 3 rtr))
+	 (uid (mew-refileinfo-get-uid rtr))
+	 (siz (mew-refileinfo-get-size rtr))
+	 (fld-msg (mew-refileinfo-get-folders rtr))
 	 (truncated (mew-nntp-get-truncated pnm))
 	 fld msg vec file msg-file lmsg)
     (cond
@@ -252,7 +251,7 @@
     (mew-dot-delete)
     (cond
      ((eq directive 'scan)
-      (setq msg uid) 
+      (setq msg uid)
       (setq file (mew-expand-new-msg fld msg)))
      (t
       (setq msg-file (mew-net-get-new-message
@@ -265,9 +264,8 @@
     (catch 'write-error
       (condition-case nil
 	  (let ((write-region-inhibit-fsync mew-use-async-write))
-	    (mew-frwlet 
-	     mew-cs-dummy mew-cs-text-for-write
-	     (write-region (point-min) (point-max) file nil 'no-msg)))
+	    (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	      (write-region (point-min) (point-max) file nil 'no-msg)))
 	(error
 	 (mew-nntp-set-status pnm "quit")
 	 (mew-nntp-command-quit pro pnm)
@@ -311,7 +309,7 @@
     (while (not (eobp))
       (when (looking-at "\\([a-z][^ \t\n]+\\)")
 	(setq group2 (mew-match-string 1))
-	(setq group (concat mew-folder-nntp group2)) 
+	(setq group (concat mew-folder-nntp group2))
 	(setq groups (cons (mew-folder-func group) groups))
 	(setq groups2 (cons (mew-folder-func group2) groups2)))
       (forward-line))
@@ -343,19 +341,19 @@
 
 (defconst mew-nntp-info-prefix "mew-nntp-info-")
 
-(defsubst mew-nntp-info-name (case newsgroup)
+(defun mew-nntp-info-name (case newsgroup)
   (let ((server (mew-nntp-server case))
-	(port (mew-nntp-port case))
+	(port (mew-*-to-string (mew-nntp-port case)))
 	(sshsrv (mew-nntp-ssh-server case))
 	(name mew-nntp-info-prefix))
     (setq name (concat name server "/" newsgroup))
-    (unless (string= port mew-nntp-port)
+    (unless (mew-port-equal port mew-nntp-port)
       (setq name (concat name ":" port)))
     (if sshsrv
 	(concat name "%" sshsrv)
       name)))
 
-(defsubst mew-nntp-buffer-name (pnm)
+(defun mew-nntp-buffer-name (pnm)
   (concat mew-buffer-prefix pnm))
 
 (defun mew-nntp-process-send-string (pro &rest args)
@@ -371,10 +369,10 @@
 	(user (mew-nntp-get-user pnm)))
     (concat user "@" server ":" port)))
 
-(defsubst mew-nntp-message (pnm &rest args)
+(defun mew-nntp-message (pnm &rest args)
   (or (mew-nntp-get-no-msg pnm) (apply 'message args)))
 
-(defsubst mew-bnm-to-newsgroup (bnm)
+(defun mew-bnm-to-newsgroup (bnm)
   (mew-folder-string (mew-case:folder-folder bnm)))
 
 (defun mew-nntp-input-passwd (prompt pnm)
@@ -391,14 +389,14 @@
 ;;;
 
 (defun mew-nntp-open (pnm server port no-msg)
-  (let ((sprt (mew-port-sanity-check port))
+  (let ((sprt (mew-*-to-port port))
 	pro tm)
     (condition-case emsg
 	(progn
 	  (setq tm (run-at-time mew-nntp-timeout-time nil 'mew-nntp-timeout))
 	  (or no-msg (message "Connecting to the NNTP server..."))
 	  (setq pro (open-network-stream pnm nil server sprt))
-	  (process-kill-without-query pro)
+	  (mew-process-silent-exit pro)
 	  (mew-set-process-cs pro mew-cs-text-for-net mew-cs-text-for-net)
 	  (or no-msg (message "Connecting to the NNTP server...done")))
       (quit
@@ -421,7 +419,7 @@
 (defun mew-nntp-retrieve (case directive bnm &rest args)
   (let* ((server (mew-nntp-server case))
          (user (mew-nntp-user case))
-	 (port (mew-nntp-port case))
+	 (port (mew-*-to-string (mew-nntp-port case)))
 	 (sshsrv (mew-nntp-ssh-server case))
 	 (sslp (mew-nntp-ssl case))
 	 (sslport (mew-nntp-ssl-port case))
@@ -430,7 +428,7 @@
 	 (buf (get-buffer-create (mew-nntp-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
 	 process sshname sshpro sslname sslpro lport tls
-	 thread-info disp-info virtual)
+	 virtual-info disp-info virtual)
     (if (mew-nntp-get-process pnm)
 	(message "Another NNTP process is running. Try later")
       (cond
@@ -442,7 +440,7 @@
 	  (when lport
 	    (setq process (mew-nntp-open pnm "localhost" lport no-msg)))))
        (sslp
-	(if (string= port sslport) (setq tls mew-tls-nntp))
+	(if (mew-port-equal port sslport) (setq tls mew-tls-nntp))
 	(setq sslpro (mew-open-ssl-stream case server sslport tls))
 	(when sslpro
 	  (setq sslname (process-name sslpro))
@@ -454,6 +452,7 @@
       (when process
 	(mew-summary-lock process "NNTPing" (or sshpro sslpro))
 	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
+	(mew-sinfo-set-summary-column (mew-get-summary-column bnm))
 	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
 	(mew-sinfo-set-scan-md5 nil)
@@ -481,11 +480,11 @@
 	 ((eq directive 'get)
 	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
 	  (mew-nntp-set-refs pnm (nth 0 args))
-	  (setq thread-info (nth 1 args))
-	  (mew-nntp-set-thread-info pnm thread-info)
+	  (setq virtual-info (nth 1 args))
+	  (mew-nntp-set-virtual-info pnm virtual-info)
 	  (setq disp-info (nth 1 args))
 	  (mew-nntp-set-disp-info pnm disp-info)
-	  (setq virtual (mew-net-thread-info-get-virtual thread-info))
+	  (setq virtual (mew-net-virtual-info-get-virtual virtual-info))
 	  (when virtual
 	    (mew-nntp-set-status-buf pnm virtual)
 	    (save-excursion
@@ -533,7 +532,7 @@
        (mew-net-status2 (mew-nntp-get-status-buf pnm)
 			(mew-nntp-get-rttl pnm)
 			(mew-nntp-get-rcnt pnm)
-			(nth 1 (car (mew-nntp-get-rtrs pnm)))
+			(mew-refileinfo-get-size (car (mew-nntp-get-rtrs pnm)))
 			nil
 			(mew-nntp-secure-p pnm)))
      (cond
@@ -579,7 +578,7 @@
 	 (error (mew-nntp-get-error pnm))
 	 (file (mew-expand-file bnm mew-nntp-msgid-file))
 	 (buf (process-buffer process))
-	 (thread-info (mew-nntp-get-thread-info pnm))
+	 (virtual-info (mew-nntp-get-virtual-info pnm))
 	 (disp-info (mew-nntp-get-disp-info pnm)))
     (save-excursion
       (mew-nntp-debug "NNTP SENTINEL" event)
@@ -590,7 +589,7 @@
       (if (not done)
 	  (let* ((rtrs (mew-nntp-get-rtrs pnm))
 		 (lefts (length rtrs))
-		 (msgid (nth 0 (car rtrs)))
+		 (msgid (mew-refileinfo-get-uid (car rtrs)))
 		 recovered)
 	    (mew-nntp-message pnm "NNTP connection is lost")
 	    (when (mew-nntp-get-dispatched pnm)
@@ -605,9 +604,9 @@
 		 "%d message retrieved. %d messages are left due to an error"
 		 (- rttl lefts) lefts)
 		(mew-summary-folder-cache-save))))
-	(if thread-info (mew-summary-retrieve-message-for-thread thread-info))
+	(if virtual-info (mew-summary-retrieve-message-for-virtual virtual-info))
 	(cond
-	 (error 
+	 (error
 	  ;; retain the error message
 	  )
 	 ((eq directive 'list)
@@ -640,7 +639,7 @@
 	    (mew-lisp-save file msgid nil 'unlimit)
 	    (mew-summary-folder-cache-save))))))
       ;;
-      (and (functionp 'unix-sync) (unix-sync))
+      (and mew-use-async-write (functionp 'unix-sync) (unix-sync))
       (mew-net-status-clear (mew-nntp-get-status-buf pnm))
       (mew-info-clean-up pnm)
       (set-buffer-modified-p nil)
@@ -662,10 +661,14 @@
   (setq mew-nntp-folder-alist2 nil))
 
 (defun mew-nntp-folder-alist (&optional case)
-  (let ((ent (assoc (or case mew-case-default) mew-nntp-folder-alist)))
-    (if ent
+  (let ((ent (assoc (or case mew-case-default) mew-nntp-folder-alist))
+	alist)
+    (if (and ent (cdr ent))
 	(cdr ent)
-      (mew-nntp-folder-load case))))
+      (setq alist (mew-nntp-folder-load case))
+      (if alist
+	  alist
+	(list (mew-folder-func (mew-nntp-newsgroup case)))))))
 
 (defun mew-nntp-folder-alist2 (&optional case)
   (let ((ent (assoc (or case mew-case-default) mew-nntp-folder-alist2)))
@@ -704,24 +707,21 @@
 	      (delete (assoc (or case mew-case-default) mew-nntp-folder-alist2)
 		      mew-nntp-folder-alist2))))
 
-(defun mew-nntp-update ()
-  (let ((bnm (mew-summary-folder-name 'ext))
-	(case (mew-sinfo-get-case)))
-    (if mew-config-cases
-	(setq case (mew-input-case case "NNTP")))
+(defun mew-nntp-update (case)
+  (let ((bnm (mew-summary-folder-name 'ext)))
     (mew-nntp-retrieve case 'list bnm)))
 
 (provide 'mew-nntp)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2007 Mew developing team.
+;; Copyright (C) 1999-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -730,7 +730,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
