@@ -8,6 +8,12 @@
 
 (require 'mew)
 
+(defmacro mew-summary-refilable (&rest body)
+  `(mew-pickable
+    (mew-summary-not-in-draft
+     (mew-summary-local-or-imap
+      ,@body))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Variables
@@ -50,7 +56,7 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Guess functions 
+;;; Guess functions
 ;;;
 
 ;; We have two types of functions in mew-refile-guess-control,
@@ -68,23 +74,22 @@
 ;;
 
 (defun mew-refile-guess (&optional auto show-all)
-  (let ((funcs mew-refile-guess-control) ret guess info stop)
+  (let ((case-fold-search t)
+	(funcs mew-refile-guess-control) ret guess info stop)
     (catch 'last
-      (while funcs
-	(if (string-match "ctrl" (symbol-name (car funcs)))
+      (dolist (func funcs)
+	(if (string-match "ctrl" (symbol-name func))
 	    ;; func is control function
-	    (when (setq ret (funcall (car funcs) guess auto))
+	    (when (setq ret (funcall func guess auto))
 	      (setq stop t)
 	      (or show-all (throw 'last t)))
 	  ;; func is guess function
-	  (setq ret (funcall (car funcs))))
+	  (setq ret (funcall func)))
 	(unless (listp ret) (setq ret (list ret)))
-	(setq info (nconc info (list (cons (car funcs) ret))))
+	(setq info (nconc info (list (cons func ret))))
 	(unless stop
-	  (while ret
-	    (mew-addq guess (car ret))
-	    (setq ret (cdr ret))))
-	(setq funcs (cdr funcs))))
+	  (dolist (re ret)
+	    (mew-addq guess re)))))
     (setq guess (nreverse guess))
     (if (not mew-refile-ctrl-multi) (setq guess (list (car guess))))
     (cons guess info)))
@@ -114,10 +119,9 @@ mew-refile-guess-from-me-is-special."
 	     (mew-is-my-address mew-regex-my-address-list from))
 	(let ((addrs (mew-header-parse-address-list mew-refile-guess-key-list))
 	      ret adr)
-	  (while addrs
-	    (setq adr (funcall func (car addrs)))
-	    (mew-addq ret adr)
-	    (setq addrs (cdr addrs)))
+	  (dolist (addr addrs)
+	    (setq adr (funcall func addr))
+	    (mew-addq ret adr))
 	  (nreverse ret))
       (funcall func from))))
 
@@ -129,7 +133,7 @@ mew-refile-guess-from-me-is-special."
 (defvar mew-assoc-folder nil)
 
 (defun mew-folder-func (full &optional name)
-  (cons full name))
+  (if full (cons full name)))
 
 (defun mew-assoc-folder-setup ()
   (if mew-use-fast-refile
@@ -143,16 +147,15 @@ mew-refile-guess-from-me-is-special."
 	(node-regex (concat (regexp-quote sep) "$"))
 	name ret first)
     (catch 'loop
-      (while alist
- 	(setq ret (car alist))
+      (dolist (al alist)
+ 	(setq ret al)
  	(setq name (cdr ret))
  	(when (and (stringp name) (string= (downcase name) skey))
 	  (unless first
 	    (setq first ret))
  	  (unless (string-match node-regex (car ret))
 	    (throw 'loop nil)))
-	(setq ret nil)
- 	(setq alist (cdr alist))))
+	(setq ret nil)))
     (if mew-use-node-folder
 	(or ret first)
       ret)))
@@ -191,14 +194,13 @@ mew-refile-guess-from-me-is-special."
       (setq sep (mew-imap-separator case)))
      (t
       (setq sep mew-path-separator)))
-    (while to-cc
-      (setq ml-addr (mew-addrstr-parse-address (or (car to-cc) "")))
+    (dolist (tc to-cc)
+      (setq ml-addr (mew-addrstr-parse-address (or tc "")))
       (when ml-addr
 	(setq ml-name (mew-addrstr-extract-user ml-addr))
 	(setq ent (or (funcall mew-assoc-folder ml-addr alist sep)
 		      (funcall mew-assoc-folder ml-name alist sep)))
-	(mew-addq ret (nth 0 ent)))
-      (setq to-cc (cdr to-cc)))
+	(mew-addq ret (nth 0 ent))))
     (nreverse ret)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -215,26 +217,26 @@ mew-refile-guess-from-me-is-special."
 RULE is as follows:
     rule      ::= '<rule>
     <rule>    ::= ((<key> <alist>) (<key> <alist>) ... [<special>])
-    <alist>   ::= (<value> . <folder>|<rule>) (<value> . <folder>|<rule>) ...
-    <special> ::= (t . <folder>) | (nil . <folder>)
+    <alist>   ::= (<value> <folder>|<rule>) (<value> <folder>|<rule>) ...
+    <special> ::= (t <folder>) | (nil <folder>)
 
 There are two special <key>s: @samp{nil} and @samp{t}. @samp{nil} is
 used to specify <folder> to be returned when nothing is
 guessed. @samp{t} can specify <folder> to be returned in addition to
 guessed values."
   (let (key alist val f-or-r header ent ret)
-    (while rule
-      (setq key (car (car rule)))
-      (setq alist (cdr (car rule)))
+    (dolist (rl rule)
+      (setq key (car rl))
+      (setq alist (cdr rl)) ;; must not use mew-alist-get-value
       (cond
        ((eq key t)
-	(mew-addq ret alist))
+	(mew-addq ret (if (consp alist) (car alist) alist)))
        ((eq key nil)
-	(or ret (mew-addq ret alist)))
+	(or ret (mew-addq ret (if (consp alist) (car alist) alist))))
        ((setq header (mew-header-get-value key))
-	(while alist
-	  (setq val (car (car alist)))
-	  (setq f-or-r (cdr (car alist)))
+	(dolist (al alist)
+	  (setq val (mew-alist-get-key al))
+	  (setq f-or-r (mew-alist-get-value al))
 	  (setq ent nil)
 	  (when (and (stringp val) (string-match val header))
 	    (cond
@@ -244,17 +246,14 @@ guessed values."
 	      (setq ent (mew-refile-guess-by-alist1 f-or-r)))))
 	  (when ent
 	    (if (listp ent)
-		(while ent
-		  (mew-addq ret (car ent))
-		  (setq ent (cdr ent)))
-	      (mew-addq ret ent)))
-	  (setq alist (cdr alist)))))
-      (setq rule (cdr rule)))
+		(dolist (et ent)
+		  (mew-addq ret et))
+	      (mew-addq ret ent)))))))
     (nreverse ret)))
 
 (defun mew-refile-guess-by-alist2 (regex string template)
   ;; regex: "\\([^@]+\\)@\\([^.]+\\)\\.ad\\.jp"
-  ;; string: "admin@iij.ad.jp"
+  ;; string: "admin@example.org"
   ;; template: "+net/\\2/\\1"
   ;; -> "+net/iij/admin"
   (let (str strs match repl)
@@ -332,15 +331,13 @@ guessed values."
 
 (defun mew-refile-guess-by-newsgroups ()
   ;; Guess folders by the Newsgroups field with folder alist.
-  (let* ((newsgroups (mew-addrstr-parse-value-list2 
+  (let* ((newsgroups (mew-addrstr-parse-value-list2
 		     (mew-header-get-value mew-newsgroups:)))
 	 (proto mew-inherit-refile-proto)
 	 (case mew-inherit-refile-case)
 	 (alist (mew-proto-folder-alist proto case)) ;; local folders
-	 newsgroup ent ret)
-    (while newsgroups
-      (setq newsgroup (car newsgroups))
-      (setq newsgroups (cdr newsgroups))
+	 ent ret)
+    (dolist (newsgroup newsgroups)
       (setq ent (funcall mew-assoc-folder newsgroup alist mew-path-separator))
       (mew-addq ret (nth 0 ent)))
     (nreverse ret)))
@@ -403,14 +400,13 @@ guessed values."
       ;; or a folder was not in other people's opinion,
       ;; I accept it.
       (catch 'match
-	(while chosen
-	  (if (or (member (car chosen) mho)
+	(dolist (csn chosen)
+	  (if (or (member csn mho)
 		  (not (catch 'find
-		    (while oho
-		      (and (member (car chosen) (car oho)) (throw 'find t))
-		      (setq oho (cdr oho))))))
-	      (throw 'match (setq folder (car chosen))))
-	  (setq chosen (cdr chosen))))
+			 (while oho
+			   (and (member csn (car oho)) (throw 'find t))
+			   (setq oho (cdr oho))))))
+	      (throw 'match (setq folder csn)))))
       (setq mew-refile-msgid-alist
 	    (cons (list msgid folder "??")
 		  (delq (assoc msgid mew-refile-msgid-alist) ;; delq is right
@@ -422,7 +418,7 @@ guessed values."
 	 ;; 'my honest opinion' guessed by mew-refile-guess-by-from.
 	 (mho (nth 1 (assoc 'mew-refile-guess-by-from info)))
 	 folder to-cc)
-    ;; default leaning key X is the address derived from From: header, 
+    ;; default leaning key X is the address derived from From: header,
     ;; but only when
     ;;    mew-refile-guess-from-me-is-special is t
     ;;    X is my address.
@@ -437,13 +433,13 @@ guessed values."
 	(setq from (car to-cc)))
     (unless (or (or (null from) (null chosen)) (and mho (member mho chosen)))
       ;; I decide which folder is most important among the user chosen
-      ;; folders. 
+      ;; folders.
       (catch 'match
-	(while chosen
+	(dolist (csn chosen)
 	  ;; searching a folder anyone couldn't predict.
-	  (unless (mew-member* (car chosen) info)
-	    (throw 'match (setq folder (car chosen))))
-	  (setq chosen (cdr chosen))))
+	  (unless (mew-member* csn info)
+	    (throw 'match (setq folder csn)))))
+      (run-hooks 'mew-refile-guess-by-from-learn-hook)
       ;; If candidate was found, I memorize it.
       (when folder
 	(setq mew-refile-from-alist
@@ -520,7 +516,7 @@ Return t if exists or created. Otherwise, return nil."
   ;;   (func2 guess12 guess22...))
   (let ((proto mew-inherit-refile-proto)
 	(case mew-inherit-refile-case)
-	learn-info folders ret cands singlep lst-lst lst fld)
+	learn-info folders ret cands singlep lst-lst)
     (save-excursion
       (set-buffer buf)
       (setq learn-info (mew-refile-guess auto)))
@@ -529,12 +525,12 @@ Return t if exists or created. Otherwise, return nil."
 	(setq folders (car learn-info))
       (cond
        (cur-folders
-	;; add new folder 
+	;; add new folder
 	(setq cands cur-folders))
        ((nth 1 (car learn-info))
 	;; multi guess
 	(setq cands (car learn-info)))
-       (t 
+       (t
 	;; single guess
 	(setq singlep t)
 	(setq cands (list (nth 0 (car learn-info))))))
@@ -551,12 +547,8 @@ Return t if exists or created. Otherwise, return nil."
       (setq folders (mew-input-refile-folders cands singlep case proto)))
     ;; check folder existence.
     (setq lst-lst (mapcar (lambda (x) (mew-split x ?,)) folders))
-    (while lst-lst
-      (setq lst (car lst-lst))
-      (setq lst-lst (cdr lst-lst))
-      (while lst
-	(setq fld (car lst))
-	(setq lst (cdr lst))
+    (dolist (lst lst-lst)
+      (dolist (fld lst)
 	(if (and fld (not (member fld ret)) (mew-refile-folder-check fld))
 	    (setq ret (cons fld ret)))))
     (cons (nreverse ret) (cdr learn-info)))) ;; return value
@@ -571,17 +563,16 @@ Return t if exists or created. Otherwise, return nil."
 the current folder as a candidate in addition to guessed folders."
  (interactive)
  (mew-summary-msg-or-part
-  (mew-summary-not-in-draft
-   (mew-summary-local-or-imap
-    (mew-summary-refile-body
-     nil nil nil nil (mew-summary-folder-name))))))
+  (mew-summary-refilable
+   (mew-summary-refile-body
+    nil nil nil nil (mew-summary-folder-name)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Refile (aka move)
 ;;;
 
-(defsubst mew-refile-log-buffer (fld)
+(defun mew-refile-log-buffer (fld)
   (concat mew-buffer-prefix "refile " fld))
 
 (defun mew-summary-refile (&optional report)
@@ -592,9 +583,8 @@ message. If it marks newly, displays the next message. If executed
 with '\\[universal-argument]', it displays how the refile rules work in Message mode."
  (interactive "P")
  (mew-summary-msg-or-part
-  (mew-summary-not-in-draft
-   (mew-summary-local-or-imap
-    (if report (mew-summary-refile-report) (mew-summary-refile-body))))))
+  (mew-summary-refilable
+   (if report (mew-summary-refile-report) (mew-summary-refile-body)))))
 
 (defvar mew-override-body-open "<")
 (defvar mew-override-body-close "> ")
@@ -603,13 +593,13 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	  "[^>\r\n]*"
 	  (regexp-quote mew-override-body-close)))
 
-(defun mew-summary-refile-override-body (folders-str)
+(defun mew-summary-refile-override-body (folders-str &optional force)
   (let* ((open mew-override-body-open)
 	 (close mew-override-body-close)
 	 (len (+ (length folders-str) (length open) (length close))))
     (save-excursion
       (when (mew-summary-goto-body 'after)
-	(when (looking-at mew-regex-override-body)
+	(when (and (not force) (looking-at mew-regex-override-body))
 	  (mew-elet
 	   (delete-region (point) (match-end 0))))
 	(mew-elet
@@ -663,7 +653,7 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 
 (defun mew-summary-refile-body (&optional exp-flds auto no-msg no-mark exfld)
   (let ((pos (point))
-	fld msg folder folders cur-folders mark
+	fld msg folders cur-folders mark
 	buf learn-info tmp delbuf invalidp folders-str
 	mew-inherit-refile-proto mew-inherit-refile-case)
     (mew-summary-goto-message)
@@ -709,9 +699,7 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	  (quit (goto-char pos)))
 	(setq folders (car learn-info)))
       ;; we must prevent refiling a message to +queue
-      (while folders
-	(setq folder (car folders))
-	(setq folders (cdr folders))
+      (dolist (folder folders)
 	(unless (mew-folder-queuep folder)
 	  (setq tmp (cons folder tmp))))
       (setq folders (nreverse tmp))
@@ -726,8 +714,7 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	    (save-excursion
 	      (set-buffer fld)
 	      (save-excursion
-		(when (or (re-search-forward  (mew-regex-sumsyn-msg msg) nil t)
-			  (re-search-backward (mew-regex-sumsyn-msg msg) nil t))
+		(when (mew-summary-search-msg msg)
 		  (mew-refile-reset msg)
 		  (mew-refile-set msg folders)
 		  (if cur-folders (mew-summary-refile-unlog fld))
@@ -780,18 +767,16 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
        (insert (format "** Guess result: %s\n" (car guess)))
        ;; report status of customize variables.
        (insert "\n** Current Configurations:\n\n")
-       (while customize-var
-	 (insert (format "%-40s:  " (car customize-var)))
-	 (insert (format "%s\n"     (eval (car customize-var))))
-	 (setq customize-var (cdr customize-var)))
+       (dolist (cvar customize-var)
+	 (insert (format "%-40s:  " cvar))
+	 (insert (format "%s\n"     (eval cvar))))
        (insert "\n** Each function's opinion:\n\n")
        ;; report how each functions guessed.
        (setq guess (cdr guess))
-       (while guess
-	 (insert (format "%-32s  " (car (car guess))))
+       (dolist (gs guess)
+	 (insert (format "%-32s  " (car gs)))
 	 (insert (format "return: %s\n"
-			 (mapconcat 'identity (cdr (car guess)) ",")))
-	 (setq guess (cdr guess)))))
+			 (mapconcat 'identity (cdr gs) ","))))))
     (mew-summary-display-postscript 'no-hook)
     (select-window win)))
 
@@ -801,17 +786,16 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 ;;;
 
 (defun mew-summary-refile-again ()
-  "Put a refile mark on this message according to the previous 
+  "Put a refile mark on this message according to the previous
 refile folder."
   (interactive)
-  (mew-summary-not-in-draft
-   (mew-summary-local-or-imap
-    (save-excursion
-      (mew-summary-goto-message)
-      (let ((msg (mew-summary-message-number)))
-	(when (mew-msg-validp msg)
-	  (message "%s to %s" msg (mew-join ", " mew-refile-last-folder)))))
-    (mew-summary-refile-body mew-refile-last-folder))))
+  (mew-summary-refilable
+   (save-excursion
+     (mew-summary-goto-message)
+     (let ((msg (mew-summary-message-number)))
+       (when (mew-msg-validp msg)
+	 (message "%s to %s" msg (mew-join ", " mew-refile-last-folder)))))
+   (mew-summary-refile-body mew-refile-last-folder)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -819,48 +803,47 @@ refile folder."
 ;;;
 
 (defun mew-summary-auto-refile (&optional mew-mark-review-only)
-  "Refile each message in the folder automatically. If 
+  "Refile each message in the folder automatically. If
 'mew-refile-auto-refile-skip-any-mark' is non-nil,
 any previously marked message will be skipped.
 If '\\[universal-argument]' is specified, only messages marked with
 'mew-mark-review' will be concerned."
   (interactive "P")
-  (mew-pickable
-   (mew-summary-not-in-draft
-    (mew-summary-local-or-imap
-     (mew-decode-syntax-delete)
-     (let ((mew-use-highlight-x-face nil)
-	   (lines (mew-sinfo-get-ttl-line))
-	   (line 1) (mark nil) msg)
-       (cond
-	(mew-mark-review-only
-	 (setq msg (format "Refile all messages marked with '%c'? "
-			   mew-mark-review)))
-	(mew-refile-auto-refile-skip-any-mark
-	 (setq msg "Refile all non-marked messages? "))
-	(t
-	 (setq msg "Refile messages including marked with weak marks?")))
-       (if (and mew-refile-auto-refile-confirm (not (y-or-n-p msg)))
-	   (message "Not refiled")
-	 (message "Auto refiling...")
-	 (save-window-excursion
-	   (goto-char (point-min))
-	   (while (re-search-forward mew-regex-sumsyn-valid nil t)
-	     (setq mark (mew-summary-get-mark))
-	     (if mew-mark-review-only
-		 (and mark
-		      (char-equal mark mew-mark-review)
-		      (mew-summary-refile-body nil t 'no-msg))
-	       (or (and mark
-			(or mew-refile-auto-refile-skip-any-mark
-			    (>= (mew-markdb-level mark)
-				(mew-markdb-level mew-mark-refile))))
-		   (mew-summary-refile-body nil t 'no-msg)))
-	     (if (= (% (/ (* 100 line) lines) 10) 0)
-		 (message "Auto refiling...%s%%" (/ (* 100 line) lines)))
-	     (setq line (1+ line))
-	     (forward-line)))
-	 (message "Auto refiling...done")))))))
+  (mew-summary-refilable
+   (mew-decode-syntax-delete)
+   (let ((mew-use-highlight-x-face nil)
+	 (lines (mew-sinfo-get-ttl-line))
+	 (case-fold-search nil)
+	 (line 1) (mark nil) msg)
+     (cond
+      (mew-mark-review-only
+       (setq msg (format "Refile all messages marked with '%c'? "
+			 mew-mark-review)))
+      (mew-refile-auto-refile-skip-any-mark
+       (setq msg "Refile all non-marked messages? "))
+      (t
+       (setq msg "Refile messages including marked with weak marks?")))
+     (if (and mew-refile-auto-refile-confirm (not (y-or-n-p msg)))
+	 (message "Not refiled")
+       (message "Auto refiling...")
+       (save-window-excursion
+	 (goto-char (point-min))
+	 (while (re-search-forward mew-regex-sumsyn-valid nil t)
+	   (setq mark (mew-summary-get-mark))
+	   (if mew-mark-review-only
+	       (and mark
+		    (char-equal mark mew-mark-review)
+		    (mew-summary-refile-body nil t 'no-msg))
+	     (or (and mark
+		      (or mew-refile-auto-refile-skip-any-mark
+			  (>= (mew-markdb-level mark)
+			      (mew-markdb-level mew-mark-refile))))
+		 (mew-summary-refile-body nil t 'no-msg)))
+	   (if (= (% (/ (* 100 line) lines) 10) 0)
+	       (message "Auto refiling...%s%%" (/ (* 100 line) lines)))
+	   (setq line (1+ line))
+	   (forward-line)))
+       (message "Auto refiling...done")))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -869,57 +852,55 @@ If '\\[universal-argument]' is specified, only messages marked with
 
 (defun mew-summary-mark-copy ()
   "\\<mew-summary-mode-map>
-Put the refile mark onto all messages marked with '*', 
+Put the refile mark onto all messages marked with '*',
 with the current folder as a candidate in addition to guessed folders.
-This is very convenient to refile all messages  picked by '\\[mew-summary-pick-mark]' and \"\\[mew-summary-grep-mark]\"."
+This is very convenient to refile all messages  picked by '\\[mew-summary-pick]'."
   (interactive)
-  (mew-summary-not-in-draft
-   (mew-summary-local-or-imap
-    (let ((mew-use-highlight-x-face nil)
-	  last)
-      (message "Mark copying...")
-      (save-excursion
-	(save-window-excursion
-	  (goto-char (point-min))
-	  (catch 'loop
-	    (while (re-search-forward mew-regex-msg-review nil t)
-	      (setq last (mew-summary-refile-body
-			  last nil 'no-msg nil (mew-summary-folder-name 'ext)))
-	      (unless last (throw 'loop t))
-	      (forward-line)))
-	  (message "Mark copying...done")))))))
+  (mew-summary-refilable
+   (let ((mew-use-highlight-x-face nil)
+	 last)
+     (message "Mark copying...")
+     (save-excursion
+       (save-window-excursion
+	 (goto-char (point-min))
+	 (catch 'loop
+	   (while (re-search-forward mew-regex-msg-review nil t)
+	     (setq last (mew-summary-refile-body
+			 last nil 'no-msg nil (mew-summary-folder-name 'ext)))
+	     (unless last (throw 'loop t))
+	     (forward-line)))
+	 (message "Mark copying...done"))))))
 
 (defun mew-summary-mark-refile ()
   "\\<mew-summary-mode-map>
 Put the refile mark onto all messages marked with '*'.
-This is very convenient to refile all messages picked by '\\[mew-summary-pick-mark]' and \"\\[mew-summary-grep-mark]\"."
+This is very convenient to refile all messages picked by '\\[mew-summary-pick]'."
   (interactive)
-  (mew-summary-not-in-draft
-   (mew-summary-local-or-imap
-    (let ((mew-use-highlight-x-face nil)
-	  last)
-      (message "Mark refiling...")
-      (save-excursion
-	(save-window-excursion
-	  (catch 'loop
-	    (while (or (re-search-forward  mew-regex-msg-review nil t)
-                       (re-search-backward mew-regex-msg-review nil t))
-	      (setq last (mew-summary-refile-body last nil 'no-msg))
-	      (unless last (throw 'loop t))
-	      (forward-line)))
-	  (message "Mark refiling...done")))))))
+  (mew-summary-refilable
+   (let ((mew-use-highlight-x-face nil)
+	 last)
+     (message "Mark refiling...")
+     (save-excursion
+       (save-window-excursion
+	 (catch 'loop
+	   (while (or (re-search-forward  mew-regex-msg-review nil t)
+		      (re-search-backward mew-regex-msg-review nil t))
+	     (setq last (mew-summary-refile-body last nil 'no-msg))
+	     (unless last (throw 'loop t))
+	     (forward-line)))
+	 (message "Mark refiling...done"))))))
 
 (provide 'mew-refile)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994-2005 Mew developing team.
+;; Copyright (C) 1994-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -928,7 +909,7 @@ This is very convenient to refile all messages picked by '\\[mew-summary-pick-ma
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
