@@ -20,7 +20,7 @@
     ;; parameters used internally
     "server" "port" "ssh-server"
     "user" "auth-user" "auth-list"
-    "helo-domain" 
+    "helo-domain"
     "status" "process" "ssh-process" "ssl-process"
     "qfld" "messages"
     ;; parameters used internally and should be initialized
@@ -59,10 +59,10 @@
     ("content"       ("250" . "done"))
     ("quit"          (t     . "noop"))))
 
-(defsubst mew-smtp-fsm-by-status (status)
+(defun mew-smtp-fsm-by-status (status)
   (assoc status mew-smtp-fsm))
 
-(defsubst mew-smtp-fsm-next (status code)
+(defun mew-smtp-fsm-next (status code)
   (cdr (mew-assoc-match2 code (cdr (mew-smtp-fsm-by-status status)) 0)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -249,7 +249,7 @@
     ("PLAIN"    mew-smtp-command-auth-plain)     ;; RFC 2595
     ("LOGIN"    mew-smtp-command-auth-login)))   ;; No spec
 
-(defsubst mew-smtp-auth-get-func (auth)
+(defun mew-smtp-auth-get-func (auth)
   (nth 1 (mew-assoc-case-equal auth mew-smtp-auth-alist 0)))
 
 ;;
@@ -334,9 +334,9 @@
 
 (defconst mew-smtp-info-prefix "mew-smtp-info-")
 
-(defsubst mew-smtp-info-name (case &optional fallbacked)
+(defun mew-smtp-info-name (case &optional fallbacked)
   (let ((server (mew-smtp-server case))
-	(port (mew-smtp-port case))
+	(port (mew-*-to-string (mew-smtp-port case)))
 	(user (mew-smtp-user-only case))
 	(sshsrv (mew-smtp-ssh-server case))
 	(name mew-smtp-info-prefix))
@@ -348,7 +348,7 @@
 	       (fboundp 'make-network-process)
 	       (string= port "smtp"))
       (setq port "submission"))
-    (unless (string= port mew-smtp-port)
+    (unless (mew-port-equal port mew-smtp-port)
       (setq name (concat name ":" port)))
     (if sshsrv
 	(concat name "%" sshsrv)
@@ -404,17 +404,17 @@
   (defalias 'mew-open-network-stream 'open-network-stream))
 
 (defun mew-smtp-open (pnm server port)
-  (let ((sprt (mew-port-sanity-check port))
+  (let ((sprt (mew-*-to-port port))
 	pro tm)
     ;; xxx some OSes do not define "submission", sigh.
     (when (and (stringp sprt) (string= sprt "submission"))
-      (setq sprt (string-to-int (mew-serv-to-port sprt))))
+      (setq sprt (mew-serv-to-port sprt)))
     (condition-case emsg
 	(progn
 	  (setq tm (run-at-time mew-smtp-timeout-time nil 'mew-smtp-timeout))
 	  (message "Connecting to the SMTP server...")
 	  (setq pro (mew-open-network-stream pnm nil server sprt))
-	  (process-kill-without-query pro)
+	  (mew-process-silent-exit pro)
 	  (mew-set-process-cs pro mew-cs-text-for-net mew-cs-text-for-net)
 	  (message "Connecting to the SMTP server...done"))
       (quit
@@ -437,14 +437,14 @@
 (defun mew-smtp-send-message (case qfld msgs &optional fallbacked)
   (let ((server (mew-smtp-server case))
         (user (mew-smtp-user-only case))
-	(port (mew-smtp-port case))
+	(port (mew-*-to-string (mew-smtp-port case)))
 	(pnm (mew-smtp-info-name case fallbacked))
 	(sshsrv (mew-smtp-ssh-server case))
 	(sslp (mew-smtp-ssl case))
 	(sslport (mew-smtp-ssl-port case))
 	mew-inherit-submission
 	process sshname sshpro sslname sslpro lport tlsp tls fallback)
-    (when (and sslp (string= port sslport))
+    (when (and sslp (mew-port-equal port sslport))
       (setq tlsp t)
       ;; let stunnel know that a wrapper protocol is SMTP
       (setq tls mew-tls-smtp))
@@ -454,14 +454,14 @@
 	       (not fallbacked)
 	       mew-use-submission
 	       (fboundp 'make-network-process) ;; Emacs 22 or later
-	       (string= port "smtp"))
+	       (mew-port-equal port "smtp"))
       (setq port "submission")
       (setq fallback t)
       (unless tlsp
 	;; TLS uses stunnel. So, we should not use non-blocking connect.
 	;; Timeout should be carried out by stunnel.
 	(setq mew-inherit-submission t))
-      (if (string= sslport "smtp") (setq sslport "submission")))
+      (if (mew-port-equal sslport "smtp") (setq sslport "submission")))
     (cond
      (sshsrv
       (setq sshpro (mew-open-ssh-stream case server port sshsrv))
@@ -490,7 +490,7 @@
 		   (not fallbacked)
 		   mew-use-submission
 		   (fboundp 'make-network-process)
-		   (string= port "submission"))
+		   (mew-port-equal port "submission"))
 	      (progn
 		;; make-network-process with :nowait t sometime
 		;; returns nil, why?
@@ -530,7 +530,7 @@
 	(run-hooks 'mew-smtp-flush-hook)
 	(message "Flushing %s..." qfld)
 	(mew-smtp-send-message case qfld msgs)))))
-  
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Filter and sentinel
@@ -623,7 +623,7 @@
 	  ;; A message file is not inserted at the beginning of the SMTP
 	  ;; session.
 	  (set-buffer buf)
-	  (mew-smtp-queue case error))
+	  (mew-smtp-queue case error pnm))
 	(mew-smtp-log pnm error))
        (done
 	(message "Sending in background...done"))
@@ -631,7 +631,7 @@
 	(if (null buf)
 	    (message "SMTP connection is lost")
 	  (set-buffer buf)
-	  (mew-smtp-queue case "SMTP connection is lost"))))
+	  (mew-smtp-queue case "SMTP connection is lost" pnm))))
       (mew-info-clean-up pnm)
       (if (and (processp sshpro) (not mew-ssh-keep-connection))
 	  (process-send-string sshpro "exit\n"))
@@ -646,9 +646,9 @@
 ;;; Queuing
 ;;;
 
-(defun mew-smtp-queue (case err)
+(defun mew-smtp-queue (case err &optional apnm)
   ;; Must be in a buffer where a message is contained.
-  (let* ((pnm (mew-smtp-info-name case))
+  (let* ((pnm (or apnm (mew-smtp-info-name case)))
 	 (qfld (mew-queue-folder case))
 	 (oname (buffer-name))
 	 (work (buffer-file-name))
@@ -663,11 +663,9 @@
     ;;
     (mew-smtp-set-recipients pnm (mew-smtp-get-orig-recipients pnm))
     (let* ((n mew-smtp-info-list-save-length)
-	   (data (make-vector n nil))
-	   (i 0))
-      (while (< i n)
-	(aset data i (aref (mew-info pnm) i))
-	(setq i (1+ i)))
+	   (data (make-vector n nil)))
+      (dotimes (i n)
+	(aset data i (aref (mew-info pnm) i)))
       (mew-lisp-save info data))
     ;;
     (message "%s has been queued to %s (%s)" oname nname err)
@@ -683,7 +681,7 @@
   (let* ((dir (file-name-directory back))
 	 (msg (file-name-nondirectory back))
 	 (case (mew-smtp-get-case pnm))
-	 msgid-logtime)
+	 msgid logtime)
     (mew-elet
      (mew-erase-buffer)
      (mew-set-buffer-multibyte t)
@@ -691,9 +689,9 @@
      (mew-smtp-set-orig-recipients pnm (mew-smtp-get-bcc pnm))
      (mew-smtp-set-bcc pnm nil)
      ;;
-     (setq msgid-logtime (mew-encode-id-date pnm (mew-smtp-message-id case))) ;; save-excursion
-     (mew-smtp-set-msgid pnm (nth 0 msgid-logtime))
-     (mew-smtp-set-logtime pnm (nth 1 msgid-logtime))
+     (mew-set '(msgid logtime) (mew-encode-id-date pnm (mew-smtp-message-id case))) ;; save-excursion
+     (mew-smtp-set-msgid pnm msgid)
+     (mew-smtp-set-logtime pnm logtime)
      (goto-char (point-max))
      (mew-draft-header-insert mew-to: "Bcc-Receiver:;")
      (mew-draft-header-insert mew-subj: mew-bcc-subject)
@@ -758,13 +756,13 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2007 Mew developing team.
+;; Copyright (C) 1999-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -773,7 +771,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
