@@ -19,9 +19,6 @@
       (mew-elet
        (goto-char (point-max))
        (unless (bolp) (insert "\n"))
-       (if (and mew-xemacs-p (extent-at (point) nil nil nil 'at))
-	   ;; to skip graphics
-	   (insert "\n"))
        (mew-message-clear-end-of)
        (when (or mew-end-of-message-string mew-end-of-part-string)
 	 (move-overlay (mew-minfo-get-eom) (point-max) (point-max))
@@ -31,7 +28,7 @@
 	       (mew-message-set-end-of-part))
 	   (mew-message-set-end-of-message)))))))
 
-(defsubst mew-message-clear-end-of ()
+(defun mew-message-clear-end-of ()
   (unless (overlayp (mew-minfo-get-eom))
     (mew-minfo-set-eom (mew-overlay-make (point-max) (point-max))))
   (mew-message-set-end-of-nil))
@@ -98,8 +95,8 @@ If nodisplay is non-nil, displays the cached message."
 ;;; After step
 ;;;
 
-(defsubst mew-summary-display-after (direction)
-  (cond 
+(defun mew-summary-display-after (direction)
+  (cond
    ((eq direction 'down)
     (mew-summary-display-down))
    ((eq direction 'up)
@@ -119,7 +116,7 @@ If nodisplay is non-nil, displays the cached message."
   (if (or mew-summary-recenter-p (interactive-p))
       (recenter (/ (- (window-height) 2) 2))))
 
-(defsubst mew-summary-display-preamble ()
+(defun mew-summary-display-preamble ()
   ;; message buffer
   (mew-erase-buffer)
   (mew-message-clear-end-of)
@@ -130,7 +127,7 @@ If nodisplay is non-nil, displays the cached message."
 
 (defvar mew-message-last-buffer nil)
 
-(defsubst mew-summary-display-postscript (&optional no-hook)
+(defun mew-summary-display-postscript (&optional no-hook)
   ;; message buffer
   (unless no-hook (run-hooks 'mew-message-hook))
   (mew-minfo-set-reob nil)
@@ -156,7 +153,7 @@ If nodisplay is non-nil, displays the cached message."
 ;;; Displaying
 ;;;
 
-(defsubst mew-summary-message-toobig (fld msg)
+(defun mew-summary-message-toobig (fld msg)
   (let ((file (mew-expand-msg fld msg)))
     (and (file-readable-p file)
 	 (> (mew-file-get-size file) mew-file-max-size))))
@@ -349,7 +346,7 @@ This function does not create a cache."
 ;;;
 
 (defun mew-summary-analyze-again-alternative (&optional arg)
-  "This command analyzes the message again with 
+  "This command analyzes the message again with
 'mew-use-alternative' and 'mew-use-text-body' reversed."
   (interactive "P")
   (mew-summary-goto-message)
@@ -434,7 +431,7 @@ displays it."
   "Insert this message or part into Message mode in the raw format.
 This command treats PGP/MIME transparent. That is an encrypted/signed
 part is displayed as a decrypted/verified part. Also, if this command
-is used for a message whose entire body is encrypted/signed, 
+is used for a message whose entire body is encrypted/signed,
 a decrypted/verified body is displayed."
   (interactive)
   (mew-summary-msg-or-part
@@ -466,7 +463,7 @@ a decrypted/verified body is displayed."
 	   ;; message buffer
 	   (mew-current-set fld msg nums)
 	   (mew-minfo-set-summary vfld)
-	   ;; The following codes are not necessary. 
+	   ;; The following codes are not necessary.
 	   ;; (mew-decode-syntax-clear)
 	   ;; (mew-decode-syntax-delete)
 	   (mew-elet
@@ -506,11 +503,10 @@ If called with '\\[universal-argument]', it stays writable."
 	  (file (mew-expand-msg fld msg)))
      (when (mew-sinfo-get-disp-msg)
        (mew-summary-toggle-disp-msg))
-     (mew-frwlet
-      (if (mew-folder-draftp fld) mew-cs-m17n mew-cs-autoconv) mew-cs-dummy
-      (if arg
-	  (switch-to-buffer (mew-find-file-noselect2 file))
-	(view-buffer (mew-find-file-noselect2 file) 'kill-buffer))))))
+     (mew-frwlet (if (mew-folder-draftp fld) mew-cs-m17n mew-cs-autoconv) mew-cs-dummy
+       (if arg
+	   (switch-to-buffer (mew-find-file-noselect2 file))
+	 (view-buffer (mew-find-file-noselect2 file) 'kill-buffer))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -524,7 +520,7 @@ If called with '\\[universal-argument]', it stays writable."
    (let* ((win (selected-window))
 	  (fld (mew-summary-folder-name))
 	  (msg (mew-summary-message-number2))
-	  lst ent svr-date (snd "?") (rcv "?") last svr date tmp)
+	  lst svr-date (snd "?") (rcv "?") last svr date tmp)
      (mew-summary-set-message-buffer fld msg)
      ;; message buffer or cache buffer
      (setq lst (nreverse (mew-header-get-value mew-received: 'as-list)))
@@ -532,9 +528,7 @@ If called with '\\[universal-argument]', it stays writable."
      ;; message buffer
      (mew-summary-display-preamble)
      (save-excursion
-       (while lst
-	 (setq ent (car lst))
-	 (setq lst (cdr lst))
+       (dolist (ent lst)
 	 (setq svr-date (mew-split ent ?\;))
 	 (mew-set '(svr date) svr-date)
 	 (unless date
@@ -612,7 +606,7 @@ ad-hoc solution for putting the review mark on a thread.")
 
 (defun mew-summary-down ()
   (funcall mew-summary-down-function)
-  (cond 
+  (cond
    ((mew-re-search-forward-visibile mew-regex-msg-or-part)
     (mew-thread-move-cursor)
     (mew-sinfo-set-direction 'down)
@@ -630,12 +624,12 @@ ad-hoc solution for putting the review mark on a thread.")
 
 (defun mew-summary-up ()
   (beginning-of-line)
-  (cond 
+  (cond
    ((mew-re-search-backward-visible mew-regex-msg-or-part)
     (mew-thread-move-cursor)
     (mew-sinfo-set-direction 'up)
     t)
-   (t 
+   (t
     (mew-thread-move-cursor)
     (mew-decode-syntax-delete)
     (mew-summary-reset)
@@ -651,7 +645,7 @@ ad-hoc solution for putting the review mark on a thread.")
   (if (mew-summary-next) (mew-summary-display)))
 
 (defun mew-summary-display-up (&optional arg)
-  "Move to above then display. Targets includes parts, messages 
+  "Move to above then display. Targets includes parts, messages
 marked with '*', and non-marked messages. When called with '\\[universal-argument]',
 parts are skipped."
   (interactive "P")
@@ -662,7 +656,7 @@ parts are skipped."
   (if (mew-summary-up) (mew-summary-display)))
 
 (defun mew-summary-display-down (&optional arg)
-  "Move to below then display. Targets includes parts, messages 
+  "Move to below then display. Targets includes parts, messages
 marked with '*', and non-marked messages. When called with '\\[universal-argument]',
 parts are skipped."
   (interactive "P")
@@ -742,8 +736,8 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
   (or arg (setq arg 1))
   (if (< arg 0)
       (mew-summary-previous-line (- arg))
-    (let (col) 
-      (if (eq last-command this-command) 
+    (let (col)
+      (if (eq last-command this-command)
 	  (setq col (mew-sinfo-get-column)))
       (unless col
 	(setq col (current-column))
@@ -762,8 +756,8 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
   (or arg (setq arg 1))
   (if (< arg 0)
       (mew-summary-next-line (- arg))
-    (let (col) 
-      (if (eq last-command this-command) 
+    (let (col)
+      (if (eq last-command this-command)
 	  (setq col (mew-sinfo-get-column)))
       (unless col
 	(setq col (current-column))
@@ -791,13 +785,13 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2007 Mew developing team.
+;; Copyright (C) 1996-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -806,7 +800,7 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
