@@ -133,8 +133,8 @@
   :type 'string)
 
 (defcustom mew-imap-trash-folder "%trash"
-  "*The trash folder for IMAP. If non-nil, deleted messages are 
-moved to this folder. Deleting messages in this folder makes them 
+  "*The trash folder for IMAP. If non-nil, deleted messages are
+moved to this folder. Deleting messages in this folder makes them
 really deleted."
   :group 'mew-folder
   :type '(choice string (const nil)))
@@ -190,17 +190,17 @@ really deleted."
   :type 'string)
 
 (defcustom mew-imap-spam-field nil
-  "*The field name of anti spam information to be used for IMAP spam filter. 
-If both 'mew-imap-spam-field' and 'mew-imap-spam-word' are defined, 
+  "*The field name of anti spam information to be used for IMAP spam filter.
+If both 'mew-imap-spam-field' and 'mew-imap-spam-word' are defined,
 messages whose the field contains the word are automatically
 removed or refiled to 'mew-imap-spam-folder' or 'mew-imap-trash-folder'
-(if defined) while scanning %inbox."
+\(if defined) while scanning %inbox."
   :group 'mew-imap
   :type '(choice (const :tag "Not use" nil)
 		 (string :tag "Spam field")))
 
 (defcustom mew-imap-spam-word nil
-  "*The word of anti spam information to be used for IMAP spam filter. 
+  "*The word of anti spam information to be used for IMAP spam filter.
 See 'mew-imap-spam-field'."
   :group 'mew-imap
   :type '(choice (const :tag "Not use" nil)
@@ -215,7 +215,7 @@ treated as 'nil'."
 		 (string :tag "Spam word")))
 
 (defcustom mew-imap-spam-folder nil
-  "*A folder to be used for IMAP spam filter. 
+  "*A folder to be used for IMAP spam filter.
 See 'mew-imap-spam-field'."
   :group 'mew-imap
   :type '(choice (const :tag "Not use" nil)
@@ -375,12 +375,12 @@ Note that this hook is called after composition of the message."
   :type 'hook)
 
 (defcustom mew-scan-sentinel-hook nil
-  "*Hook called when scan finished." 
+  "*Hook called when scan finished."
   :group 'mew-summary
   :type 'hook)
 
 (defcustom mew-summary-ls-no-scan-hook nil
-  "*Hook called when mew-summary-ls does not scan a folder." 
+  "*Hook called when mew-summary-ls does not scan a folder."
   :group 'mew-summary
   :type 'hook)
 
@@ -390,7 +390,7 @@ Note that this hook is called after composition of the message."
   :type 'hook)
 
 (defcustom mew-sort-hook nil
-  "*Hook called when sort finished." 
+  "*Hook called when sort finished."
   :group 'mew-summary
   :type 'hook)
 
@@ -399,6 +399,18 @@ Note that this hook is called after composition of the message."
   :group 'mew-summary
   :type 'hook)
 
+(defcustom mew-summary-delete-folder-hook nil
+  "*Hook called when a folder deleted.
+Eash function is called with a deleted folder as the argument."
+  :group 'mew-summary
+  :type 'hook)
+
+(defcustom mew-summary-rename-folder-hook nil
+  "*Hook called when a folder renamed.
+Each function is called with a source folder and a destination folder as the argument."
+  :group 'mew-summary
+  :type 'hook)
+
 (defcustom mew-syntax-format-hook nil
   "*Hook called when mew-syntax-format is called."
   :group 'mew-message
@@ -419,8 +431,18 @@ Note that this hook is called after composition of the message."
   :group 'mew-env
   :type 'hook)
 
+(defcustom mew-status-update-hook nil
+  "*Hook called to update status."
+  :group 'mew-env
+  :type 'hook)
+
+(defcustom mew-refile-guess-by-from-learn-hook nil
+  "*Hook called in mew-refile-guess-by-from-learn."
+  :group 'mew-env
+  :type 'hook)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; 
+;;;
 ;;; Mail Addresses for Draft
 ;;;
 
@@ -432,8 +454,8 @@ on a draft buffer. If nil, this value is automatically generated from
   :type '(choice (const nil) (repeat string)))
 
 (defcustom mew-mail-domain-list nil
-  "*Your e-mail address domain list like 
-(\"example.org\" \"example.jp\").
+  "*Your e-mail address domain list like
+\(\"example.org\" \"example.jp\").
 They are used for mail-domain completion in Draft mode(C-cTAB). If
 nil, this value is automatically generated from 'mew-config-alist'."
   :group 'mew-draft
@@ -467,12 +489,12 @@ also 'mew-from'."
   :type '(choice (const nil) (repeat string)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; 
+;;;
 ;;; Mail Addresses
 ;;;
 
 (defcustom mew-name (user-full-name)
-  "*Friendly name of e-mail address. 
+  "*Friendly name of e-mail address.
 i.e. \"Friendly name <user@mail-domain>\"."
   :group 'mew-basic
   :type '(choice string (const nil)))
@@ -554,8 +576,7 @@ server, set an appropriate value."
   :type 'string)
 
 (defcustom mew-smtp-port "smtp"
-  "*The SMTP port. If you want to set it to a numeric value,
-use a string. (e.g. \"25\")"
+  "*The SMTP port. (e.g. \"smtp\" or 25)"
   :group 'mew-smtp
   :type 'string)
 
@@ -570,14 +591,14 @@ use a string. (e.g. \"25\")"
   :group 'mew-smtp
   :type 'boolean)
 
-(defcustom mew-smtp-ssl-port "465" ;; no universal consensus
-  "*The port for SMTP over SSL/TLS. Set this to \"smtp\" if you 
+(defcustom mew-smtp-ssl-port 465 ;; no universal consensus
+  "*The port for SMTP over SSL/TLS. Set this to \"smtp\" if you
 want to use TLS."
   :group 'mew-smtp
   :type 'string)
 
 (defcustom mew-smtp-user nil
-  "*The user name on your SMTP server. If not configured, 
+  "*The user name on your SMTP server. If not configured,
 your e-mail address is automatically set"
   :group 'mew-smtp
   :type 'string)
@@ -645,7 +666,7 @@ For maildir, set this to '-u -d /path/to/mbox'."
 ;;;
 
 (defcustom mew-biff-interval 5
-  "*Minutes of biff interval. This should be smaller than 
+  "*Minutes of biff interval. This should be smaller than
 'mew-passwd-timer-unit' * 'mew-passwd-lifetime'."
   :group 'mew-net
   :type 'integer)
@@ -667,8 +688,7 @@ server, set an appropriate value."
   :type 'string)
 
 (defcustom mew-pop-port "pop3"
-  "*The POP3 port. If you want to set it to a numeric value,
-use a string. (e.g. \"110\")"
+  "*The POP3 port. (e.g. \"pop3\" or 110)"
   :group 'mew-pop
   :type 'string)
 
@@ -761,8 +781,7 @@ server, set an appropriate value."
   :type 'string)
 
 (defcustom mew-imap-port "imap"
-  "*The IMAP4 port. If you want to set it to a numeric value,
-use a string. (e.g. \"143\")"
+  "*The IMAP4 port. (e.g. \"imap\" or 143)"
   :group 'mew-imap
   :type 'string)
 
@@ -879,8 +898,7 @@ server, set an appropriate value."
   :type 'string)
 
 (defcustom mew-nntp-port "nntp"
-  "*The NNTP port. If you want to set it to a numeric value,
-use a string. (e.g. \"119\")"
+  "*The NNTP port. (e.g. \"nntp\" or 119)"
   :group 'mew-nntp
   :type 'string)
 
@@ -961,7 +979,7 @@ Candidate values for 'mew-rage-list-string-type' and
 'mew-range-list-list-type' are as follows:
 
 	'regex		Regular expression.
-	'recursive	Initial substring match. That is, 
+	'recursive	Initial substring match. That is,
 			not only a folder specified but all so
 			all sub-folders are matched.
 	'string		String comparison.
@@ -983,13 +1001,13 @@ If this value is nil, an appropriate value is set when Mew is booting.
 		 string))))
 
 (defcustom mew-range-list-string-type 'regex
-  "*A value to specify an action if the key is a string in 
+  "*A value to specify an action if the key is a string in
 'mew-range-list'. Candidates are 'regex, 'recursive, and 'string."
   :group 'mew-summary
   :type '(choice (const regex) (const recursive) (const string)))
 
 (defcustom mew-range-list-list-type 'recursive
-  "*A value to specify an action if the key is a list of strings in 
+  "*A value to specify an action if the key is a list of strings in
 'mew-range-list'. Candidates are 'regex, 'recursive, and 'string."
   :group 'mew-summary
   :type '(choice (const regex) (const recursive) (const string)))
@@ -1022,7 +1040,7 @@ Candidate values for 'mew-rage-list-string-type' and
 'mew-unread-mark-list-list-type' are as follows:
 
 	'regex		Regular expression.
-	'recursive	Initial substring match. That is, 
+	'recursive	Initial substring match. That is,
 			not only a folder specified but all so
 			all sub-folders are matched.
 	'string		String comparison.
@@ -1048,13 +1066,13 @@ is non-nil.
 		 string))))
 
 (defcustom mew-unread-mark-list-string-type 'regex
-  "*A value to specify an action if the key is a string in 
+  "*A value to specify an action if the key is a string in
 'mew-unread-mark-list'. Candidates are 'regex, 'recursive, and 'string."
   :group 'mew-summary
   :type '(choice (const regex) (const recursive) (const string)))
 
 (defcustom mew-unread-mark-list-list-type 'recursive
-  "*A value to specify an action if the key is a list of strings in 
+  "*A value to specify an action if the key is a list of strings in
 'mew-unread-mark-list'. Candidates are 'regex, 'recursive, and 'string."
   :group 'mew-summary
   :type '(choice (const regex) (const recursive) (const string)))
@@ -1067,7 +1085,7 @@ is non-nil.
 (defconst mew-summary-form-header '(mark)
   "*A list to be appended to summary-form if the first element of
 summary-form is not an integer. Since Mew assumes that
-each line of Summary mode begins with one mark, this variable 
+each line of Summary mode begins with one mark, this variable
 MUST be '(mark).")
 
 (defconst mew-custom-type-of-summary-form
@@ -1119,12 +1137,12 @@ MUST be '(mark).")
   "*The format in Summary mode, called summary-form. summary-form is a list
 of list, symbol, and string.
 
-(a) A string is printed as it is.
+\(a) A string is printed as it is.
 
-(b) A list consists of an integer and a symbol. The symbol specifies a
+\(b) A list consists of an integer and a symbol. The symbol specifies a
 function to be called. The name of the function is produced by
 concatenating 'mew-summary-form-func-prefix' and the symbol name.
-(e.g. 'mew-summary-form-date' for the symbol 'date'). 
+\(e.g. 'mew-summary-form-date' for the symbol 'date').
 
 Pre-defined symbols are 'type, 'time, 'date, 'year, 'size, 'from,
 'subj and 'body. For more details, see the explanation of the
@@ -1133,7 +1151,7 @@ functions called 'mew-summary-form-<symbol>'.
 The integer specifies the width of field which will be filled with a
 return string of the function.
 
-A positive value means padding SPCs on the right if necessary. 
+A positive value means padding SPCs on the right if necessary.
 
 A negative value means padding SPCs on the left if necessary. If a
 negative value is specified, the corresponding function must return an
@@ -1141,12 +1159,12 @@ ASCII string.
 
 0 means the remaining width and is treated as a positive value.
 
-(c) A symbol except 't' is equivalent to (1 symbol).
+\(c) A symbol except 't' is equivalent to (1 symbol).
 
-(d) The value of mew-summary-form-header (mark) is appended to
+\(d) The value of mew-summary-form-header (mark) is appended to
 summary-form when used. See also 'mew-summary-form-header'.
 
-(e) 't' means the position where thread indentation is inserted.
+\(e) 't' means the position where thread indentation is inserted.
 
 An example is as follows:
 	'(type (5 date) \" \" (-4 size) \" \" (14 from) \" \" t (30 subj) \"|\" (0 body))
@@ -1157,7 +1175,7 @@ You can also set this value in 'mew-summary-form-list'."
 
 (defcustom mew-summary-form-list nil
   "*A list to define 'mew-summary-form' for each folder. Each component
-is (key summary-form). 
+is (key summary-form).
 
 If KEY is t, all folders matches it. The corresponding SUMMARY-FORM is
 returned always.
@@ -1174,7 +1192,7 @@ Candidate values for 'mew-summary-form-list-string-type' and
 'mew-summary-form-list-list-type' are as follows:
 
 	'regex		Regular expression.
-	'recursive	Initial substring match. That is, 
+	'recursive	Initial substring match. That is,
 			not only a folder specified but all so
 			all sub-folders are matched.
 	'string		String comparison.
@@ -1199,13 +1217,13 @@ An example is as follows:
 			 (const nil))))))
 
 (defcustom mew-summary-form-list-string-type 'regex
-  "*A value to specify an action if the key is a string in 
+  "*A value to specify an action if the key is a string in
 'mew-summary-form-list'. Candidates are 'regex, 'recursive, and 'string."
   :group 'mew-summary
   :type '(choice (const regex) (const recursive) (const string)))
 
 (defcustom mew-summary-form-list-list-type 'recursive
-  "*A value to specify an action if the key is a list of strings in 
+  "*A value to specify an action if the key is a list of strings in
 'mew-summary-form-list'. Candidates are 'regex, 'recursive, and 'string."
   :group 'mew-summary
   :type '(choice (const regex) (const recursive) (const string)))
@@ -1269,7 +1287,7 @@ instead of the 'D' mark."
 
 (defcustom mew-summary-form-mark-spam nil
   "*If non-nil, the 'D' mark automatically is put onto
-SPAM messages. A message is considered a SPAM message 
+SPAM messages. A message is considered a SPAM message
 if the MD5 checksum of its body is duplicated in a scan.
 If a character, use it as a mark instead of the 'D' mark."
   :group 'mew-summary
@@ -1300,7 +1318,7 @@ quantization by 'mew-summary-form-size-unit'."
   :type 'boolean)
 
 (defcustom mew-spam: "X-Bogosity:"
-  "*The field name which your spam filter inserts. 
+  "*The field name which your spam filter inserts.
 E.g.
 	\"X-Spam-Flag:\" for 'spamassassin'
 	\"X-Bogosity:\" for 'bogofilter'"
@@ -1348,8 +1366,8 @@ only. Use 't' in 'mew-summary-form instead."
   :group 'mew-env
   :type 'boolean)
 
-(defcustom mew-demo-picture (or mew-xemacs-p mew-gemacs-p)
-  "*A picture of cats is displayed if *non-nil* on XEmacs."
+(defcustom mew-demo-picture t
+  "*A picture of cats is displayed if *non-nil*."
   :group 'mew-env
   :type 'boolean)
 
@@ -1375,8 +1393,8 @@ obtain any fields."
   :group 'mew-message
   :type 'integer)
 
-(defcustom mew-header-max-length 100
-  "*If the length of a header exceeds this value, 
+(defcustom mew-header-max-length 200
+  "*If the length of a header exceeds this value,
 it is not arranged nor MIME decoded.
 See also 'mew-header-max-depth'."
   :group 'mew-message
@@ -1437,7 +1455,7 @@ the 'D' mark onto different messages."
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Ask?
-;;; 
+;;;
 
 (defcustom mew-ask-subject nil
   "*If *non-nil* and Subject: is empty, ask a user its value."
@@ -1549,13 +1567,13 @@ Otherwise, windows are configured dynamically."
   :group 'mew-env
   :type 'boolean)
 
-(defcustom mew-use-text/enriched (= mew-mule-ver 3)
+(defcustom mew-use-text/enriched t
   "*If non-nil, Mew parses enriched format text messages."
   :group 'mew-message
   :type 'boolean)
 
 (defcustom mew-use-text/html nil
-  "*If non-nil, Mew parses text/html messages. 
+  "*If non-nil, Mew parses text/html messages.
 If you want to use 'mew-use-text/html-list', this value should be 'nil'."
   :group 'mew-message
   :type 'boolean)
@@ -1578,7 +1596,7 @@ Candidate values for 'mew-use-text/html-string-type' and
 'mew-use-text/html-list-type' are as follows:
 
 	'regex		Regular expression.
-	'recursive	Initial substring match. That is, 
+	'recursive	Initial substring match. That is,
 			not only a folder specified but all so
 			all sub-folders are matched.
 	'string		String comparison.
@@ -1593,13 +1611,13 @@ Candidate values for 'mew-use-text/html-string-type' and
 		 boolean))))
 
 (defcustom mew-use-text/html-string-type 'regex
-  "*A value to specify an action if the key is a string in 
+  "*A value to specify an action if the key is a string in
 'mew-use-text/html-list'. Candidates are 'regex, 'recursive, and 'string."
   :group 'mew-summary
   :type '(choice (const regex) (const recursive) (const string)))
 
 (defcustom mew-use-text/html-list-type 'recursive
-  "*A value to specify an action if the key is a list of strings in 
+  "*A value to specify an action if the key is a list of strings in
 'mew-use-text/html-list'. Candidates are 'regex, 'recursive, and 'string."
   :group 'mew-summary
   :type '(choice (const regex) (const recursive) (const string)))
@@ -1667,13 +1685,13 @@ If you want to use the biff functionality against a local mailbox, use
 
 (defcustom mew-use-biff-bell nil
   "*If non-nil, Mew beeps when the number of messages in
-your POP server becomes non-zero. To use this feature, 
+your POP server becomes non-zero. To use this feature,
 'mew-use-biff' should be 't'."
   :group 'mew-env
   :type 'boolean)
 
 (defcustom mew-use-sender nil
-  "*If From: contains multiple addresses, Mew adds Sender: anyway. 
+  "*If From: contains multiple addresses, Mew adds Sender: anyway.
 If From: contains one address and SMTP MAIL FROM
 \('mew-smtp-mail-from') is different from it, this variable effects.
 If non-nil, Mew adds Sender: and specifies SMTP MAIL FROM to its value.
@@ -1681,6 +1699,11 @@ Otherwise, Mew does not add Sender:."
   :group 'mew-draft
   :type 'boolean)
 
+(defcustom mew-use-x-mailer t
+  "*If non-nil, X-Mailer: is prepared in Draft mode"
+  :group 'mew-draft
+  :type 'boolean)
+
 (defcustom mew-use-alternative t
   "*If non-nil, Multipart/Alternative is treated as a singlepart
 according to 'mew-mime-multipart-alternative-list'."
@@ -1728,13 +1751,13 @@ are selected out of leaf folders only."
 (defcustom mew-use-case-completion t
   "*If non-nil, 'mew-case' is automatically completed when you
 input a folder.  For example, consider that you input a folder in a
-local folder after typing '\\<mew-summary-mode-map>\\[mew-summary-goto-folder]'.  
+local folder after typing '\\<mew-summary-mode-map>\\[mew-summary-goto-folder]'.
 Typing '%' automatically inserts 'mew-case' before '%'."
   :group 'mew-summary
   :type 'boolean)
 
 (defcustom mew-use-spc-padding t
-  "*If non-nil, SPCs are padded from the end of lines of Summary mode 
+  "*If non-nil, SPCs are padded from the end of lines of Summary mode
 to the right side."
   :group 'mew-summary
   :type 'boolean)
@@ -1801,7 +1824,7 @@ asynchronous write for each message and fsync in their sentinel."
 ;;; Appearance
 ;;;
 
-(defcustom mew-window-configuration 
+(defcustom mew-window-configuration
   '((summary (1  0))
     (message (8 31))
     (draft   (1  0)))
@@ -1818,7 +1841,7 @@ asynchronous write for each message and fsync in their sentinel."
   :type 'sexp)
 
 (defcustom mew-multipart-icon-position 'right
-  "*Position where multipart icons are displayed. 
+  "*Position where multipart icons are displayed.
 If 'left, displayed at the left size of the default toolbar.
 If 'right, displayed at the right size of the default toolbar.
 Otherwise, not displayed."
@@ -1846,7 +1869,7 @@ Otherwise, not displayed."
   :type 'symbol)
 
 (defcustom mew-summary-show-direction 'next
-  "*Direction for SPC at end of message. 
+  "*Direction for SPC at end of message.
 'up 'down 'next(current direction) 'stop.
 Other values are considered as 'stop.
 See also 'mew-summary-mark-direction'."
@@ -1872,8 +1895,8 @@ See also 'mew-summary-show-direction' and 'mew-summary-mark-direction'."
 marked with 'mew-mark-duplicated' when 'mew-summary-mark-duplicated'
 is executed.
 
-Possible value is 'first, 'last, or nil. 
-nil means to mark all duplicated messages. 
+Possible value is 'first, 'last, or nil.
+nil means to mark all duplicated messages.
 Other values are considered as 'first."
   :group 'mew-summary
   :type '(choice (const first) (const last) (const nil)))
@@ -1945,8 +1968,7 @@ to a region."
 
 (defcustom mew-insert-final-newline t
   "*If *non-nil*, and if text/plain does not have final newline,
-insert a newline at the end of buffer. Due to XEmacs bug,
-mew-break-pages cannot work without this."
+insert a newline at the end of buffer. "
   :group 'mew-message
   :type 'boolean)
 
@@ -1987,13 +2009,13 @@ not work resulting that no garbage file remains."
   :type 'boolean)
 
 (defcustom mew-header-alist nil
-  "*Alist of key and value pair for header field to be inserted on draft.
-'((\"X-fingerprint:\" . \"6B 63 38 88 67 5E 96 8E  CE A4 62 73 3F 11 64 94\")
-  (\"X-URL:\" . \"http://www.example.org/\"))"
+  "*List of (key value) for header field to be inserted on draft.
+'((\"X-fingerprint:\" \"6B 63 38 88 67 5E 96 8E  CE A4 62 73 3F 11 64 94\")
+  (\"X-URL:\" \"http://www.example.org/\"))"
   :group 'mew-draft
-  :type 'sexp)
+  :type '(list (list string string)))
 
-(defcustom mew-require-final-newline t 
+(defcustom mew-require-final-newline t
   "*If non-nil, Mew adds a new line to the draft if ended without a new
 line."
   :group 'mew-draft
@@ -2076,7 +2098,7 @@ super-cite, (setq mew-cite-hook 'sc-cite-original)."
   :type 'hook)
 
 (defcustom mew-cite-strings-function 'mew-cite-strings
-  "*Function called from mew-cite-original which to create cite labels 
+  "*Function called from mew-cite-original which to create cite labels
 according to 'mew-cite-format' and 'mew-cite-fields'."
   :group 'mew-cite
   :type 'function)
@@ -2087,7 +2109,7 @@ according to 'mew-cite-format' and 'mew-cite-fields'."
   :type 'hook)
 
 (defcustom mew-cite-prefix-function nil
-  "*Function called on citations. A good candidate is 
+  "*Function called on citations. A good candidate is
 'mew-cite-prefix-username"
   :group 'mew-cite
   :type 'function)
@@ -2099,10 +2121,10 @@ confirmed to be used."
   :type 'boolean)
 
 (defcustom mew-cite-fields (list mew-from: mew-subj: mew-date:)
-  "*The fields that you want to extract as citation label. 
+  "*The fields that you want to extract as citation label.
 If you change this valuable, you must change mew-cite-format.
 The value of the first field becomes the first argument for mew-cite-format.
-(e.g. The default first argument is a value of From: field.)
+\(e.g. The default first argument is a value of From: field.)
 The value of the second field becomes the second argument for mew-cite-format.
 ....
 If this is nil, label is not generated."
@@ -2124,25 +2146,25 @@ If 'truncate, cut over fill-column and insert 'mew-draft-cite-ellipses'."
                  (symbol :tag "Asis" nil)))
 
 (defcustom mew-draft-cite-ellipses " .."
-  "String that is inserted a truncated line when 
+  "String that is inserted a truncated line when
 'mew-draft-cite-fill-mode' is 'truncate."
   :group 'mew-cite
   :type 'string)
 
 (defcustom mew-draft-cite-label-fill-column nil
-  "If *non-nil*, this value is used for 'mew-draft-cite-fill-mode' 
+  "If *non-nil*, this value is used for 'mew-draft-cite-fill-mode'
 instead of 'fill-column'."
   :group 'mew-cite
   :type '(choice integer (const nil)))
 
 (defcustom mew-summary-reply-position 'body
-  "If 'body, the cursor locates in the beginning of the body. 
+  "If 'body, the cursor locates in the beginning of the body.
 Otherwise, the cursor is after To:."
   :group 'mew-cite
   :type 'symbol)
 
 (defcustom mew-summary-reply-with-citation-position 'end
-  "If 'body, the cursor locates in the beginning of the body. 
+  "If 'body, the cursor locates in the beginning of the body.
 If 'end, the cursor locates after the citation.
 Otherwise, the cursor is after To:."
   :group 'mew-cite
@@ -2174,21 +2196,23 @@ Otherwise, the cursor is after To:."
 ;;;
 
 (defcustom mew-charset-input-method-alist nil
-  "*Alist of charset and input-method. When a message is cited into
-Draft mode on X/Emacs 20 or later AND the charset of the message is
-found in this variable, the corresponding input-method is automatically
+  "*Alist of charset and input-method. When a message is cited
+into Draft mode the charset of the message is found in this
+variable, the corresponding input-method is automatically
 selected. An example configuration is as follows:
-(setq mew-charset-input-method-alist
-      '((\"iso-8859-1\" . \"latin-1-postfix\")
-	(\"iso-8859-2\" . \"latin-2-postfix\")))
+
+\(setq mew-charset-input-method-alist
+      '((\"iso-8859-1\" \"latin-1-postfix\")
+	(\"iso-8859-2\" \"latin-2-postfix\")))
 "
   :group 'mew-draft
-  :type '(alist :key-type string :value-type string))
+  :type '(list (list string string)))
 
 (defun mew-charset-to-input-method (charset)
   (if (stringp charset)
-      (cdr (mew-assoc-case-equal
-	    charset mew-charset-input-method-alist 0))))
+      (mew-alist-get-value
+       (mew-assoc-case-equal
+	charset mew-charset-input-method-alist 0))))
 
 (defcustom mew-charset-m17n "utf-8"
   "*A charset to be used if multiple character set are found and
@@ -2219,7 +2243,7 @@ the following step to decide a charset.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Signature
-;;; 
+;;;
 
 (defcustom mew-signature-file "~/.signature"
   "*A signature file to be inserted in Draft mode. To support multiple
@@ -2227,15 +2251,15 @@ signature files, use 'c-sig.el'."
   :group 'mew-draft
   :type 'file)
 
-(defcustom mew-signature-insert-last nil 
-  "*If *non-nil*, the signature file is inserted in the last of body. 
+(defcustom mew-signature-insert-last nil
+  "*If *non-nil*, the signature file is inserted in the last of body.
 Otherwise, it is inserted at the current point. If you created multipart
 and mew-signature-as-lastpart is *non-nil*, this variable is ignored."
   :group 'mew-draft
   :type 'boolean)
 
 (defcustom mew-signature-as-lastpart nil
-  "*If *non-nil*, the signature file is appended as the final part 
+  "*If *non-nil*, the signature file is appended as the final part
 when you create multipart."
   :group 'mew-draft
   :type 'boolean)
@@ -2295,7 +2319,7 @@ in Attachments."
   "*Completion field list in Draft mode."
   :group 'mew-complete
   :type '(repeat string))
-  
+
 (defcustom mew-complete-case-ignore-case nil
   "*Non-nil means Mew does not consider case is significant in case
 completion."
@@ -2352,7 +2376,7 @@ circular domain completion."
 	    "Precedence:" mew-organization: "X-"
 	    "Lines:" "Status:" "Posted:" "Forwarded:" "Replied:" "X-UIDL:"
 	    "DomainKey-Signature:" "DKIM-Signature:"
-	    "Authentication-Results:"))
+	    "Authentication-Results:" "List-.*:" "Errors-To:" "Sender:"))
   "*A field list to be deleted when edit again."
   :group 'mew-draft
   :type '(choice (const nil)
@@ -2401,14 +2425,14 @@ circular domain completion."
   "*A field list to print"
   :group 'mew-summary
   :type '(repeat :tag "Field list" (string :tag "Field")))
-  
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Directory stuff
 ;;;
 
 (defcustom mew-summary-trace-directory t
-  "*If non-nil, change to the folder directory when 
+  "*If non-nil, change to the folder directory when
 mew-summary-goto-folder."
   :group 'mew-summary
   :type 'boolean)
@@ -2420,7 +2444,7 @@ See also 'mew-copy-dir'."
   :type 'directory)
 
 (defcustom mew-summary-preserve-dir nil
-  "*If non-nil, the previous directory is used as the default 
+  "*If non-nil, the previous directory is used as the default
 directory for save, etc. See also 'mew-draft-preserve-dir'."
   :group 'mew-summary
   :type 'boolean)
@@ -2430,13 +2454,13 @@ directory for save, etc. See also 'mew-draft-preserve-dir'."
 See also 'mew-save-dir'."
   :group 'mew-draft
   :type 'directory)
-  
+
 (defcustom mew-draft-preserve-dir nil
-  "*If non-nil, the previous directory is used as the default 
+  "*If non-nil, the previous directory is used as the default
 directory for copy, etc. See also 'mew-summary-preserve-dir'."
   :group 'mew-draft
   :type 'boolean)
-  
+
 (defcustom mew-file-append-p nil
   "*If *non-nil*, a message or a part is appended to the existing file
 on '\\<mew-summary-mode-map>\\[mew-summary-save]'. Otherwise overwrote."
@@ -2451,7 +2475,7 @@ on '\\<mew-summary-mode-map>\\[mew-summary-save]'. Otherwise overwrote."
 (defcustom mew-regex-folder-candidate "^[^.#0-9]\\|^[0-9].*[^0-9]"
   "*Regular expression used in 'mew-dir-list-with-link-count'
 and 'mew-dir-list-without-link-count' to list up folder
-candidates. The default value \"^[^.#0-9]\\\\|^[0-9].*[^0-9]\" lists up 
+candidates. The default value \"^[^.#0-9]\\\\|^[0-9].*[^0-9]\" lists up
 folders whose name is alphabet only."
   :group 'mew-env
   :type 'regexp)
@@ -2462,24 +2486,21 @@ folders whose name is alphabet only."
 ;;;
 
 (defconst mew-custom-type-of-guess-alist
-  '(choice (const nil)
-           (repeat
-             (cons string
-                   (repeat (cons regexp string))))))
+  '(list (list string (list string string))))
 
 (defcustom mew-refile-guess-alist nil
   "*If non-nil, mew guesses destination folder by using this hint.
 The format is like this:
 
     (setq mew-refile-guess-alist
-          '((\"To:\" 
-              (\"wide@wide\" . \"+wide/wide\")
-              (\"adam\"      . \"+labo/adam\"))
+          '((\"To:\"
+              (\"wide@wide\" \"+wide/wide\")
+              (\"adam\"      \"+labo/adam\"))
             (\"Newsgroups:\"
-              (\"^nifty\\\\.\\\\([^ ]+\\\\)\" . \"+Nifty/\\\\1\"))
-            (\"From:\" 
-              (\"uucp\" . \"+adm/uucp\")
-              (\".*\"   . \"+misc\"))
+              (\"^nifty\\\\.\\\\([^ ]+\\\\)\" \"+Nifty/\\\\1\"))
+            (\"From:\"
+              (\"uucp\" \"+adm/uucp\")
+              (\".*\"   \"+misc\"))
             ))
 "
   :group 'mew-refile
@@ -2590,7 +2611,7 @@ setup phase Mew make a secure directory from this variable and set
 mew-temp-file a file name prefix contained the directory name. The
 directory must be unreadable from others, otherwise it might become a
 big security hole. And this directory must not be gained access
-through network to prevent tire-tapping. Mew never uses 
+through network to prevent tire-tapping. Mew never uses
 'call-process-region' rather does use 'call-process' creating a
 temporary file with mew-temp-file by itself. If 'call-process-region'
 is used, Emacs creates a temporary file (probably in /tmp). So bad
@@ -2609,7 +2630,7 @@ guys can wiretap the temporary file."
 ;;;
 
 (defcustom mew-auto-get t
-  "If *non-nil*, Mew gets messages from your mailbox automatically. 
+  "If *non-nil*, Mew gets messages from your mailbox automatically.
 '\\[universal-argument] \\[mew]' equals to '\\[mew]' with 'mew-auto-get'
 revered."
   :group 'mew-summary
@@ -2640,24 +2661,24 @@ The first member is displayed as a default value."
   :type '(choice (const nil) (repeat string)))
 
 (defcustom mew-sort-default-key "date"
-  "*Default sort key when inputting a sort key. 
+  "*Default sort key when inputting a sort key.
 Its format is key:mode where more is found in 'mew-sort-mode'."
   :group 'mew-summary
   :type 'string)
 
 (defcustom mew-sort-default-key-alist nil
-  "*Alist of (folder . sort-key) to decide a default sort-key
+  "*List of (folder sort-key) to decide a default sort-key
 of a specific folder. An example is follows:
-(setq mew-sort-default-key-alist
-      '((\"+tmp/beginners\" . \"x-sequence\")
-        (\"+tmp/elips\" . \"x-mail-count\")))"
+\(setq mew-sort-default-key-alist
+      '((\"+tmp/beginners\" \"x-sequence\")
+        (\"+tmp/elips\" \"x-mail-count\")))"
   :group 'mew-summary
-  :type 'sexp)
+  :type '(list (list string string)))
 
 (defcustom mew-sort-key-alist
-  '(("date" . "date") ("subject") ("from") ("to") ("newsgroups")
-    ("posted" . "date") ("x-sequence" . "postnum") ("x-mail-count" . "num")
-    ("x-ml-count" . "num"))
+  '(("date" "date") ("subject") ("from") ("to") ("newsgroups")
+    ("posted" "date") ("x-sequence" "postnum")
+    ("x-mail-count" "num") ("x-ml-count" "num"))
   "*List of fields for 'mew-summary-sort'.
 Each element is (FIELD-NAME) or (FIELD-NAME . MODE).
 MODE is one of \"date\" (by chronological order) or
@@ -2666,7 +2687,7 @@ MODE is one of \"date\" (by chronological order) or
                \"text\" (by alphabetical order) or
                \"ml\"   (by alphabetical order with ml prefix removed) or
                \"mlnum\" (by numerical order of ml number).
-(nil means \"text\")."
+\(nil means \"text\")."
   :group 'mew-summary
   :type 'sexp)
 
@@ -2813,7 +2834,7 @@ will be appended (mew-mail-domain) when composing."
 
 (defcustom mew-addrbook-override-by-newone t
   "If non-nil, the 'user' entry in 'mew-alias-auto-alist'
-is override by a new entry of (user different-address). 
+is override by a new entry of (user different-address).
 This means that addresses in To: and Cc: in Draft mode are
 always learned with an exception 'user' is defined in Addrbook.
 If nil,  the old 'user' entry remains."
@@ -2871,6 +2892,12 @@ list. the address is not registered to Addrbook."
 ;;; Privacy
 ;;;
 
+(defcustom mew-draft-privacy-method 'pgp
+  "Privacy method to create messages in Draft mode.
+You should choose 'pgp or 'smime."
+  :group 'mew-privacy
+  :type '(choice (const pgp) (const smime)))
+
 (defcustom mew-protect-privacy-always nil
   "*If non-nil, a draft is to be protected according to
 'mew-protect-privacy-always-type'."
@@ -2895,15 +2922,15 @@ appropriate for this value."
                  (other :tag "nil" nil)))
 
 (defcustom mew-protect-privacy-encrypted nil
-  "*If non-nil, a draft replying a encrypted message is to be protected 
+  "*If non-nil, a draft replying a encrypted message is to be protected
 according to 'mew-protect-privacy-encrypted-type'."
   :group 'mew-privacy
   :type 'boolean)
 
 (defcustom mew-protect-privacy-encrypted-type 'pgp-encryption
   "*A type of privacy protection for drafts replying encrypted
-messages. Currently, 'pgp-signature, 'pgp-encryption, 
-'pgp-signature-encryption, 'pgp-encryption-signature, 
+messages. Currently, 'pgp-signature, 'pgp-encryption,
+'pgp-signature-encryption, 'pgp-encryption-signature,
 and nil are available."
   :group 'mew-privacy
   :type '(choice (const pgp-signature)
@@ -2917,7 +2944,7 @@ and nil are available."
                  (other :tag "nil" nil)))
 
 (defcustom mew-protect-privacy-with-old-pgp-signature nil
-  "*If non-nil and 'mew-use-old-pgp' is non-nil, 
+  "*If non-nil and 'mew-use-old-pgp' is non-nil,
 the old fashioned PGP message are used when composing."
   :group 'mew-privacy
   :type 'boolean)
@@ -2944,6 +2971,16 @@ your public key as well as receivers' one."
   :group 'mew-privacy
   :type 'boolean)
 
+(defcustom mew-pgp-signer nil
+  "*ID of a PGP key to use for sign"
+  :group 'mew-privacy
+  :type 'string)
+
+(defcustom mew-smime-signer nil
+  "*ID of a S/MIME key to use for sign"
+  :group 'mew-privacy
+  :type 'string)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; PGP Public key fetch
@@ -2953,8 +2990,8 @@ your public key as well as receivers' one."
   '("x-pgp-key:" "x-pgp-key-url:" "x-pgp-public-key:" "x-pgp-public-key-url:"
     "x-pgp5-key:" "x-pgp5-key-url:" "x-pgp5-public-key:"
     "x-pgp5-public-key-url:" "x-public-key:"))
-   
-(defvar mew-pgp-pks-port "11371")
+
+(defvar mew-pgp-pks-port 11371)
 (defvar mew-pgp-pks-lang "/pks/lookup?op=get&search=")
 
 ;; Tue Dec 16 16:28:11 JST 2003
@@ -3022,18 +3059,6 @@ your public key as well as receivers' one."
 
 (defvar mew-prog-grep-max-msgs 10000)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; External-body
-;;;
-
-(defcustom mew-ext-anon-ftp-method 'ftp
-  "A method to get the message body for access-type=anon-ftp.
-If 'ftp is specified, ange-ftp or efs is used. If 'http is specified,
-w3 is used."
-  :group 'mew-message
-  :type '(choice (const ftp) (const http)))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; X face
@@ -3097,16 +3122,16 @@ w3 is used."
 (defcustom mew-decode-broken t
   "If non-nil, Mew decode broken messages.
 
-(1) Decode quoted encoded-words, which violates RFC 2047.
+\(1) Decode quoted encoded-words, which violates RFC 2047.
 
 Good example:
-	From: Kazu Yamamoto (=?ISO-2022-JP?B?GyRCOzNLXE9CSScbKEI=?=) 
+	From: Kazu Yamamoto (=?ISO-2022-JP?B?GyRCOzNLXE9CSScbKEI=?=)
 		<kazu@example.org>
 Bad example:
-	From: Kazu Yamamoto (\"=?ISO-2022-JP?B?GyRCOzNLXE9CSScbKEI=?=\") 
+	From: Kazu Yamamoto (\"=?ISO-2022-JP?B?GyRCOzNLXE9CSScbKEI=?=\")
 		<kazu@example.org>
 
-(2) Decode raw non-ASCII text, which violates RFC 822.
+\(2) Decode raw non-ASCII text, which violates RFC 822.
 This is typically found in Subject: field.
 
 Good example:
@@ -3114,9 +3139,9 @@ Good example:
 Bad example:
 	Subject: Raw non-ASCII text.
 
-(3) Decode invalid MIME parameters, which violates RFC 2231.
-(i)  Raw text.
-(ii) Encorded-word defined in RFC 2047.
+\(3) Decode invalid MIME parameters, which violates RFC 2231.
+\(i)  Raw text.
+\(ii) Encorded-word defined in RFC 2047.
 
 Good example:
 	Content-Disposition: attachment;
@@ -3126,7 +3151,7 @@ Bad example
 	Content-Disposition: attachment;
 	 filename=\"=?iso-2022-jp?B?GyRCJEYkOSRIGyhC?=\"
 
-(4) Decode invalid non-ASCII MIME body, whose charset is US-ASCII or
+=(4) Decode invalid non-ASCII MIME body, whose charset is US-ASCII or
 not present.
 
 Good example:
@@ -3139,7 +3164,7 @@ Bad example:
 
 	ISO-2022-JP text
 
-(5) Decode invalid non-ASCII text, whose charset is UTF-8 or not present.
+\(5) Decode invalid non-ASCII text, whose charset is UTF-8 or not present.
 
 Good example:
 	Content-Type: text/plain; charset=ISO-8859-15
@@ -3203,13 +3228,13 @@ Level 2: syntax error."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2007 Mew developing team.
+;; Copyright (C) 1996-2008 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -3218,7 +3243,7 @@ Level 2: syntax error."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
